Package: orthoMTL
Type: Package
Title: Multi-Task Learning with Orthogonal Constraints
Version: 0.1.0
Authors@R: c(
    person("Kevin", "Vervier",
        email = "kevin.vervier@novartis.com",
        role = c("aut", "cre")),
    person("Novartis Pharma AG", role = c("cph", "fnd")))
Description: Fits regularised multi-task learning models where relationships
    between tasks are controlled via orthogonality or disjoint-support constraints.
    Supports regression, binary classification, and censored survival data.
    In survival mode, time-to-event outcomes are converted into binary labels at
    user-defined thresholds, enabling the discovery of features with time-varying
    effects that standard proportional-hazards models cannot detect.
    Implements the penalty described in Vervier et al. (2014)
    <https://hal.science/hal-00985654>.
License: GPL-3
Encoding: UTF-8
Imports: parallel, doParallel, foreach, ggplot2, rlang, stats
VignetteBuilder: knitr
Suggests: knitr, glmnet, testthat (>= 3.0.0), survival, rmarkdown
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
NeedsCompilation: no
Packaged: 2026-08-07 13:45:33 UTC; Shadow
Author: Kevin Vervier [aut, cre],
  Novartis Pharma AG [cph, fnd]
Maintainer: Kevin Vervier <kevin.vervier@novartis.com>
Repository: CRAN
Date/Publication: 2026-08-23 10:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(coef,orthoMTL)
S3method(predict,orthoMTL)
S3method(print,bootstrap_orthoMTL)
S3method(print,cv_orthoMTL)
S3method(print,orthoMTL)
S3method(print,simulated_mtl)
S3method(summary,orthoMTL)
export(accuracy_mtl)
export(auc_mtl)
export(bootstrap_orthoMTL)
export(cindex_mtl)
export(create_constraint_matrix)
export(create_indicator_matrix)
export(create_longitudinal_labels)
export(cv_orthoMTL)
export(nnmaxheap_C)
export(orthoMTL)
export(plot_bootstrap)
export(plot_correlation)
export(plot_heatmap)
export(plot_prediction)
export(r2_mtl)
export(rmse_mtl)
export(simulate_mtl)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(rlang,.data)
importFrom(stats,coef)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,runif)

# orthoMTL

> Multi-task learning with orthogonal constraints for time-varying survival analysis

<!-- badges: start -->
[![R-CMD-check](https://github.com/Novartis/orthoMTL/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Novartis/orthoMTL/actions/workflows/R-CMD-check.yaml)
[![License: GPL-3](https://img.shields.io/badge/License-GPL%203-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

## Overview

Standard survival models such as Cox proportional hazards assume that each
feature's effect is constant over time. **orthoMTL** relaxes this assumption
by recasting survival analysis as a **multi-task learning** problem: binary
classification tasks are defined at multiple time thresholds, and a shared
coefficient matrix is estimated with an orthogonality penalty that encourages
temporally distant tasks to use different features.

The method is based on:

> Vervier, K., Mahé, P., d'Aspremont, A., Veyrieras, J.-B., & Vert, J.-P.
> (2014). On Learning Matrices with Orthogonal Columns or Disjoint Supports.
> *ECML-PKDD 2014*.
> <https://hal.science/hal-00985654>

The survival extension is described in:

> Juric, D. et al. (2026). Genomic Determinants of Response to Alpelisib
> Plus Fulvestrant in the SOLAR-1 Trial. *Annals of Oncology*.
> <https://doi.org/10.1016/j.annonc.2026.04.003>

## Installation

### From CRAN (once released)

```r
install.packages("orthoMTL")
```

### Development version from GitHub

```r
# install.packages("pak")
pak::pak("Novartis/orthoMTL")
```

Or using `remotes`:

```r
# install.packages("remotes")
remotes::install_github("Novartis/orthoMTL")
```

## Quick start

```r
library(orthoMTL)

# Simulate data with known time-varying effects
sim <- simulate_mtl(n = 200, p = 20, n_signals = 5, seed = 42)

# Convert survival times to binary labels at each threshold
Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, sim$thresholds)
W <- create_indicator_matrix(Y)   # censoring mask
K <- create_constraint_matrix(length(sim$thresholds))

# Fit the model
fit <- orthoMTL(sim$X, Y, lambda = 1e-3, K = K,
                survival = TRUE, censored.mat = W)
summary(fit)

# Visualise coefficients across time thresholds
plot_heatmap(fit)
```

### Regression and classification

The same solver handles plain multi-task regression and classification —
survival is just one of three modes. Select the mode on the solver with the
`logistic` / `survival` flags, score with the matching metric, and (for CV)
let `cv_orthoMTL()` pick the metric automatically.

```r
library(orthoMTL)

# --- Multi-task regression ---
sim <- simulate_mtl(n = 200, p = 20, n_signals = 5,
                    mode = "regression", seed = 42)
fit <- orthoMTL(sim$X, sim$Y, lambda = 1e-3)      # logistic = survival = FALSE
pred <- predict(fit, newdata = sim$X)             # type = "link"
rmse_mtl(sim$Y, pred)
r2_mtl(sim$Y, pred)

# --- Multi-task classification (labels in {-1, +1}) ---
sim <- simulate_mtl(n = 200, p = 20, n_signals = 5,
                    mode = "classification", seed = 42)
fit <- orthoMTL(sim$X, sim$Y, lambda = 1e-3, logistic = TRUE)
prob <- predict(fit, newdata = sim$X, type = "response")   # P(Y = +1)
cls  <- predict(fit, newdata = sim$X, type = "class")      # {-1, +1}
auc_mtl(sim$Y, predict(fit, newdata = sim$X))              # AUC on link scores
accuracy_mtl(sim$Y, predict(fit, newdata = sim$X))

# CV picks the metric by mode (cindex / auc / rmse) unless overridden
cv <- cv_orthoMTL(sim$X, sim$Y, logistic = TRUE, survival = FALSE,
                  lambdas = c(1e-3, 1e-2), n_cores = 1)
cv$best
```

For a full walkthrough — including cross-validation, bootstrap inference, and
comparison with Cox — see the package vignette:

```r
vignette("introduction", package = "orthoMTL")
```

## Key functions

| Function | Purpose |
|---|---|
| `orthoMTL()` | Core solver (regression / classification / survival) |
| `cv_orthoMTL()` | Parallel hyperparameter grid search |
| `bootstrap_orthoMTL()` | Bootstrap coefficient stability vs null |
| `simulate_mtl()` | Simulate data with time-varying ground truth |
| `create_longitudinal_labels()` | Survival → binary label matrix |
| `create_indicator_matrix()` | Censoring indicator matrix |
| `create_constraint_matrix()` | Diffusion constraint matrix for tasks |
| `predict()`, `coef()` | Predictions (`type = "link"/"response"/"class"`) and coefficients |
| `cindex_mtl()` | Survival concordance index |
| `rmse_mtl()`, `r2_mtl()` | Regression metrics |
| `accuracy_mtl()`, `auc_mtl()` | Classification metrics |
| `plot_heatmap()`, `plot_correlation()`, `plot_bootstrap()` | Visualisation |

## Citation

If you use orthoMTL in your work, please cite:

```bibtex
@inproceedings{vervier2014,
  author    = {Vervier, Kevin and Mah{\'e}, Pierre and d'Aspremont,
               Alexandre and Veyrieras, Jean-Baptiste and Vert, Jean-Philippe},
  title     = {On Learning Matrices with Orthogonal Columns or Disjoint Supports},
  booktitle = {Machine Learning and Knowledge Discovery in Databases (ECML-PKDD 2014)},
  year      = {2014},
  publisher = {Springer},
  url       = {https://hal.science/hal-00985654}
}
```

## License

GPL-3.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  fig.align = "center"
)
library(orthoMTL)

## ----simulate-----------------------------------------------------------------
set.seed(42)

sim <- simulate_mtl(
  n = 300,
  p = 15,
  n_signals = 5,
  thresholds = c(4, 8, 14, 20),
  effect_strength = 1.2
)
sim

## ----ground-truth-------------------------------------------------------------
# True coefficients for signal features
gt <- sim$ground_truth
signal_coefs <- gt$coefficients[gt$signal_features, ]
signal_coefs

## ----ground-truth-types-------------------------------------------------------
# Effect types
gt$effect_types

## ----prepare-data-------------------------------------------------------------
thresholds <- sim$thresholds
Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, thresholds)
head(Y, 10)

## ----indicator----------------------------------------------------------------
W <- create_indicator_matrix(Y)
cat("Proportion observed per threshold:\n")
colMeans(W)

## ----constraint---------------------------------------------------------------
K <- create_constraint_matrix(length(thresholds))
K

## ----fit-initial--------------------------------------------------------------
fit <- orthoMTL(
  X = sim$X, Y = Y,
  lambda = 1e-3, step_size = 0.5,
  K = K, survival = TRUE, censored.mat = W,
  alpha = 0.5
)
summary(fit)

## ----heatmap-initial, fig.width=8, fig.height=6-------------------------------
plot_heatmap(fit)

## ----cv, results='hide'-------------------------------------------------------
n_tasks <- length(sim[["thresholds"]])
folds <- rep(1:5, length.out = nrow(sim[["X"]]))

cv_res <- cv_orthoMTL(
  X.train   = sim[["X"]],
  Y.train   = Y,
  W.train   = W,
  K         = K,
  lambdas   = c(1e-5, 1e-4, 1e-3),
  alphas    = c(0, 0.5, 1),
  stepsizes = c(1, 2, 5),
  diag_vals = c(0.5, n_tasks, 2 * n_tasks),
  survival  = TRUE,
  folds     = folds,
  n_cores   = 1,
  seed      = 42,
  verbose   = FALSE
)

## ----cv-results---------------------------------------------------------------
print(cv_res)

## ----fit-final----------------------------------------------------------------
best <- cv_res$best
K_final <- K
diag(K_final) <- best$diag_val

fit_final <- orthoMTL(
  X = sim$X, Y = Y,
  lambda    = best$lambda,
  alpha     = best$alpha,
  step_size = best$stepsize,
  K         = K_final,
  survival  = TRUE,
  censored.mat = W
)
summary(fit_final)

## ----heatmap-final, fig.width=8, fig.height=6---------------------------------
plot_heatmap(fit_final)

## ----correlation, fig.width=6, fig.height=5-----------------------------------
plot_correlation(fit_final)

## ----bootstrap, results='hide'------------------------------------------------
boot_res <- bootstrap_orthoMTL(
  X = sim$X, Y = Y,
  lambda    = best$lambda,
  alpha     = best$alpha,
  step_size = best$stepsize,
  K         = K_final,
  survival  = TRUE,
  censored.mat = W,
  n_repeats = 200,
  n_cores   = 1,
  verbose   = FALSE
)

## ----bootstrap-print----------------------------------------------------------
print(boot_res)

## ----bootstrap-select---------------------------------------------------------
signal_feats <- gt[["signal_features"]]
null_feats <- gt[["null_features"]]
effect_types <- gt[["effect_types"]]

selected <- c(
  signal_feats[effect_types == "switch"][1],
  signal_feats[effect_types == "constant"][1],
  signal_feats[effect_types == "early"][1],
  null_feats[1]
)

cat("Selected features:\n")
cat("  Switch signal:  ", selected[1], "(Cox should miss)\n")
cat("  Constant signal:", selected[2], "(Cox should find)\n")
cat("  Early signal:   ", selected[3], "(Cox may dilute)\n")
cat("  Null feature:   ", selected[4], "(neither should find)\n")

## ----bootstrap-plot, fig.width=8, fig.height=6--------------------------------
suppressWarnings(
  plot_bootstrap(boot_res, features = selected)
)

## ----cox-comparison, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)----
library(survival)
library(glmnet)

surv_obj <- Surv(time = sim[["SurvTime"]], event = sim[["Event"]])

# Cross-validate alpha (mixing parameter)
alphas <- seq(0, 1, by = 0.1)
cv_scores <- numeric(length(alphas))

for (i in seq_along(alphas)) {
  set.seed(42)
  cvfit <- cv.glmnet(
    x = sim[["X"]],
    y = surv_obj,
    family = "cox",
    type.measure = "C",
    alpha = alphas[i]
  )
  cv_scores[i] <- cvfit[["cvm"]][cvfit[["index"]]["min", ]]
}

best_alpha <- alphas[which.max(cv_scores)]
cat("Best alpha:", best_alpha, "(CV C-index:", max(cv_scores), ")\n")

# Refit with best alpha
set.seed(42)
cox_fit <- cv.glmnet(
  x = sim[["X"]],
  y = surv_obj,
  family = "cox",
  type.measure = "C",
  alpha = best_alpha
)

cox_coefs <- as.numeric(coef(cox_fit, s = "lambda.min"))
names(cox_coefs) <- colnames(sim[["X"]])

## ----cox-table, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)----
# Build comparison table
ortho_mean_abs <- apply(abs(coef(fit_final)), 1, mean)

comparison <- data.frame(
  feature = gt$signal_features,
  effect_type = as.character(gt$effect_types),
  orthoMTL_mean_abs = round(ortho_mean_abs[gt$signal_features], 4),
  cox_coef = round(cox_coefs[gt$signal_features], 4),
  cox_detected = cox_coefs[gt$signal_features] != 0,
  stringsAsFactors = FALSE
)

cat("Signal feature detection comparison:\n\n")
print(comparison, row.names = FALSE)

## ----cox-vs-ortho, fig.width=8, fig.height=4, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)----
# Pick the switch feature — most interesting temporal pattern
switch_feat <- signal_feats[effect_types == "switch"][1]

# What Cox sees: one number
cat(switch_feat, "— Cox coefficient:", 
    round(cox_coefs[switch_feat], 3), "\n")

# What orthoMTL sees: a trajectory
cat(switch_feat, "— orthoMTL coefficients:\n")
print(round(coef(fit_final)[switch_feat, ], 3))

# What the truth is
cat(switch_feat, "— True coefficients (sign-aligned):\n")
print(round(-gt[["coefficients"]][switch_feat, ], 3))

## ----ground-truth-recovery----------------------------------------------------
true_coefs <- gt$coefficients[gt$signal_features, ]
est_coefs <- coef(fit_final)[gt$signal_features, ]

# Sign convention:
#   Simulation uses log-hazard scale (negative = protective, reduces hazard)
#   orthoMTL models P(progression-free) (positive = protective)
# Negate true coefficients to align
true_aligned <- -true_coefs

task_cors <- sapply(seq_len(ncol(true_aligned)), function(k) {
  cor(true_aligned[, k], est_coefs[, k])
})
names(task_cors) <- colnames(true_aligned)

cat("Correlation between true and estimated coefficients per threshold:\n")
print(round(task_cors, 3))

## ----recovery-heatmap, fig.width=8, fig.height=5------------------------------
make_long <- function(mat, source_label) {
  data.frame(
    feature = rep(rownames(mat), ncol(mat)),
    threshold = rep(colnames(mat), each = nrow(mat)),
    weight = as.vector(mat),
    source = source_label,
    stringsAsFactors = FALSE
  )
}

# Normalize each matrix to [-1, 1] by dividing by its own max absolute value
normalize <- function(mat) mat / max(abs(mat), na.rm = TRUE)

combined <- rbind(
  make_long(normalize(true_aligned), "True (sign-aligned)"),
  make_long(normalize(est_coefs),    "Estimated")
)

combined$feature <- factor(combined$feature, levels = rev(rownames(true_aligned)))
combined$threshold <- factor(combined$threshold, levels = colnames(true_aligned))
combined$source <- factor(combined$source, levels = c("True (sign-aligned)", "Estimated"))



# Now both panels use the full color range
ggplot2::ggplot(combined, ggplot2::aes(threshold, feature, fill = weight)) +
  ggplot2::geom_tile() +
  ggplot2::scale_fill_gradient2(low = "red", mid = "white", high = "blue", midpoint = 0,
                       limits = c(-1, 1)) +
  ggplot2::facet_wrap(~ source) +
  ggplot2::labs(x = "Threshold (months)", y = NULL, fill = "Normalized\nCoefficient") +
  ggplot2::theme_minimal()

#' Print Bootstrap Inference Results
#'
#' Displays a summary of bootstrap inference results from
#' \code{\link{bootstrap_orthoMTL}}.
#'
#' @param x An object of class \code{"bootstrap_orthoMTL"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @method print bootstrap_orthoMTL
#' @export
#'
#' @examples
#' # See ?bootstrap_orthoMTL for a full example
print.bootstrap_orthoMTL <- function(x, ...) {

  cat("Bootstrap inference for orthoMTL\n")
  cat("Repeats:", x$n_repeats, "(real) +",
      x$n_repeats, "(null permutation)\n")
  cat("Features:", x$n_features, "| Tasks:", x$n_tasks, "\n")

  cat("---\n")

  cat("Real models -- objective:\n")
  cat("  mean =", format(mean(x$obj_real), digits = 4),
      " range = [", format(min(x$obj_real), digits = 4), ",",
      format(max(x$obj_real), digits = 4), "]\n")

  cat("Null models -- objective:\n")
  cat("  mean =", format(mean(x$obj_null), digits = 4),
      " range = [", format(min(x$obj_null), digits = 4), ",",
      format(max(x$obj_null), digits = 4), "]\n")

  invisible(x)
}

#' Extract Coefficients from an orthoMTL Model
#'
#' Returns the coefficient matrix from a fitted \code{orthoMTL} model.
#'
#' @param object A fitted model object of class \code{"orthoMTL"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric matrix of regression coefficients with dimensions
#'   \code{n_features x n_tasks}. Row names correspond to feature names
#'   and column names to task names, if available.
#'
#' @method coef orthoMTL
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)
#' coef(fit)
coef.orthoMTL <- function(object, ...) {
  return(object$B)
}

#' Project a Vector onto Non-Negative Non-Increasing Space
#'
#' Applies an isotonic regression-style projection to enforce that the
#' output vector is non-negative and non-increasing. Used internally
#' by \code{\link{predict.orthoMTL}} to ensure survival predictions
#' are monotonically decreasing across time thresholds.
#'
#' @param m A numeric vector to project.
#'
#' @return A numeric vector of the same length as \code{m}, projected
#'   onto the non-negative non-increasing constraint space.
#'
#' @details
#' This function implements a pool-adjacent-violators style algorithm.
#' It replaces the external \code{Iso} package dependency used in the
#' predecessor \code{orthopen} package.
#'
#' @export
#'
#' @examples
#' # Project a vector onto the non-negative non-increasing space
#' nnmaxheap_C(c(3, 1, 2, -1))
#'
#' # Already valid input: returned unchanged
#' nnmaxheap_C(c(5, 3, 3, 1))
nnmaxheap_C <- function(m) {
  n <- length(m)

  if (n < 1) {
    stop("n=", n, " should be an integer over 1!")
  }

  # handle length-1 case
  if (n == 1) {
    return(max(m, 0))   # non-negative projection of single value
  }

  x <- rep(0, n)
  location <- rep(0, n)

  i <- n
  x[i] <- m[i]
  location[i] <- i

  # Process remaining elements bottom-up
  for (i in (n - 1):1) {
    if (m[i] > x[i + 1]) {
      x[i] <- m[i]
      location[i] <- i
    } else {
      # Merge with the first group
      num <- location[i + 1] - i
      x[i] <- (m[i] + x[i + 1] * num) / (num + 1)
      location[i] <- location[i + 1]
      j <- location[i + 1] + 1
      while (TRUE) {
        if (j > n) break
        if (x[i] <= x[j]) {
          num <- location[j] - j + 1
          x[i] <- (x[i] * (j - i) + x[j] * num) / (location[j] - i + 1)
          location[i] <- location[j]
          j <- location[j] + 1
        } else {
          break
        }
      }
    }
  }

  # Compute the solution using mean and location
  i <- 1
  while (i < n) {
    if (x[i] > 0) {
      for (j in (i + 1):location[i]) {
        x[j] <- x[i]
      }
      i <- location[i] + 1
    } else {
      for (j in i:n) {
        x[j] <- 0
      }
      break
    }
  }

  return(x)
}

# Tests for cindex_mtl and nnmaxheap_C

test_that("cindex_mtl returns value between 0 and 1", {
  set.seed(42)
  n <- 20; n_tasks <- 3
  true_mat <- matrix(sample(c(0, 1), n * n_tasks, replace = TRUE), n, n_tasks)
  pred_mat <- matrix(rnorm(n * n_tasks), n, n_tasks)

  cindex <- cindex_mtl(true_mat, pred_mat)

  expect_true(is.numeric(cindex))
  expect_true(length(cindex) == 1)
  expect_true(cindex >= 0 && cindex <= 1)
})

test_that("cindex_mtl handles perfect concordance", {
  # Construct a case where predictions perfectly match ordering
  true_mat <- matrix(c(
    0, 0, 0,  # worst survival
    1, 0, 0,
    1, 1, 0,
    1, 1, 1   # best survival
  ), nrow = 4, byrow = TRUE)

  # Predictions that perfectly match the ordering (higher = better)
  pred_mat <- matrix(c(
    0.1, 0.1, 0.1,
    0.5, 0.3, 0.2,
    0.8, 0.6, 0.3,
    0.9, 0.9, 0.9
  ), nrow = 4, byrow = TRUE)

  cindex <- cindex_mtl(true_mat, pred_mat)

  expect_true(cindex > 0.8)  # should be high, may not be exactly 1 due to implementation
})

test_that("nnmaxheap_C enforces non-increasing output", {
  # Input that violates monotonicity
  m <- c(0.5, 0.8, 0.3, 0.1)
  result <- nnmaxheap_C(m)

  diffs <- diff(result)
  expect_true(all(diffs <= .Machine$double.eps * 100))
})

test_that("nnmaxheap_C enforces non-negative output", {
  m <- c(0.5, -0.3, -0.8, -1.0)
  result <- nnmaxheap_C(m)

  expect_true(all(result >= 0))
})

test_that("nnmaxheap_C output is non-negative and non-increasing", {
  m <- c(1.0, 0.8, 0.5, 0.2)
  result <- nnmaxheap_C(m)

  expect_true(all(result >= 0))
  diffs <- diff(result)
  expect_true(all(diffs <= .Machine$double.eps * 100))
})

test_that("nnmaxheap_C handles length 1", {
  expect_equal(nnmaxheap_C(0.5), 0.5)
  expect_equal(nnmaxheap_C(-0.5), 0)
})

test_that("nnmaxheap_C errors on empty input", {
  expect_error(nnmaxheap_C(numeric(0)), "should be an integer over 1")
})

#' Predict from an orthoMTL Model
#'
#' Generate predictions from a fitted \code{orthoMTL} model for new
#' observations. In survival mode, predictions are projected onto the
#' non-negative non-increasing space to ensure monotonicity across tasks.
#'
#' @param object A fitted model object of class \code{"orthoMTL"}.
#' @param newdata A numeric matrix of new observations with dimensions
#'   \code{n_new x p}. Column names are used for feature alignment if
#'   available in the fitted object.
#' @param type Character; scale of the returned predictions. One of:
#'   \describe{
#'     \item{\code{"link"}}{(default) the raw linear predictor
#'       \eqn{X B} (after the survival monotonicity projection, if
#'       applicable). Preserves the historical behaviour.}
#'     \item{\code{"response"}}{for \code{logistic} fits, the sigmoid
#'       \eqn{1 / (1 + e^{-XB})} giving \eqn{P(Y = +1)}; for non-logistic
#'       fits identical to \code{"link"}.}
#'     \item{\code{"class"}}{for \code{logistic} fits, the predicted
#'       class label in \eqn{\{-1, +1\}} (\code{sign(XB)}, with 0 mapped
#'       to \code{+1}). Errors for non-logistic fits.}
#'   }
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric matrix of predictions with dimensions
#'   \code{n_new x n_tasks}. Column names correspond to task names
#'   if available.
#'
#' @details
#' If the fitted object contains \code{feature_names} (i.e., the training
#' matrix \code{X} had column names), \code{newdata} columns are aligned
#' to match. Missing features cause an error. Extra features trigger a
#' warning and are dropped.
#'
#' In survival mode (\code{object$hyperparameters$survival == TRUE}),
#' each row of the raw prediction matrix is projected via
#' \code{nnmaxheap_C()} to enforce non-negative, non-increasing values
#' across tasks (time thresholds).
#'
#' @references
#' Vervier, K., Mahe, P., d'Aspremont, A., Veyrieras, J.-B., and
#' Vert, J.-P. (2014). On Learning Matrices with Orthogonal Columns
#' or Disjoint Supports. \emph{ECML-PKDD 2014}.
#' \url{https://hal.science/hal-00985654}
#'
#' @method predict orthoMTL
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' W_true <- qr.Q(qr(matrix(rnorm(p * n_tasks), p, n_tasks)))
#' Y <- X %*% W_true + matrix(rnorm(n * n_tasks), n) * 0.1
#' K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K, disjoint = FALSE)
#'
#' X_new <- matrix(rnorm(20 * p), 20, p)
#' colnames(X_new) <- paste0("V", seq_len(p))
#' preds <- predict(fit, newdata = X_new)
#' dim(preds)
predict.orthoMTL <- function(object, newdata,
                             type = c("link", "response", "class"), ...) {

  type <- match.arg(type)
  is_logistic <- isTRUE(object$hyperparameters$logistic)
  if (type == "class" && !is_logistic) {
    stop("type = \"class\" is only available for logistic ",
         "(classification) fits.", call. = FALSE)
  }

  # --- Column alignment ---
  if (!is.null(object$feature_names)) {

    if (is.null(colnames(newdata))) {
      stop("'newdata' must have column names when the fitted model ",
           "contains feature_names.", call. = FALSE)
    }

    expected <- object$feature_names
    provided <- colnames(newdata)

    # Error on missing features
    missing_feats <- setdiff(expected, provided)
    if (length(missing_feats) > 0) {
      stop("The following features required by the model are missing ",
           "from 'newdata': ",
           paste(missing_feats, collapse = ", "), call. = FALSE)
    }

    # Warn on extra features
    extra_feats <- setdiff(provided, expected)
    if (length(extra_feats) > 0) {
      warning("The following features in 'newdata' are not used by the ",
              "model and will be ignored: ",
              paste(extra_feats, collapse = ", "), call. = FALSE)
    }

    # Align and subset
    newdata <- newdata[, expected, drop = FALSE]

  } else {
    # No feature names — fall back to dimension check
    if (ncol(newdata) != object$n_features) {
      stop("'newdata' has ", ncol(newdata), " columns but the model ",
           "expects ", object$n_features, ".", call. = FALSE)
    }
  }

  # --- Prediction ---
  B <- object$B
  Mt <- newdata %*% B

  # --- Survival monotonicity projection ---
  # TODO(v1.1): Monotonicity projection is applied here in predict() but
  #   not during training loss computation. This asymmetry means the model
  #   is trained on raw scores but evaluated on projected scores.
  #   Investigate impact on SOLAR-1 results and simulated vignette.
  if (isTRUE(object$hyperparameters$survival)) {
    Mt <- t(apply(Mt, 1, nnmaxheap_C))
  }

  # --- Response-scale / class transforms (logistic fits only) ---
  if (is_logistic && type == "response") {
    Mt <- 1 / (1 + exp(-Mt))
  } else if (type == "class") {
    # sign(0) is 0; map the boundary to the positive class for a clean
    # {-1, +1} output.
    Mt <- ifelse(Mt >= 0, 1, -1)
  }

  # --- Label output ---
  if (!is.null(object$task_names)) {
    colnames(Mt) <- object$task_names
  }
  if (!is.null(rownames(newdata))) {
    rownames(Mt) <- rownames(newdata)
  }

  return(Mt)
}

#' Multi-task learning with orthogonal constraints
#'
#' This function solves a multi-task problem where relationships between tasks can be complex
#'
#' @param X a matrix of predictor variables with dimensions n x p
#' @param Y a matrix of response variables with dimensions n x numTasks, where numTasks is the number of response variables. NAs can be used for censored data.
#' @param lambda the regularization parameter for the OrthoPen penalty, default is 1
#' @param step_size the step size for updating the regression coefficients in gradient descent, default is 0.1
#' @param verbose the level of verbosity, default is 0 (no messages)
#' @param stop_no_improve the number of iterations without improvement in the objective function to trigger convergence, default is 100
#' @param max_iter the maximum number of iterations, default is 1e+06
#' @param W_0 a matrix of initial values for the regression coefficients, default is NULL, as not provided and will be randomly attributed
#' @param seed an optional random seed for reproducibility, default is
#'   \code{NULL} (no seed is set; the ambient RNG state is used as-is).
#'   Only relevant when \code{W_0} is not provided and \code{disjoint = TRUE},
#'   since that is the only case where \code{orthoMTL} draws random numbers.
#'   Pass an integer to make that random initialisation reproducible.
#' @param K a constraint matrix of weights to adjust the OrthoPen penalty, default is an identity matrix with dimensions numTasks x numTasks
#' @param disjoint a logical value indicating whether the response variables should have disjoint supports, default is FALSE
#' @param logistic a logical value indicating whether logistic regression should be used instead of linear regression, default is FALSE
#' @param alpha the elastic-net mixing parameter in \eqn{[0, 1]}, default is 0.
#'   The penalty is \eqn{\lambda [(1 - \alpha)/2\, \Omega_K(W)^2 + \alpha \|W\|_1]};
#'   \code{alpha = 0} gives the pure orthogonality penalty and \code{alpha = 1}
#'   gives a pure Lasso.
#' @param schedule Character; the gradient-step decay schedule -- how the
#'   per-iteration scale grows with the iteration index \eqn{i}. The coefficient
#'   update is \eqn{W \leftarrow W - \texttt{step\_size}\, \nabla /
#'   (\texttt{scale} \cdot \|\nabla\|)}, so a faster-growing \code{scale} means a
#'   faster-decaying effective step. One of:
#'   \describe{
#'     \item{\code{"sqrt"}}{(default) \eqn{\texttt{scale} = \sqrt{i}}, effective
#'       step \eqn{\propto 1/\sqrt{i}} -- the classic subgradient schedule and
#'       the historical hardcoded behaviour.}
#'     \item{\code{"log"}}{\eqn{\texttt{scale} = \log(i) + 1} (slow decay).}
#'     \item{\code{"const"}}{\eqn{\texttt{scale} = 1} (no decay).}
#'     \item{\code{"linear"}}{\eqn{\texttt{scale} = i}, effective step
#'       \eqn{\propto 1/i} (Robbins-Monro).}
#'   }
#'   The default \code{"sqrt"} is retained for backward compatibility: it
#'   reproduces previously published results exactly. An A/B study shipped with
#'   the package (\code{validation/ab_s02_gradient_schedule.R}) found that
#'   \code{"sqrt"} reaches the \emph{correct} optimum but converges to it roughly
#'   12--17x slower than \code{"log"} and \code{"const"} (which reach the same
#'   objective in a small fraction of the iterations), while \code{"linear"}
#'   decays too aggressively and can stall short of the optimum. Prefer
#'   \code{"log"} or \code{"const"} when convergence speed matters; note that
#'   changing the schedule changes the optimisation path and therefore the exact
#'   coefficients, so it should not be altered when reproducing published runs.
#' @param survival a logical value indicating whether survival analysis should be performed, default is FALSE
#' @param censored.mat a matrix indicating whether observations are censored, used only if survival=TRUE
#' @param tol Convergence tolerance. The algorithm stops when the
#'   improvement in the objective function is less than \code{tol}.
#'   Default: \code{1e-5}.
#' @return a list containing the following elements:
#' \item{B}{a matrix of regression coefficients with dimensions p x numTasks}
#' \item{obj}{the final objective function when algorithms stops}
#' \item{imax}{the number of iterations}
#'
#' @export
#'
#' @references Kevin Vervier, Pierre Mahé, Alexandre d’Aspremont, Jean-Baptiste Veyrieras, Jean-Philippe Vert (2014). On learning matrices with orthogonal columns or disjoint supports. https://hal.science/hal-00985654/file/learningDisjointSupports.pdf
#' @examples
#' # Regression with orthogonal columns
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' W_true <- qr.Q(qr(matrix(rnorm(p * n_tasks), p, n_tasks)))
#' Y <- X %*% W_true + matrix(rnorm(n * n_tasks), n) * 0.1
#' K <- matrix(1, n_tasks, n_tasks)
#' diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K, disjoint = FALSE)
#' fit$B           # coefficient matrix
#' fit$converged   # did optimisation converge?
#'
# TODO(v1.1): Loss is normalised by nrow(X) but penalty terms are not.
#   Effective regularisation strength depends on sample size.
#   Investigate impact on SOLAR-1 results and simulated vignette.

# RESOLVED (S-02, 2026-06): the gradient scaling is now the `schedule`
#   parameter (default "sqrt" reproduces the historical hardcoded behaviour).
#   The A/B in validation/ab_s02_gradient_schedule.R quantifies the trade-off:
#   "log"/"const" reach the same optimum far faster; "linear" can stall.

# TODO(v1.1): stop_no_improve default (100) may be insufficient for
#   high-dimensional or survival problems. Investigate on SOLAR-1 data.

# TODO(v1.1): W_0 initialisation differs between disjoint (random) and
#   non-disjoint (zero). This creates asymmetric seed-dependence.
#   Investigate whether consistent initialisation changes results.

orthoMTL <- function(X, Y, lambda = 1,
                     step_size = 0.1, tol = 1e-5,
                     stop_no_improve = 100, max_iter = 1e+06,
                     W_0 = NULL, seed = NULL,
                     K = NULL, disjoint = FALSE, logistic = FALSE,
                     alpha = 0,
                     schedule = c("sqrt", "log", "const", "linear"),
                     survival = FALSE, censored.mat = NULL,
                     verbose = 0){
  # gradient-step decay schedule (S-02); default "sqrt" = historical behaviour
  schedule <- match.arg(schedule)
  # only touch the RNG state if the caller explicitly asked for reproducibility
  if (!is.null(seed)) set.seed(seed)

  # extract problem dimensions
  if(is.null(X) | is.null(Y)) stop('Training data X and Y need to be provided \n')
  m = nrow(X)
  p = ncol(X)
  numTasks = ncol(Y)

  # regularization parameters test
  if(lambda <0) stop('Regularization parameter lambda needs to be positive value \n')

  # if the K matrix is not defined, set it as identity matrix
  if (is.null(K)) K = diag(1, numTasks)
  # if no initial W_0 was provided, generate a random one.
  if(is.null(W_0)){
    # in the non-disjoint case, we observed that results were stable when starting with the zero matrix
    if (!disjoint) {
      W_k = matrix(0, nrow = p, ncol = numTasks)
    }
    else {
      # in the disjoint case, we initiate with a random matrix with positive values
      W_k = matrix(rnorm(p * numTasks), ncol = numTasks)
      W_k = abs(W_k)
    }
  }else{
    W_k = W_0
  }
  # store current model
  W = W_k
  # in the disjoint case, a complementary matrix V is created
  if (disjoint) {
    V_k = W_k
    V <- V_k
  }

  # elastic-net mixing parameter must lie in [0, 1]
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha < 0 || alpha > 1) {
    stop("Mixing parameter alpha must be a single number in [0, 1]", call. = FALSE)
  }

  # initiate variables tracking progress
  new <- Inf # new objective value
  no_improv = 0 # number of steps without improvement
  i = 0 # step counter

  # print header
  if (verbose > 0) cat("Step \t ObjFun\t NonImproving\n", sep = "")

  # main loop
  while ((no_improv < stop_no_improve) && (i < max_iter)) {
    i <- i + 1
    # print update every 1000 steps
    if (verbose > 0 & i%%1000 == 0) {
      cat(i, "\t", new, "\t", no_improv, "\n", sep = "")
      idx = which(W_k != 0, arr.ind = TRUE)
      cat("Current sparse support:", length(W_k) - nrow(idx),
          "\n")
    }

    ####################
    # loss calculation #
    #------------------#

    # loss pre-computation
    # precompute the linear predictor once and reuse in loss and gradient
    XW_k = X %*% W_k
    if (!logistic) {
      LS = XW_k - Y #MSE
    }
    else {
      # Numerically stable softplus: log(1+exp(-z)) = max(-z,0) + log1p(exp(-|z|))
      # where z = Y * (X W). Avoids exp() overflow for large |z|. (REG-02)
      score = Y * XW_k
      LS = pmax(-score, 0) + log1p(exp(-abs(score))) # Logistic
    }

    # for survival analysis, the loss calculation is restricted to non-censored data
    if(survival){
      if(!is.null(censored.mat)){
        LS[censored.mat == 0] = 0
      }else{stop("A matrix with censoring information needs to be provided\n")}
    }

    #compute a scale for gradient descent (S-02: `schedule` controls the decay)
    scale = switch(schedule,
                   sqrt   = sqrt(i),
                   log    = log(i) + 1,
                   const  = 1,
                   linear = i)

    # orthogonality penalty matrix (V^T V for disjoint, W^T W otherwise)
    if (disjoint) {
      PEN = crossprod(V_k)
      W_or_V = V_k
    }else {
      PEN = crossprod(W_k)
      W_or_V = W_k
    }

    # elastic-net penalty:
    #   lambda * [ (1 - alpha)/2 * Omega_K(W)^2  +  alpha * ||W||_1 ]
    # alpha = 0 -> pure orthogonality penalty; alpha = 1 -> pure Lasso.
    pen_obj = (1 - alpha) / 2 * lambda * sum(abs(PEN) * K) +
      alpha * lambda * sum(abs(W_or_V))

    # total objective calculation
    if (!logistic) {
      tmp = 0.5 * sum(LS^2) / nrow(X) + pen_obj
    }else {
      tmp = sum(LS) / nrow(X) + pen_obj
    }

    # Previously: abs(tmp - new) > 10^-5
    if (tmp < new && abs(tmp - new) > tol) {
      # if improvment, update all terms
      no_improv <- 0
      new <- tmp
      W <- W_k
      if (disjoint)
        V <- V_k
    }
    else { #if no improvement, add one
      no_improv <- no_improv + 1
    }

    ########################
    # gradient computation #
    #----------------------#

    if (disjoint) {
      # gradient of the data-fit term w.r.t. W
      if (!logistic) {
        gradientW <- crossprod(X, LS)/nrow(X)
      }else {
        # Correct subgradient for y in {-1,+1}: -X^T(y/(1+exp(y*Xw)))/m (REG-01)
        gradientW <- -crossprod(X, Y/(1 + exp(pmin(Y * XW_k, 500))))/nrow(X)
      }
      # subgradient of the elastic-net penalty w.r.t. V
      gradientV <- (1 - alpha) * lambda * V_k %*% (sign(PEN) * K) +
        alpha * lambda * sign(V_k)
      norm_gradient <- sqrt(sum((gradientW + gradientV)^2))
    }else {
      # subgradient of data-fit + elastic-net penalty w.r.t. W
      if (!logistic) {
        gradientW <- crossprod(X, LS)/nrow(X) +
          (1 - alpha) * lambda * W_k %*% (sign(PEN) * K) +
          alpha * lambda * sign(W_k)
      }else {
        gradientW <- -crossprod(X, Y/(1 + exp(pmin(Y * XW_k, 500))))/nrow(X) +
          (1 - alpha) * lambda * W_k %*% (sign(PEN) * K) +
          alpha * lambda * sign(W_k)
      }
      norm_gradient <- sqrt(sum((gradientW)^2))
    }

    ############################################
    # update coefficients via gradient descent #
    #------------------------------------------#

    if (norm_gradient > 0) {
      scale_grad = scale * norm_gradient

      W_k <- W_k - step_size * gradientW/scale_grad
      if (disjoint)
        V_k <- V_k - step_size * gradientV/scale_grad

      if (disjoint) {
        projection <- proj_disjoint(w = W_k, v = V_k)
        W_k <- projection$w
        V_k <- projection$v
      }

    }else { #if norm is zero, we stop
      no_improv <- stop_no_improve + 1
    }
  }

  # create final output
  imax = i
  if(imax == max_iter) warning('Optimization reached the maximal number of iterations without converging \n
                               Please consider increasing the number of iterations or the step size \n')
  best_obj <- new
  w_star <- W
  # Build S3 return object
  structure(
    list(
      B              = w_star,
      W              = w_star,  # orthopen backward compatibility alias
      obj            = new,
      imax           = imax,
      converged      = imax < max_iter,
      call           = match.call(),
      hyperparameters = list(
        lambda          = lambda,
        alpha           = alpha,
        schedule        = schedule,
        step_size       = step_size,
        tol             = tol,
        stop_no_improve = stop_no_improve,
        max_iter        = max_iter,
        seed            = seed,
        disjoint        = disjoint,
        logistic        = logistic,
        survival        = survival
      ),
      K              = K,
      n_tasks        = numTasks,
      n_features     = p,
      n_obs          = m,
      feature_names  = colnames(X),
      task_names     = colnames(Y)
    ),
    class = "orthoMTL"
  )
}

# Tests for predict.orthoMTL type= argument (GEN-03)

make_logistic_fit <- function(seed = 42) {
  set.seed(seed)
  n <- 60; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  # labels in {-1, +1}, the encoding the logistic solver expects
  lin <- X[, 1] - X[, 2]
  Y <- matrix(0, n, n_tasks)
  for (k in seq_len(n_tasks)) {
    Y[, k] <- ifelse(stats::runif(n) < 1 / (1 + exp(-lin)), 1, -1)
  }
  list(fit = orthoMTL(X, Y, lambda = 0.01, logistic = TRUE),
       X = X, n_tasks = n_tasks)
}

test_that("type = 'link' is the default and unchanged", {
  obj <- make_logistic_fit()
  p_default <- predict(obj$fit, newdata = obj$X)
  p_link    <- predict(obj$fit, newdata = obj$X, type = "link")
  expect_equal(p_default, p_link)
})

test_that("type = 'response' returns probabilities in (0, 1)", {
  obj <- make_logistic_fit()
  resp <- predict(obj$fit, newdata = obj$X, type = "response")
  expect_true(all(resp > 0 & resp < 1))
  # response is the sigmoid of the link
  link <- predict(obj$fit, newdata = obj$X, type = "link")
  expect_equal(resp, 1 / (1 + exp(-link)))
})

test_that("type = 'class' returns labels in {-1, +1}", {
  obj <- make_logistic_fit()
  cls <- predict(obj$fit, newdata = obj$X, type = "class")
  expect_true(all(cls %in% c(-1, 1)))
  link <- predict(obj$fit, newdata = obj$X, type = "link")
  expect_equal(cls, ifelse(link >= 0, 1, -1))
})

test_that("type = 'class' errors for non-logistic fits", {
  set.seed(1)
  n <- 30; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  fit <- orthoMTL(X, Y, lambda = 0.01)  # regression
  expect_error(predict(fit, newdata = X, type = "class"),
               "only available for logistic")
})

test_that("type = 'response' equals link for non-logistic fits", {
  set.seed(1)
  n <- 30; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  fit <- orthoMTL(X, Y, lambda = 0.01)
  expect_equal(predict(fit, newdata = X, type = "response"),
               predict(fit, newdata = X, type = "link"))
})

# Tests for the orthoMTL `schedule` argument (S-02)

make_reg <- function(seed = 5, n = 120, p = 10, n_tasks = 3) {
  set.seed(seed)
  X <- matrix(rnorm(n * p), n, p)
  B <- matrix(rnorm(p * n_tasks), p, n_tasks)
  Y <- X %*% B + matrix(rnorm(n * n_tasks), n, n_tasks) * 0.3
  list(X = X, Y = Y)
}

test_that("default schedule is 'sqrt' and recorded in hyperparameters", {
  d <- make_reg()
  fit <- orthoMTL(d$X, d$Y, lambda = 1e-2)
  expect_equal(fit$hyperparameters$schedule, "sqrt")
})

test_that("explicit schedule = 'sqrt' equals the default (back-compat)", {
  d <- make_reg()
  fit_default <- orthoMTL(d$X, d$Y, lambda = 1e-2, seed = 1)
  fit_sqrt    <- orthoMTL(d$X, d$Y, lambda = 1e-2, seed = 1, schedule = "sqrt")
  expect_identical(fit_default$B, fit_sqrt$B)
})

test_that("all schedules produce finite coefficients of the right shape", {
  d <- make_reg()
  for (sc in c("sqrt", "log", "const", "linear")) {
    # 'linear' (1/i) may not converge within the budget; that emits the
    # documented max_iter warning, which is expected here -- we only check
    # the returned coefficients are well-formed.
    fit <- suppressWarnings(
      orthoMTL(d$X, d$Y, lambda = 1e-2, schedule = sc,
               stop_no_improve = 200, max_iter = 5e4))
    expect_true(all(is.finite(fit$B)), info = sc)
    expect_equal(dim(fit$B), c(ncol(d$X), ncol(d$Y)), info = sc)
  }
})

test_that("'log' and 'const' reach the sqrt optimum in fewer iterations", {
  d <- make_reg()
  fit_sqrt  <- orthoMTL(d$X, d$Y, lambda = 1e-2, schedule = "sqrt",
                        stop_no_improve = 300, max_iter = 1e5)
  fit_log   <- orthoMTL(d$X, d$Y, lambda = 1e-2, schedule = "log",
                        stop_no_improve = 300, max_iter = 1e5)
  fit_const <- orthoMTL(d$X, d$Y, lambda = 1e-2, schedule = "const",
                        stop_no_improve = 300, max_iter = 1e5)

  # Same optimum (within a small tolerance) ...
  expect_lt(abs(fit_log$obj   - fit_sqrt$obj), 1e-3)
  expect_lt(abs(fit_const$obj - fit_sqrt$obj), 1e-3)
  # ... reached strictly faster than sqrt.
  expect_lt(fit_log$imax,   fit_sqrt$imax)
  expect_lt(fit_const$imax, fit_sqrt$imax)
})

test_that("invalid schedule is rejected", {
  d <- make_reg()
  expect_error(orthoMTL(d$X, d$Y, lambda = 1e-2, schedule = "bogus"),
               "'arg'")
})

test_that("cv_orthoMTL accepts a schedule and rejects a bad one", {
  d <- make_reg(n = 30, p = 4, n_tasks = 2)
  folds <- rep(1:2, length.out = nrow(d$X))
  cv_res <- cv_orthoMTL(
    X.train = d$X, Y.train = d$Y, schedule = "log",
    lambdas = c(0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )
  expect_s3_class(cv_res, "cv_orthoMTL")
  expect_true(is.finite(cv_res$best$cv_score))

  expect_error(
    cv_orthoMTL(X.train = d$X, Y.train = d$Y, schedule = "bogus",
                lambdas = 0.01, stepsizes = 0.1, diag_vals = 1,
                survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE),
    "'arg'"
  )
})

test_that("bootstrap_orthoMTL accepts a schedule and rejects a bad one", {
  d <- make_reg(n = 30, p = 4, n_tasks = 2)
  colnames(d$X) <- paste0("V", seq_len(ncol(d$X)))
  boot <- bootstrap_orthoMTL(
    X = d$X, Y = d$Y, lambda = 0.01, schedule = "const",
    survival = FALSE, n_repeats = 3, n_cores = 1, verbose = FALSE
  )
  expect_s3_class(boot, "bootstrap_orthoMTL")
  expect_length(boot$obj_real, 3)

  expect_error(
    bootstrap_orthoMTL(X = d$X, Y = d$Y, lambda = 0.01, schedule = "bogus",
                       survival = FALSE, n_repeats = 2, n_cores = 1,
                       verbose = FALSE),
    "'arg'"
  )
})

#' Print an orthoMTL Object
#'
#' Displays a compact summary of a fitted \code{orthoMTL} model.
#'
#' @param x A fitted model object of class \code{"orthoMTL"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @method print orthoMTL
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' fit <- orthoMTL(X, Y, lambda = 1e-3)
#' print(fit)
print.orthoMTL <- function(x, ...) {

  # Determine mode
  mode <- if (isTRUE(x$hyperparameters$survival)) {
    "survival"
  } else if (isTRUE(x$hyperparameters$logistic)) {
    "classification"
  } else {
    "regression"
  }

  cat("orthoMTL model:", x$n_features, "features,",
      x$n_tasks, "tasks (", mode, ")\n", sep = " ")

  conv_str <- if (x$converged) "yes" else "NO"
  cat("Converged:", conv_str, "(", x$imax, "iterations, objective:",
      format(x$obj, digits = 4, nsmall = 4), ")\n", sep = " ")

  pen_str <- paste0("lambda = ", x$hyperparameters$lambda)
  if (!is.null(x$hyperparameters$alpha) && x$hyperparameters$alpha > 0) {
    pen_str <- paste0(pen_str, ", alpha = ", x$hyperparameters$alpha)
  }
  cat("Penalty:", pen_str, "\n")

  cat("Constraints: disjoint =", x$hyperparameters$disjoint, "\n")

  invisible(x)
}


#' Summarise an orthoMTL Object
#'
#' Displays a detailed summary of a fitted \code{orthoMTL} model,
#' including hyperparameters, coefficient matrix statistics, and
#' top features by mean absolute coefficient.
#'
#' @param object A fitted model object of class \code{"orthoMTL"}.
#' @param n_top Number of top features to display. Default: 5.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns \code{object}.
#'
#' @method summary orthoMTL
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' fit <- orthoMTL(X, Y, lambda = 1e-3)
#' summary(fit)
summary.orthoMTL <- function(object, n_top = 5, ...) {

  # Print compact header
  print.orthoMTL(object)

  cat("---\n")

  # Hyperparameters
  cat("Hyperparameters:\n")
  hp <- object$hyperparameters
  for (nm in names(hp)) {
    cat("  ", nm, "=", hp[[nm]], "\n")
  }

  cat("---\n")

  # Coefficient matrix summary
  B <- object$B
  n_zero <- sum(B == 0)
  n_total <- length(B)
  sparsity <- 100 * n_zero / n_total

  cat("Coefficients:", object$n_features, "x", object$n_tasks, "matrix\n")
  cat("  Sparsity:", format(sparsity, digits = 1, nsmall = 1), "%\n")
  cat("  Range: [", format(min(B), digits = 4), ",",
      format(max(B), digits = 4), "]\n")

  cat("---\n")

  # Top features by mean absolute coefficient
  if (!is.null(object$feature_names)) {
    mean_abs <- sort(apply(abs(B), 1, mean), decreasing = TRUE)
    n_show <- min(n_top, length(mean_abs))
    cat("Top", n_show, "features (mean |coefficient| across tasks):\n")
    top <- mean_abs[seq_len(n_show)]
    for (i in seq_along(top)) {
      cat("  ", names(top)[i], ":", format(top[i], digits = 4), "\n")
    }
  }

  # Survival task thresholds
  if (isTRUE(object$hyperparameters$survival) && !is.null(object$task_names)) {
    cat("---\n")
    cat("Task thresholds:", paste(object$task_names, collapse = ", "), "\n")
  }

  invisible(object)
}

#' Concordance Index for Multi-Task Predictions
#'
#' Computes a concordance index (C-index) adapted for the multi-task
#' survival framework. Predictions across tasks are aggregated by
#' row-sum to produce a single score per patient, then concordance
#' is evaluated over pairs where at least one member has a fully
#' observed outcome.
#'
#' @param true.label.mat A numeric matrix of true response labels with
#'   dimensions \code{n x numTasks}. May contain \code{NA} for censored
#'   observations (as produced by \code{\link{create_longitudinal_labels}}).
#' @param pred.label.mat A numeric matrix of predicted response values
#'   with dimensions \code{n x numTasks} (as produced by
#'   \code{\link{predict.orthoMTL}}).
#'
#' @return A numeric value between 0 and 1 (higher is better).
#'   A value of 0.5 indicates random concordance.
#'
#' @details
#' The effective survival time for each patient is derived as the
#' highest task index where the true label is positive (i.e., the
#' last threshold at which the patient was progression-free).
#'
#' A patient is considered "uncensored" only if all task labels are
#' non-\code{NA}. Concordant pairs require both a correct ordering
#' of effective survival times and a matching ordering of predicted
#' scores.
#'
#' @section Known limitations:
#' The following limitations are documented and flagged for future
#' investigation:
#' \itemize{
#'   \item No credit for tied predictions or tied survival times
#'   \item Strict censoring: uncensored requires all tasks observed
#'   \item Equal weighting of all tasks in the row-sum aggregation
#' }
#'
#' @seealso \code{\link{predict.orthoMTL}} for generating the
#'   prediction matrix.
#'
#' @export
#'
#' @examples
#' # Simulate a small multi-task prediction scenario
#' set.seed(42)
#' n <- 30; n_tasks <- 4
#' SurvTime <- rexp(n, rate = 0.1)
#' Event <- rbinom(n, 1, 0.7)
#' thresholds <- c(4, 6, 10, 15)
#'
#' Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
#'
#' # Simulate imperfect predictions (add noise to true labels)
#' pred <- Y
#' pred[is.na(pred)] <- 0.5
#' pred <- pred + matrix(rnorm(n * n_tasks, sd = 0.3), n, n_tasks)
#'
#' cindex_mtl(Y, pred)
cindex_mtl <- function(true.label.mat, pred.label.mat) {

  # TODO(v1.1): Row-sum aggregation weights all tasks equally regardless
  #   of interval width (e.g., threshold gap 2->4 months counts the same
  #   as 15->22 months). Investigate whether weighted aggregation by
  #   interval width improves discrimination.
  pred.vec <- apply(pred.label.mat, 1, sum)

  sum1 <- 0
  sum2 <- 0
  sum3 <- 0
  sum4 <- 0

  for (i in 1:(nrow(true.label.mat) - 1)) {
    for (j in (i + 1):nrow(true.label.mat)) {

      # Effective survival time: last threshold where label > 0
      if (length(which(true.label.mat[i, ] > 0)) > 0) {
        stime1 <- max(which(true.label.mat[i, ] > 0))
      } else {
        stime1 <- 0
      }

      if (length(which(true.label.mat[j, ] > 0)) > 0) {
        stime2 <- max(which(true.label.mat[j, ] > 0))
      } else {
        stime2 <- 0
      }

      pred1 <- pred.vec[i]
      pred2 <- pred.vec[j]

      # TODO(v1.1): Strict censoring definition — a patient is considered
      #   uncensored only if ALL task labels are non-NA. Patients censored
      #   at later thresholds never contribute as the event member of a
      #   concordant pair. This may bias the metric toward patients with
      #   early events. Investigate per-task censoring alternatives.
      status1 <- as.numeric(all(!is.na(true.label.mat[i, ])))
      status2 <- as.numeric(all(!is.na(true.label.mat[j, ])))

      # TODO(v1.1): No tie handling — concordance only counts strict
      #   inequalities. Standard implementations (e.g., survival::concordance)
      #   give 0.5 credit for ties. With binary features producing many
      #   tied predictions, this may systematically undercount concordance.
      #   Investigate impact on reported C-index values.
      if (stime1 < stime2 & pred1 < pred2 & status1 == 1) sum1 <- sum1 + 1
      if (stime2 < stime1 & pred2 < pred1 & status2 == 1) sum2 <- sum2 + 1
      if (stime1 < stime2 & status1 == 1) sum3 <- sum3 + 1
      if (stime2 < stime1 & status2 == 1) sum4 <- sum4 + 1
    }
  }

  return((sum1 + sum2) / (sum3 + sum4))
}


# ---------------------------------------------------------------------
# General-purpose metrics for non-survival modes (GEN-01)
#
# These complement the survival-specific cindex_mtl for the regression
# and classification use cases of orthoMTL. All operate cell-wise on the
# n x numTasks true/predicted matrices, pooling over every non-NA cell
# (so a single NA-masked or fully observed matrix both work). They never
# touch cindex_mtl, which stays faithful to the MTLSA reference.
# ---------------------------------------------------------------------

# Internal: flatten true/pred matrices to aligned non-NA vectors.
.mtl_pool <- function(true.mat, pred.mat) {
  true.mat <- as.matrix(true.mat)
  pred.mat <- as.matrix(pred.mat)
  if (!all(dim(true.mat) == dim(pred.mat))) {
    stop("true and predicted matrices must have identical dimensions.",
         call. = FALSE)
  }
  t_vec <- as.numeric(true.mat)
  p_vec <- as.numeric(pred.mat)
  keep <- !is.na(t_vec) & !is.na(p_vec)
  if (!any(keep)) {
    stop("No non-NA cells to evaluate.", call. = FALSE)
  }
  list(true = t_vec[keep], pred = p_vec[keep])
}

#' Root Mean Squared Error for Multi-Task Regression
#'
#' Pooled RMSE over every non-\code{NA} cell of the true/predicted
#' matrices, for \code{orthoMTL} fits in regression mode
#' (\code{logistic = FALSE}, \code{survival = FALSE}).
#'
#' @param true.label.mat A numeric matrix of true responses
#'   (\code{n x numTasks}). \code{NA} cells are ignored.
#' @param pred.label.mat A numeric matrix of predictions of the same
#'   dimensions (as produced by \code{\link{predict.orthoMTL}}).
#'
#' @return A single non-negative numeric value (lower is better).
#'
#' @seealso \code{\link{r2_mtl}}, \code{\link{cindex_mtl}}
#' @export
#'
#' @examples
#' set.seed(1)
#' Y <- matrix(rnorm(30), 10, 3)
#' P <- Y + matrix(rnorm(30, sd = 0.1), 10, 3)
#' rmse_mtl(Y, P)
rmse_mtl <- function(true.label.mat, pred.label.mat) {
  v <- .mtl_pool(true.label.mat, pred.label.mat)
  sqrt(mean((v$true - v$pred)^2))
}

#' Coefficient of Determination (R-squared) for Multi-Task Regression
#'
#' Pooled \eqn{R^2 = 1 - SS_{res} / SS_{tot}} over every non-\code{NA}
#' cell, for \code{orthoMTL} fits in regression mode. \eqn{SS_{tot}} uses
#' the global mean of the pooled true values.
#'
#' @inheritParams rmse_mtl
#'
#' @return A single numeric value (higher is better; 1 = perfect).
#'   Can be negative when predictions are worse than the mean.
#'
#' @seealso \code{\link{rmse_mtl}}, \code{\link{cindex_mtl}}
#' @export
#'
#' @examples
#' set.seed(1)
#' Y <- matrix(rnorm(30), 10, 3)
#' P <- Y + matrix(rnorm(30, sd = 0.1), 10, 3)
#' r2_mtl(Y, P)
r2_mtl <- function(true.label.mat, pred.label.mat) {
  v <- .mtl_pool(true.label.mat, pred.label.mat)
  ss_res <- sum((v$true - v$pred)^2)
  ss_tot <- sum((v$true - mean(v$true))^2)
  if (ss_tot == 0) {
    stop("Cannot compute R-squared: true values have zero variance.",
         call. = FALSE)
  }
  1 - ss_res / ss_tot
}

#' Classification Accuracy for Multi-Task Predictions
#'
#' Pooled fraction of correctly classified non-\code{NA} cells, for
#' \code{orthoMTL} fits in classification mode (\code{logistic = TRUE}).
#' A cell is positive when its true label is \code{> 0} and predicted
#' positive when its score is \code{> 0}. This works for both the
#' \eqn{\{-1, +1\}} encoding the solver optimises and a \eqn{\{0, 1\}}
#' encoding, and for raw \code{"link"} scores or \code{"response"}
#' probabilities (threshold 0.5 corresponds to a link of 0... see note).
#'
#' @inheritParams rmse_mtl
#' @param threshold Decision threshold applied to \code{pred.label.mat}.
#'   Default \code{0} matches raw link scores; pass \code{0.5} for
#'   probabilities from \code{predict(..., type = "response")}.
#'
#' @return A single numeric value in \code{[0, 1]} (higher is better).
#'
#' @seealso \code{\link{auc_mtl}}, \code{\link{cindex_mtl}}
#' @export
#'
#' @examples
#' set.seed(1)
#' Y <- matrix(sample(c(-1, 1), 30, replace = TRUE), 10, 3)
#' P <- Y * abs(matrix(rnorm(30), 10, 3))   # mostly correct signs
#' accuracy_mtl(Y, P)
accuracy_mtl <- function(true.label.mat, pred.label.mat, threshold = 0) {
  v <- .mtl_pool(true.label.mat, pred.label.mat)
  true_pos <- v$true > 0
  pred_pos <- v$pred > threshold
  mean(true_pos == pred_pos)
}

#' Area Under the ROC Curve for Multi-Task Predictions
#'
#' Pooled AUC over every non-\code{NA} cell, computed via the
#' Mann-Whitney U statistic (with 0.5 credit for ties), for
#' \code{orthoMTL} fits in classification mode. The positive class is
#' defined by true label \code{> 0}. AUC is invariant to monotone
#' transforms, so raw \code{"link"} scores and \code{"response"}
#' probabilities give identical results.
#'
#' @inheritParams rmse_mtl
#'
#' @return A single numeric value in \code{[0, 1]} (higher is better;
#'   0.5 = random).
#'
#' @seealso \code{\link{accuracy_mtl}}, \code{\link{cindex_mtl}}
#' @export
#'
#' @examples
#' set.seed(1)
#' Y <- matrix(sample(c(-1, 1), 30, replace = TRUE), 10, 3)
#' P <- Y + matrix(rnorm(30), 10, 3)
#' auc_mtl(Y, P)
auc_mtl <- function(true.label.mat, pred.label.mat) {
  v <- .mtl_pool(true.label.mat, pred.label.mat)
  pos <- v$pred[v$true > 0]
  neg <- v$pred[v$true <= 0]
  n_pos <- length(pos)
  n_neg <- length(neg)
  if (n_pos == 0 || n_neg == 0) {
    stop("AUC requires both positive (label > 0) and negative cells.",
         call. = FALSE)
  }
  # Mann-Whitney U via rank-sum, with 0.5 credit for ties.
  r <- rank(c(pos, neg))
  auc <- (sum(r[seq_len(n_pos)]) - n_pos * (n_pos + 1) / 2) /
    (n_pos * n_neg)
  auc
}

# Tests for simulate_mtl mode = regression / classification (SIM-01)

test_that("default mode is survival and back-compatible", {
  sim <- simulate_mtl(n = 50, p = 10, n_signals = 3, seed = 42)
  expect_equal(sim$mode, "survival")
  expect_true(is.numeric(sim$SurvTime))
  expect_true(is.numeric(sim$Event))
  expect_null(sim$Y)
})

test_that("regression mode returns a numeric Y and no survival fields", {
  sim <- simulate_mtl(n = 60, p = 10, n_signals = 3,
                      mode = "regression", seed = 42)
  expect_equal(sim$mode, "regression")
  expect_true(is.matrix(sim$Y))
  expect_equal(dim(sim$Y), c(60, length(sim$thresholds)))
  expect_null(sim$SurvTime)
  expect_null(sim$Event)
  expect_true(all(is.finite(sim$Y)))
})

test_that("regression noise_sd influences residual spread", {
  s_lo <- simulate_mtl(n = 200, p = 8, mode = "regression",
                       noise_sd = 0.1, seed = 1)
  s_hi <- simulate_mtl(n = 200, p = 8, mode = "regression",
                       noise_sd = 5, seed = 1)
  signal_lo <- s_lo$X %*% s_lo$ground_truth$coefficients
  signal_hi <- s_hi$X %*% s_hi$ground_truth$coefficients
  expect_true(sd(s_hi$Y - signal_hi) > sd(s_lo$Y - signal_lo))
})

test_that("classification mode returns labels in {-1, +1}", {
  sim <- simulate_mtl(n = 80, p = 10, n_signals = 3,
                      mode = "classification", seed = 42)
  expect_equal(sim$mode, "classification")
  expect_true(is.matrix(sim$Y))
  expect_equal(dim(sim$Y), c(80, length(sim$thresholds)))
  expect_true(all(sim$Y %in% c(-1, 1)))
  expect_null(sim$SurvTime)
})

test_that("non-survival modes keep the same ground-truth machinery", {
  sim <- simulate_mtl(n = 50, p = 20, n_signals = 5,
                      mode = "regression", seed = 42)
  gt <- sim$ground_truth
  expect_equal(dim(gt$coefficients), c(21, length(sim$thresholds)))
  expect_equal(length(gt$signal_features), 5)
})

test_that("regression mode round-trips through orthoMTL + metrics", {
  sim <- simulate_mtl(n = 80, p = 10, n_signals = 4,
                      mode = "regression", noise_sd = 0.2, seed = 42)
  fit <- orthoMTL(sim$X, sim$Y, lambda = 1e-3)
  pred <- predict(fit, newdata = sim$X)
  expect_true(rmse_mtl(sim$Y, pred) < sd(sim$Y))  # better than trivial
})

test_that("classification mode round-trips through orthoMTL + auc", {
  sim <- simulate_mtl(n = 120, p = 8, n_signals = 4,
                      mode = "classification", seed = 7)
  fit <- orthoMTL(sim$X, sim$Y, lambda = 1e-3, logistic = TRUE)
  pred <- predict(fit, newdata = sim$X, type = "link")
  expect_true(auc_mtl(sim$Y, pred) > 0.5)  # learns something
})

test_that("print.simulated_mtl adapts to mode", {
  sim_r <- simulate_mtl(n = 40, p = 8, mode = "regression", seed = 1)
  expect_output(print(sim_r), "Simulated multi-task regression data")
  expect_output(print(sim_r), "Response range")

  sim_c <- simulate_mtl(n = 40, p = 8, mode = "classification", seed = 1)
  expect_output(print(sim_c), "Simulated multi-task classification data")
  expect_output(print(sim_c), "Positive-label rate")
})

# Tests for general-purpose metrics (GEN-01):
# rmse_mtl, r2_mtl, accuracy_mtl, auc_mtl

test_that("rmse_mtl is zero for perfect predictions and positive otherwise", {
  set.seed(1)
  Y <- matrix(rnorm(30), 10, 3)
  expect_equal(rmse_mtl(Y, Y), 0)
  expect_true(rmse_mtl(Y, Y + 0.5) > 0)
})

test_that("rmse_mtl matches the manual pooled formula", {
  Y <- matrix(c(1, 2, 3, 4), 2, 2)
  P <- matrix(c(1, 2, 3, 5), 2, 2)   # one cell off by 1
  expect_equal(rmse_mtl(Y, P), sqrt(1 / 4))
})

test_that("r2_mtl is 1 for perfect fit and <= 1 otherwise", {
  set.seed(1)
  Y <- matrix(rnorm(30), 10, 3)
  expect_equal(r2_mtl(Y, Y), 1)
  P <- Y + matrix(rnorm(30, sd = 0.1), 10, 3)
  r2 <- r2_mtl(Y, P)
  expect_true(r2 < 1 && r2 > 0.9)
})

test_that("r2_mtl errors when true values have zero variance", {
  Y <- matrix(3, 5, 2)
  expect_error(r2_mtl(Y, Y + 1), "zero variance")
})

test_that("accuracy_mtl is 1 when signs all match", {
  Y <- matrix(c(-1, 1, 1, -1), 2, 2)
  P <- Y * 0.3   # same signs, smaller magnitude
  expect_equal(accuracy_mtl(Y, P), 1)
})

test_that("accuracy_mtl works for 0/1 labels and a 0.5 threshold", {
  Y <- matrix(c(0, 1, 1, 0), 2, 2)          # label > 0 == positive
  probs <- matrix(c(0.2, 0.8, 0.9, 0.1), 2, 2)
  expect_equal(accuracy_mtl(Y, probs, threshold = 0.5), 1)
})

test_that("auc_mtl is 1 for perfectly separating scores and 0.5 at chance", {
  Y <- matrix(c(-1, -1, 1, 1), 2, 2)
  P <- matrix(c(0.1, 0.2, 0.8, 0.9), 2, 2)  # positives score higher
  expect_equal(auc_mtl(Y, P), 1)
})

test_that("auc_mtl is invariant to monotone transforms (link vs sigmoid)", {
  set.seed(7)
  Y <- matrix(sample(c(-1, 1), 40, replace = TRUE), 10, 4)
  link <- Y + matrix(rnorm(40), 10, 4)
  resp <- 1 / (1 + exp(-link))
  expect_equal(auc_mtl(Y, link), auc_mtl(Y, resp))
})

test_that("auc_mtl errors when only one class is present", {
  Y <- matrix(1, 4, 2)
  expect_error(auc_mtl(Y, matrix(rnorm(8), 4, 2)), "positive .* and negative")
})

test_that("general metrics ignore NA cells", {
  Y <- matrix(c(1, NA, 3, 4), 2, 2)
  P <- matrix(c(1, 99, 3, 4), 2, 2)   # the NA-aligned cell is wrong but masked
  expect_equal(rmse_mtl(Y, P), 0)
})

test_that("metrics error on dimension mismatch", {
  expect_error(rmse_mtl(matrix(1, 2, 2), matrix(1, 2, 3)),
               "identical dimensions")
})

#' Print Simulated Multi-Task Data
#'
#' Displays a summary of a simulated dataset from
#' \code{\link{simulate_mtl}}.
#'
#' @param x An object of class \code{"simulated_mtl"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @method print simulated_mtl
#' @export
#'
#' @examples
#' sim <- simulate_mtl(n = 100, p = 20, n_signals = 4)
#' print(sim)
print.simulated_mtl <- function(x, ...) {

  mode <- if (!is.null(x$mode)) x$mode else "survival"
  cat("Simulated multi-task ", mode, " data\n", sep = "")
  cat("Patients:", x$n, "| Features:", x$p + 1,
      "(incl. treatment)\n")
  cat("  Continuous:", x$n_continuous,
      "| Binary:", x$p - x$n_continuous,
      "| Treatment: 1\n")
  cat("  Signal features:", x$n_signals,
      "| Null features:", x$p - x$n_signals, "\n")
  cat("Tasks:", paste(x$thresholds, collapse = ", "), "\n")

  if (mode == "survival") {
    event_rate <- round(100 * mean(x$Event), 1)
    cat("Event rate:", event_rate, "%\n")
    cat("Median survival time:",
        format(median(x$SurvTime), digits = 3), "\n")
  } else if (mode == "classification") {
    pos_rate <- round(100 * mean(x$Y > 0), 1)
    cat("Positive-label rate:", pos_rate, "%\n")
  } else {
    cat("Response range: [",
        format(min(x$Y), digits = 3), ", ",
        format(max(x$Y), digits = 3), "]\n", sep = "")
  }

  cat("---\n")
  cat("Signal features and effect types:\n")
  for (feat in x$ground_truth$signal_features) {
    etype <- x$ground_truth$effect_types[feat]
    cat("  ", feat, ":", etype, "\n")
  }

  cat("---\n")
  effect_scale <- if (mode == "survival") "log-hazard" else "coefficient"
  cat("Treatment effect:", x$ground_truth$coefficients["treatment", 1],
      "(", effect_scale, ", constant across tasks)\n")

  invisible(x)
}

#' Simulate Multi-Task Data with Time-Varying Effects
#'
#' Generates a realistic simulated dataset with binary and continuous
#' features and a known ground-truth coefficient structure. The
#' \code{mode} argument selects the response type: piecewise-exponential
#' \strong{survival} times (default), multi-task \strong{regression}
#' targets, or multi-task binary \strong{classification} labels. Designed
#' for demonstrating and testing \code{\link{orthoMTL}}.
#'
#' @param mode Character; the response type to generate. One of
#'   \code{"survival"} (default), \code{"regression"}, or
#'   \code{"classification"}. The feature matrix and ground-truth
#'   coefficients are generated identically across modes; only the
#'   response differs:
#'   \describe{
#'     \item{survival}{piecewise-exponential \code{SurvTime}/\code{Event}
#'       (the historical behaviour).}
#'     \item{regression}{\code{Y = X \%*\% beta + N(0, noise_sd^2)},
#'       returned as the \code{n x length(thresholds)} matrix \code{Y}.}
#'     \item{classification}{binary labels in \eqn{\{-1, +1\}} sampled
#'       from \eqn{P(Y = +1) = 1 / (1 + e^{-X beta})}, returned as
#'       \code{Y}. This matches the label encoding \code{orthoMTL}
#'       optimises with \code{logistic = TRUE}.}
#'   }
#' @param noise_sd Standard deviation of the Gaussian noise added in
#'   \code{mode = "regression"}. Ignored otherwise. Default: \code{1}.
#' @param n Number of patients. Default: \code{200}.
#' @param p Number of features excluding the treatment column.
#'   Default: \code{30}.
#' @param n_signals Number of features with true non-zero effects.
#'   Default: \code{5}. Must be \code{<= p}.
#' @param n_continuous Number of continuous features (placed first
#'   in the feature matrix). Default: \code{1}. Remaining features
#'   are binary.
#' @param thresholds Numeric vector of time thresholds defining the
#'   task structure. Default: \code{c(4, 6, 10, 15)}.
#' @param censoring_max Maximum censoring time. Censoring times are
#'   drawn from \code{Unif(0, censoring_max)}. Default: \code{25}.
#' @param baseline_hazard Baseline hazard rate per interval. Default:
#'   \code{0.05}.
#' @param effect_strength Multiplier controlling the magnitude of
#'   feature effects. Default: \code{0.5}.
#' @param treatment_effect Effect of treatment on the log-hazard
#'   (negative = protective). Default: \code{-0.3}.
#' @param seed Optional random seed for reproducibility. Default:
#'   \code{NULL} (no seed is set; the ambient RNG state is used as-is).
#'   Pass an integer to make the simulated data reproducible.
#'
#' @return An object of class \code{"simulated_mtl"} containing:
#'   \describe{
#'     \item{mode}{The response type generated.}
#'     \item{Y}{For \code{mode = "regression"} / \code{"classification"},
#'       the \code{n x length(thresholds)} response matrix. \code{NULL}
#'       in survival mode (use \code{SurvTime}/\code{Event} instead).}
#'     \item{SurvTime, Event}{Survival mode only (\code{NULL} otherwise):
#'       observed times and event indicators.}
#'     \item{X}{Numeric matrix of dimensions \code{n x (p + 1)}.
#'       Columns include \code{n_continuous} continuous features
#'       (standard normal), \code{p - n_continuous} binary features
#'       (prevalences drawn from \code{Beta(2, 10)}), and a
#'       \code{treatment} column (balanced 1:1).}
#'     \item{SurvTime}{Numeric vector of observed survival times.}
#'     \item{Event}{Binary vector: \code{1} = event observed,
#'       \code{0} = censored.}
#'     \item{treatment}{Binary vector (also present as last column
#'       of \code{X}).}
#'     \item{feature_names}{Character vector of column names of
#'       \code{X}.}
#'     \item{ground_truth}{A list containing:
#'       \describe{
#'         \item{coefficients}{Matrix of true coefficients with
#'           dimensions \code{(p + 1) x length(thresholds)}.}
#'         \item{signal_features}{Names of features with non-zero
#'           effects.}
#'         \item{null_features}{Names of features with zero effects.}
#'         \item{effect_types}{Named character vector mapping signal
#'           features to their temporal effect template.}
#'         \item{thresholds}{The thresholds used.}
#'         \item{baseline_hazard}{The baseline hazard used.}
#'       }
#'     }
#'     \item{n, p, n_signals, n_continuous, thresholds, seed}{Input
#'       parameters stored for reference.}
#'     \item{call}{The matched function call.}
#'   }
#'
#' @details
#' \strong{Feature structure:}
#' \itemize{
#'   \item Continuous features are drawn from \code{N(0, 1)}.
#'   \item Binary features have prevalences drawn from \code{Beta(2, 10)},
#'     producing a realistic range (~5-30\%).
#'   \item Treatment is balanced 1:1 via random assignment.
#' }
#'
#' \strong{Effect templates:}
#' Each signal feature is assigned one of five temporal patterns:
#' \itemize{
#'   \item \code{"early"}: strong effect at early thresholds, fading to
#'     zero at late.
#'   \item \code{"late"}: zero at early thresholds, emerging at late.
#'   \item \code{"constant"}: equal effect across all thresholds
#'     (detectable by standard Cox models).
#'   \item \code{"increasing"}: effect grows over time.
#'   \item \code{"decreasing"}: effect shrinks over time.
#' }
#' Templates are assigned cyclically across signal features with
#' random sign (risk-increasing or protective).
#'
#' \strong{Survival time generation:}
#' Uses a piecewise-exponential model where the hazard in each
#' interval is
#' \code{h_k(i) = baseline_hazard * exp(X[i, ] \%*\% beta[, k])}.
#' Patients progress through intervals sequentially; an event
#' occurs when the simulated time within an interval is shorter
#' than the interval width.
#'
#' \strong{Censoring:}
#' Independent of event times. \code{C ~ Unif(0, censoring_max)}.
#' Observed time = \code{min(T, C)}, event indicator = \code{T <= C}.
#'
#' @seealso \code{\link{create_longitudinal_labels}},
#'   \code{\link{orthoMTL}}
#'
#' @export
#'
#' @examples
#' # Generate simulated data
#' sim <- simulate_mtl(n = 100, p = 20, n_signals = 4, seed = 42)
#' sim
#'
#' # Inspect ground truth
#' sim$ground_truth$signal_features
#' sim$ground_truth$effect_types
#' sim$ground_truth$coefficients[sim$ground_truth$signal_features, ]
#'
#' # Use in orthoMTL workflow
#' \donttest{
#' thresholds <- c(4, 6, 10, 15)
#' Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, thresholds)
#' W <- create_indicator_matrix(Y)
#' K <- create_constraint_matrix(length(thresholds))
#'
#' fit <- orthoMTL(sim$X, Y, lambda = 1e-3, K = K,
#'                 survival = TRUE, censored.mat = W)
#' summary(fit)
#' }
simulate_mtl <- function(n = 200,
                         p = 30,
                         n_signals = 5,
                         n_continuous = 1,
                         thresholds = c(4, 6, 10, 15),
                         mode = c("survival", "regression",
                                  "classification"),
                         noise_sd = 1,
                         censoring_max = 25,
                         baseline_hazard = 0.05,
                         effect_strength = 0.8,  # was 0.5
                         treatment_effect = -0.3,
                         seed = NULL) {

  cl <- match.call()
  mode <- match.arg(mode)

  # ---------------------------
  # Input validation
  # ---------------------------
  if (n < 10) stop("n must be at least 10.", call. = FALSE)
  if (p < 1) stop("p must be at least 1.", call. = FALSE)
  if (n_signals > p) {
    stop("n_signals (", n_signals, ") cannot exceed p (", p, ").",
         call. = FALSE)
  }
  if (n_continuous > p) {
    stop("n_continuous (", n_continuous, ") cannot exceed p (", p, ").",
         call. = FALSE)
  }
  if (n_signals > (p - n_continuous) + n_continuous) {
    # This is always true, but guard against n_signals > p
  }
  if (length(thresholds) < 2) {
    stop("At least 2 thresholds are required.", call. = FALSE)
  }
  if (any(diff(thresholds) <= 0)) {
    stop("Thresholds must be strictly increasing.", call. = FALSE)
  }
  if (baseline_hazard <= 0) {
    stop("baseline_hazard must be positive.", call. = FALSE)
  }
  if (censoring_max <= 0) {
    stop("censoring_max must be positive.", call. = FALSE)
  }

  if (!is.null(seed)) set.seed(seed)

  numTasks <- length(thresholds)
  n_binary <- p - n_continuous

  # ---------------------------
  # Feature names
  # ---------------------------
  continuous_names <- if (n_continuous > 0) {
    paste0("cont_", seq_len(n_continuous))
  } else {
    character(0)
  }
  binary_names <- if (n_binary > 0) {
    paste0("mut_", seq_len(n_binary))
  } else {
    character(0)
  }
  feature_names <- c(continuous_names, binary_names, "treatment")
  p_total <- p + 1  # features + treatment

  # ---------------------------
  # Generate feature matrix X
  # ---------------------------
  X <- matrix(0, nrow = n, ncol = p_total)
  colnames(X) <- feature_names

  # Continuous features: standard normal
  if (n_continuous > 0) {
    X[, continuous_names] <- matrix(
      rnorm(n * n_continuous), nrow = n, ncol = n_continuous
    )
  }

  # Binary features: prevalences from Beta(2, 10)
  if (n_binary > 0) {
    prevalences <- rbeta(n_binary, shape1 = 2, shape2 = 10)
    for (j in seq_len(n_binary)) {
      X[, binary_names[j]] <- rbinom(n, size = 1, prob = prevalences[j])
    }
  }

  # Treatment: balanced 1:1
  X[, "treatment"] <- sample(rep(0:1, length.out = n))

  # ---------------------------
  # Define effect templates
  # ---------------------------
  # Each template is a vector of length numTasks, normalised to max |value| = 1
  template_library <- list(
    early      = .make_template("early", numTasks),
    late       = .make_template("late", numTasks),
    constant   = .make_template("constant", numTasks),
    switch     = .make_template("switch", numTasks),
    decreasing = .make_template("decreasing", numTasks)
  )

  template_names <- names(template_library)

  # ---------------------------
  # Assign effects to features
  # ---------------------------
  # Coefficient matrix: p_total x numTasks
  beta <- matrix(0, nrow = p_total, ncol = numTasks)
  rownames(beta) <- feature_names
  colnames(beta) <- as.character(thresholds)

  # Select which features are signals
  all_feature_idx <- seq_len(p)  # exclude treatment (handled separately)
  signal_idx <- sort(sample(all_feature_idx, n_signals))
  signal_names <- feature_names[signal_idx]
  null_idx <- setdiff(all_feature_idx, signal_idx)
  null_names <- feature_names[null_idx]

  # Assign templates cyclically with random sign
  effect_types <- character(n_signals)
  names(effect_types) <- signal_names

  for (s in seq_len(n_signals)) {
    template_choice <- template_names[((s - 1) %% length(template_names)) + 1]
    sign_choice <- sample(c(-1, 1), 1)

    beta[signal_idx[s], ] <- sign_choice * effect_strength *
      template_library[[template_choice]]

    effect_types[s] <- template_choice
  }

  # Treatment effect: constant across thresholds
  beta["treatment", ] <- treatment_effect

  # ---------------------------
  # Generate the response (mode-specific)
  # ---------------------------
  SurvTime <- NULL
  Event    <- NULL
  Y        <- NULL

  if (mode == "regression") {
    # Multi-task linear response with Gaussian noise.
    Y <- X %*% beta + matrix(rnorm(n * numTasks, sd = noise_sd),
                             nrow = n, ncol = numTasks)
    colnames(Y) <- as.character(thresholds)
  } else if (mode == "classification") {
    # Bernoulli labels in {-1, +1} from the logistic link.
    prob <- 1 / (1 + exp(-(X %*% beta)))
    Y <- matrix(ifelse(matrix(runif(n * numTasks), n, numTasks) < prob,
                       1, -1),
                nrow = n, ncol = numTasks)
    colnames(Y) <- as.character(thresholds)
  } else {
    # mode == "survival": piecewise-exponential times (historical path).
    Y <- .simulate_survival_times(
      X = X, beta = beta, n = n, numTasks = numTasks,
      thresholds = thresholds, baseline_hazard = baseline_hazard,
      censoring_max = censoring_max
    )
    SurvTime <- Y$SurvTime
    Event    <- Y$Event
    Y        <- NULL
  }

  # ---------------------------
  # Build ground truth
  # ---------------------------
  ground_truth <- list(
    coefficients    = beta,
    signal_features = signal_names,
    null_features   = null_names,
    effect_types    = effect_types,
    thresholds      = thresholds,
    baseline_hazard = baseline_hazard
  )

  # ---------------------------
  # Return S3 object
  # ---------------------------
  structure(
    list(
      mode           = mode,
      X              = X,
      Y              = Y,
      SurvTime       = SurvTime,
      Event          = Event,
      treatment      = X[, "treatment"],
      feature_names  = feature_names,
      ground_truth   = ground_truth,
      n              = n,
      p              = p,
      n_signals      = n_signals,
      n_continuous   = n_continuous,
      thresholds     = thresholds,
      seed           = seed,
      call           = cl
    ),
    class = "simulated_mtl"
  )
}


# ---------------------------------------------------------------------
# Internal: piecewise-exponential survival time generator (SIM-01)
#
# Factored out of simulate_mtl so the survival path is unchanged while
# regression/classification responses are produced inline. Returns a
# list(SurvTime, Event).
# ---------------------------------------------------------------------
.simulate_survival_times <- function(X, beta, n, numTasks, thresholds,
                                     baseline_hazard, censoring_max) {

  # Interval boundaries: [0, t1), [t1, t2), ..., [t_{K-1}, t_K), [t_K, Inf)
  interval_bounds <- c(0, thresholds)
  n_intervals <- length(interval_bounds)  # includes the tail interval

  SurvTime_true <- numeric(n)

  for (i in seq_len(n)) {
    cumulative_time <- 0
    event_occurred <- FALSE

    for (k in seq_len(n_intervals)) {
      # Coefficients for this interval
      # Use the threshold index: intervals 1..numTasks use beta[,k],
      # the tail interval (k = numTasks + 1) reuses beta[, numTasks]
      beta_k <- if (k <= numTasks) beta[, k] else beta[, numTasks]

      # Hazard for this patient in this interval
      linear_predictor <- sum(X[i, ] * beta_k)
      hazard <- baseline_hazard * exp(linear_predictor)

      # Time to event in this interval (exponential)
      time_in_interval <- rexp(1, rate = hazard)

      # Interval width (Inf for the tail)
      if (k < n_intervals) {
        interval_width <- interval_bounds[k + 1] - interval_bounds[k]
      } else {
        interval_width <- Inf
      }

      if (time_in_interval < interval_width) {
        # Event occurs in this interval
        cumulative_time <- cumulative_time + time_in_interval
        event_occurred <- TRUE
        break
      } else {
        # Survive this interval, move to next
        cumulative_time <- cumulative_time + interval_width
      }
    }

    # If no event in any interval (extremely unlikely but possible)
    if (!event_occurred) {
      cumulative_time <- cumulative_time + rexp(1, rate = baseline_hazard)
    }

    SurvTime_true[i] <- cumulative_time
  }

  # ---------------------------
  # Apply censoring
  # ---------------------------
  censor_time <- runif(n, min = 0, max = censoring_max)
  SurvTime <- pmin(SurvTime_true, censor_time)
  Event <- as.numeric(SurvTime_true <= censor_time)

  list(SurvTime = SurvTime, Event = Event)
}


# ---------------------------
# Internal: build effect templates
# ---------------------------

#' Build a normalised effect template vector
#'
#' @param type Template type: "early", "late", "constant",
#'   "increasing", "decreasing".
#' @param n_tasks Number of tasks/thresholds.
#'
#' @return Numeric vector of length \code{n_tasks}, normalised so
#'   \code{max(abs(values)) == 1}.
#'
#' @keywords internal
.make_template <- function(type, n_tasks) {

  idx <- seq_len(n_tasks)
  scaled <- (idx - 1) / max(1, n_tasks - 1)  # 0 to 1

  template <- switch(type,
                     early      = rev(scaled),
                     late       = scaled,
                     constant   = rep(1, n_tasks),
                     increasing = scaled,
                     decreasing = rev(scaled),
                     switch     = 1 - 2 * (scaled ^ 0.5),   # crosses zero early
                     stop("Unknown template type: ", type, call. = FALSE)
  )

  # Normalise to max |value| = 1
  max_abs <- max(abs(template))
  if (max_abs > 0) {
    template <- template / max_abs
  }

  return(template)
}

#' Create Diffusion Constraint Matrix
#'
#' Builds a square constraint matrix \code{K} encoding the prior belief
#' that temporally distant tasks should have more orthogonal coefficients.
#' Off-diagonal entries accumulate weight proportional to the distance
#' between task indices, implementing a diffusion-like pattern.
#'
#' This matrix is passed to \code{\link{orthoMTL}} via the \code{K}
#' argument to control the orthogonality penalty between tasks.
#'
#' @param numTasks An integer specifying the number of tasks.
#' @param diag_val A numeric value for the diagonal of \code{K}.
#'   Default: \code{0.5}. This value is typically overridden during
#'   cross-validation (see \code{\link{cv_orthoMTL}}).
#'
#' @return A numeric square matrix of dimensions
#'   \code{numTasks x numTasks}. Off-diagonal entry \code{K[i,j]}
#'   is larger when tasks \code{i} and \code{j} are further apart.
#'   Diagonal entries are set to \code{diag_val}.
#'
#' @details
#' The construction rule: for each distance level \code{h} from
#' \code{1} to \code{numTasks}, add \code{1} to all entries where
#' \code{|row - col| > h}. This produces a matrix where nearby tasks
#' (adjacent thresholds) share more support, while distant tasks
#' are pushed toward orthogonality.
#'
#' For survival analysis with time thresholds, this encodes the
#' assumption that the set of predictive features changes gradually
#' over time rather than abruptly.
#'
#' @seealso \code{\link{cv_orthoMTL}}
#'
#' @export
#'
#' @examples
#' # 5-task constraint matrix
#' K <- create_constraint_matrix(5)
#' K
#'
#' # Override diagonal for a specific penalty balance
#' K <- create_constraint_matrix(7, diag_val = 6)
#' K
create_constraint_matrix <- function(numTasks, diag_val = 0.5) {

  K <- matrix(0, nrow = numTasks, ncol = numTasks)
  delta <- row(K) - col(K)

  # Accumulate off-diagonal weights by distance
  for (high in seq_len(numTasks)) {
    K[abs(delta) > high] <- K[abs(delta) > high] + 1
  }

  # Set diagonal
  diag(K) <- diag_val

  return(K)
}

# Tests for create_constraint_matrix

test_that("create_constraint_matrix has correct dimensions", {
  K <- create_constraint_matrix(5)

  expect_true(is.matrix(K))
  expect_equal(dim(K), c(5, 5))
})

test_that("create_constraint_matrix is symmetric", {
  K <- create_constraint_matrix(7)

  expect_equal(K, t(K))
})

test_that("create_constraint_matrix diagonal uses diag_val", {
  K1 <- create_constraint_matrix(4, diag_val = 0.5)
  K2 <- create_constraint_matrix(4, diag_val = 3.0)

  expect_equal(diag(K1), rep(0.5, 4))
  expect_equal(diag(K2), rep(3.0, 4))
})

test_that("create_constraint_matrix off-diagonals increase with distance", {
  K <- create_constraint_matrix(5)

  # Adjacent tasks should have smaller off-diagonal than distant tasks
  expect_true(K[1, 2] < K[1, 4])
  expect_true(K[1, 2] < K[1, 5])
  expect_true(K[2, 3] < K[2, 5])
})

test_that("create_constraint_matrix handles 2 tasks", {
  K <- create_constraint_matrix(2)

  expect_equal(dim(K), c(2, 2))
  expect_equal(diag(K), c(0.5, 0.5))
})

#' @title Projection onto Disjoint Support Feasible Set
#' @description Projects weight and constraint matrices ...
#' @param w Numeric matrix of weights used in the loss function.
#' @param v Numeric matrix of constraints used for projection.
#' @return A list with projected matrices \code{w} and \code{v}.
#' @keywords internal
proj_disjoint <- function(w, v) {
  w_tmp = w
  v_tmp = v
  idx1 = v <= 0
  v_tmp[idx1] = 0
  w_tmp[idx1] = 0
  abs_w = abs(w)
  idx2 = abs_w < v
  idx3 = !(idx1 | idx2)
  tmp = 0.5 * (v[idx3] + abs_w[idx3])
  v_tmp[idx3] = tmp
  w_tmp[idx3] = sign(w[idx3]) * tmp
  return(list(w = w_tmp, v = v_tmp))
}

# Tests for proj_disjoint (internal function)

test_that("proj_disjoint returns list with w and v", {
  w <- matrix(rnorm(12), 4, 3)
  v <- matrix(abs(rnorm(12)), 4, 3)

  result <- orthoMTL:::proj_disjoint(w, v)

  expect_true(is.list(result))
  expect_true("w" %in% names(result))
  expect_true("v" %in% names(result))
  expect_equal(dim(result$w), dim(w))
  expect_equal(dim(result$v), dim(v))
})

test_that("proj_disjoint projects negative v to zero", {
  w <- matrix(c(1, 2, 3, 4), 2, 2)
  v <- matrix(c(-1, 2, -3, 4), 2, 2)

  result <- orthoMTL:::proj_disjoint(w, v)

  # Where v was negative, both should be 0
  expect_equal(result$v[1, 1], 0)
  expect_equal(result$w[1, 1], 0)
  expect_equal(result$v[1, 2], 0)
  expect_equal(result$w[1, 2], 0)
})

test_that("proj_disjoint v is non-negative after projection", {
  set.seed(42)
  w <- matrix(rnorm(20), 5, 4)
  v <- matrix(rnorm(20), 5, 4)

  result <- orthoMTL:::proj_disjoint(w, v)

  expect_true(all(result$v >= 0))
})

#' Cross-Validation for orthoMTL Hyperparameter Selection
#'
#' Performs a parallelised grid search over hyperparameters for
#' \code{\link{orthoMTL}}, evaluating each configuration via
#' cross-validated concordance index. Returns the best configuration
#' without retraining a final model (that is the caller's responsibility).
#'
#' @param X.train A numeric matrix of training features with dimensions
#'   \code{n x p}.
#' @param Y.train A numeric matrix of training labels with dimensions
#'   \code{n x numTasks}. May contain \code{NA} for censored observations.
#' @param W.train A numeric indicator matrix of dimensions
#'   \code{n x numTasks}, where \code{1} = observed and \code{0} = censored.
#'   Required when \code{survival = TRUE}.
#' @param K A square constraint matrix of dimensions
#'   \code{numTasks x numTasks}. The diagonal will be overridden by
#'   values in \code{diag_vals} during the search.
#' @param lambdas A numeric vector of regularisation parameters to search.
#' @param alphas A numeric vector of elastic-net mixing parameters in
#'   \eqn{[0, 1]} to search. Default: \code{0} (pure orthogonality
#'   penalty, no sparsity).
#' @param stepsizes A numeric vector of gradient descent step sizes
#'   to search.
#' @param diag_vals A numeric vector of diagonal values for the
#'   constraint matrix \code{K} to search.
#' @param survival Logical. Use censored survival loss? Default:
#'   \code{TRUE}.
#' @param logistic Logical. Fit logistic (classification) models?
#'   Passed through to \code{\link{orthoMTL}}. Default: \code{FALSE}.
#' @param metric Character; the scoring metric maximised/minimised over
#'   the grid, or \code{NULL} (default) to choose automatically by mode:
#'   \code{"cindex"} when \code{survival = TRUE}, \code{"auc"} when
#'   \code{logistic = TRUE}, otherwise \code{"rmse"}. Supported values:
#'   \code{"cindex"}, \code{"auc"}, \code{"accuracy"} (higher is better),
#'   \code{"rmse"} (lower is better), \code{"r2"} (higher is better).
#' @param disjoint Logical. Enforce disjoint supports? Default:
#'   \code{FALSE}.
#' @param schedule Character; the gradient-step decay schedule passed to
#'   \code{\link{orthoMTL}} for every fit. One of \code{"sqrt"} (default),
#'   \code{"log"}, \code{"const"}, \code{"linear"}. \code{"log"} or
#'   \code{"const"} typically reach the same optimum in far fewer iterations
#'   than the default \code{"sqrt"}, which can noticeably speed up the grid
#'   search; see the \code{schedule} argument of \code{\link{orthoMTL}} for the
#'   trade-offs. Applied uniformly to all configurations (it is not part of the
#'   tuning grid).
#' @param folds An integer vector of length \code{n} assigning each
#'   training observation to a fold. If \code{NULL}, a 5-fold
#'   assignment is generated with a warning.
#' @param n_cores Integer. Number of cores for parallel execution.
#'   Default: \code{2}.
#' @param seed Optional integer random seed for reproducibility. Default:
#'   \code{NULL} (no seed is set; the ambient RNG state is used as-is).
#'   Used both for auto-generated fold assignment and for each
#'   \code{\link{orthoMTL}} fit in the grid.
#' @param verbose Logical. Print progress information? Default:
#'   \code{TRUE}.
#'
#' @return An object of class \code{"cv_orthoMTL"} containing:
#'   \describe{
#'     \item{best}{A list with the best hyperparameters: \code{lambda},
#'       \code{alpha}, \code{stepsize}, \code{diag_val}, and the
#'       corresponding \code{cv_score}.}
#'     \item{results}{A \code{data.frame} of all configurations with
#'       their mean CV C-index, sorted descending by \code{cv_score}.}
#'     \item{folds}{The fold assignment vector used.}
#'     \item{n_configs}{Total number of configurations tested.}
#'     \item{n_folds}{Number of unique folds.}
#'     \item{call}{The matched function call.}
#'   }
#'
#' @details
#' The grid is constructed as the full Cartesian product of
#' \code{lambdas}, \code{alphas}, \code{stepsizes}, and
#' \code{diag_vals}. Each configuration is evaluated independently
#' in parallel across cores. Within each configuration, folds are
#' evaluated sequentially and the per-fold C-indices are averaged.
#'
#' The best configuration is selected by joint maximisation of the
#' mean CV C-index over the entire flattened grid (not greedy
#' sequential search).
#'
#' This function does \emph{not} retrain a final model. Use the
#' returned hyperparameters to train via \code{\link{orthoMTL}}.
#'
#' @seealso \code{\link{orthoMTL}}, \code{\link{cindex_mtl}}
#'
#' @importFrom foreach foreach %dopar%
#' @importFrom doParallel registerDoParallel
#' @importFrom parallel makeCluster stopCluster
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' n <- 50; p <- 5; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' SurvTime <- rexp(n, rate = 0.1)
#' Event <- rbinom(n, 1, 0.7)
#' thresholds <- c(4, 6, 10)
#'
#' Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
#' W <- create_indicator_matrix(Y)
#' K <- create_constraint_matrix(n_tasks)
#'
#' folds <- rep(1:2, length.out = n)
#'
#' cv_res <- cv_orthoMTL(
#'   X.train = X, Y.train = Y, W.train = W, K = K,
#'   lambdas = c(1e-3, 1e-2), alphas = 0,
#'   stepsizes = c(0.1), diag_vals = c(0.5, 1),
#'   survival = TRUE, disjoint = FALSE,
#'   folds = folds, n_cores = 1, seed = 42, verbose = FALSE
#' )
#'
#' print(cv_res)
#' cv_res$best
#' }
cv_orthoMTL <- function(X.train, Y.train, W.train = NULL,
                        K = NULL,
                        lambdas = c(1e-3, 1e-2),
                        alphas = 0,
                        stepsizes = c(0.1, 0.5),
                        diag_vals = c(0.5, 1),
                        survival = TRUE,
                        logistic = FALSE,
                        metric = NULL,
                        disjoint = FALSE,
                        schedule = c("sqrt", "log", "const", "linear"),
                        folds = NULL,
                        n_cores = 2,
                        seed = NULL,
                        verbose = TRUE) {

  cl <- match.call()
  schedule <- match.arg(schedule)

  # ---------------------------
  # Input validation
  # ---------------------------
  if (is.null(X.train) || is.null(Y.train)) {
    stop("X.train and Y.train must be provided.", call. = FALSE)
  }
  if (nrow(X.train) != nrow(Y.train)) {
    stop("X.train and Y.train must have the same number of rows.",
         call. = FALSE)
  }
  if (survival && is.null(W.train)) {
    stop("W.train (censoring indicator) must be provided when ",
         "survival = TRUE.", call. = FALSE)
  }
  if (survival && !is.null(W.train)) {
    if (nrow(W.train) != nrow(X.train) || ncol(W.train) != ncol(Y.train)) {
      stop("W.train dimensions must match Y.train.", call. = FALSE)
    }
  }

  numTasks <- ncol(Y.train)

  # Default K

  if (is.null(K)) {
    K <- diag(1, numTasks)
  }

  # ---------------------------
  # Fold assignment
  # ---------------------------
  if (is.null(folds)) {
    warning("Cross-validation fold assignment not provided. ",
            "Generating 5-fold assignment.", call. = FALSE)
    if (!is.null(seed)) set.seed(seed)
    folds <- sample(1:5, nrow(X.train), replace = TRUE)
  }
  if (length(folds) != nrow(X.train)) {
    stop("Length of 'folds' (", length(folds), ") must equal nrow(X.train) (",
         nrow(X.train), ").", call. = FALSE)
  }
  unique_folds <- sort(unique(folds))
  n_folds <- length(unique_folds)

  # ---------------------------
  # Elastic-net mixing validation
  # ---------------------------
  if (any(alphas < 0 | alphas > 1)) {
    stop("All 'alphas' must lie in [0, 1].", call. = FALSE)
  }

  # ---------------------------
  # Resolve scoring metric (mode-aware default)
  # ---------------------------
  scorer <- .resolve_cv_metric(metric, survival = survival,
                               logistic = logistic)

  # ---------------------------
  # Build configuration grid
  # ---------------------------
  config_grid <- expand.grid(
    lambda   = lambdas,
    alpha    = alphas,
    stepsize = stepsizes,
    diag_val = diag_vals,
    stringsAsFactors = FALSE
  )
  n_configs <- nrow(config_grid)

  if (verbose) {
    total_fits <- n_configs * n_folds
    cat("Cross-validation for orthoMTL\n")
    cat("  Configurations:", n_configs, "\n")
    cat("  Folds:", n_folds, "\n")
    cat("  Total model fits:", total_fits, "\n")
    cat("  Cores:", n_cores, "\n")
    cat("  Running...\n")
  }

  # ---------------------------
  # Parallel setup
  # ---------------------------
  if (n_cores > 1L) {
    cl_parallel <- parallel::makeCluster(n_cores)
    doParallel::registerDoParallel(cl_parallel)
    # Workers are fresh R sessions (PSOCK on Windows); ensure they can locate
    # orthoMTL even when it lives in a temporary library (e.g. during R CMD
    # check or load_all) by inheriting the parent's library paths.
    parallel::clusterCall(cl_parallel, function(p) .libPaths(p), .libPaths())
    # Ensure cluster is stopped on exit (even on error)
    on.exit(parallel::stopCluster(cl_parallel), add = TRUE)
  } else {
    # Single core: run sequentially in-process. Avoids spawning a PSOCK worker
    # that cannot library(orthoMTL) under R CMD check (the Windows-only
    # vignette build failure).
    foreach::registerDoSEQ()
  }

  # ---------------------------
  # Main CV loop (parallel over configs)
  # ---------------------------
  cv_scores <- foreach::foreach(
    cfg_idx = seq_len(n_configs),
    .combine  = "c",
    .packages = "orthoMTL"
  ) %dopar% {

    cfg <- config_grid[cfg_idx, ]

    # Local copy of K with this config's diagonal
    K_local <- K
    diag(K_local) <- cfg$diag_val

    # Sequential fold loop
    fold_scores <- numeric(n_folds)

    for (f_idx in seq_along(unique_folds)) {
      fold_id <- unique_folds[f_idx]
      idx_train <- which(folds != fold_id)
      idx_val   <- which(folds == fold_id)

      # Fit model on training folds
      fit <- orthoMTL(
        X            = X.train[idx_train, , drop = FALSE],
        Y            = Y.train[idx_train, , drop = FALSE],
        lambda       = cfg$lambda,
        alpha        = cfg$alpha,
        step_size    = cfg$stepsize,
        K            = K_local,
        disjoint     = disjoint,
        survival     = survival,
        logistic     = logistic,
        schedule     = schedule,
        censored.mat = if (survival) W.train[idx_train, , drop = FALSE] else NULL,
        seed         = seed,
        verbose      = 0
      )

      # Predict on validation fold
      X_val <- X.train[idx_val, , drop = FALSE]
      Y_val <- Y.train[idx_val, , drop = FALSE]

      pred_val <- X_val %*% fit$B

      # Apply monotonicity projection for survival
      if (survival) {
        pred_val <- t(apply(pred_val, 1, nnmaxheap_C))
      }

      # Score on validation fold with the resolved metric
      fold_scores[f_idx] <- scorer$fn(Y_val, pred_val)
    }

    # Return mean CV score for this config
    mean(fold_scores, na.rm = TRUE)
  }

  # ---------------------------
  # Collect results
  # ---------------------------
  config_grid$cv_score <- cv_scores

  # Joint optimum over flattened grid (direction depends on the metric)
  best_idx <- if (scorer$maximize) {
    which.max(config_grid$cv_score)
  } else {
    which.min(config_grid$cv_score)
  }
  best_cfg <- config_grid[best_idx, ]

  if (verbose) {
    cat("  Done.\n")
    cat("  Best CV ", scorer$name, ": ",
        format(best_cfg$cv_score, digits = 4), "\n", sep = "")
  }

  # Check for failed fits. NA always indicates a failure; an exact 0 is
  # suspicious only for the "higher-is-better" metrics (a 0 RMSE is perfect).
  n_failed <- sum(is.na(cv_scores) |
                    (scorer$maximize & cv_scores == 0))
  if (n_failed > 0) {
    warning(n_failed, " of ", n_configs,
            " configurations returned NA or a degenerate score. ",
            "This may indicate failed convergence or degenerate folds.",
            call. = FALSE)
  }

  # Sort results best-first (direction depends on the metric)
  config_grid <- config_grid[order(config_grid$cv_score,
                                   decreasing = scorer$maximize), ]
  rownames(config_grid) <- NULL

  # ---------------------------
  # Build return object
  # ---------------------------
  structure(
    list(
      best = list(
        lambda   = best_cfg$lambda,
        alpha    = best_cfg$alpha,
        stepsize = best_cfg$stepsize,
        diag_val = best_cfg$diag_val,
        cv_score = best_cfg$cv_score
      ),
      results   = config_grid,
      folds     = folds,
      n_configs = n_configs,
      n_folds   = n_folds,
      metric    = scorer$name,
      call      = cl
    ),
    class = "cv_orthoMTL"
  )
}


# ---------------------------------------------------------------------
# Internal: resolve the CV scoring metric (GEN-02)
#
# Returns a list with the metric name, the scoring function fn(true,
# pred), and `maximize` (TRUE if higher is better). When `metric` is
# NULL the default is chosen from the fitting mode.
# ---------------------------------------------------------------------
.resolve_cv_metric <- function(metric, survival, logistic) {
  if (is.null(metric)) {
    metric <- if (survival) {
      "cindex"
    } else if (logistic) {
      "auc"
    } else {
      "rmse"
    }
  }
  metric <- match.arg(metric,
                      c("cindex", "auc", "accuracy", "rmse", "r2"))

  switch(metric,
    cindex   = list(name = "cindex",   fn = cindex_mtl,   maximize = TRUE),
    auc      = list(name = "auc",      fn = auc_mtl,      maximize = TRUE),
    accuracy = list(name = "accuracy", fn = accuracy_mtl, maximize = TRUE),
    r2       = list(name = "r2",       fn = r2_mtl,       maximize = TRUE),
    rmse     = list(name = "rmse",     fn = rmse_mtl,     maximize = FALSE)
  )
}

#' Print Cross-Validation Results
#'
#' Displays a summary of cross-validation results from
#' \code{\link{cv_orthoMTL}}.
#'
#' @param x An object of class \code{"cv_orthoMTL"}.
#' @param n_top Number of top configurations to display. Default: 5.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @method print cv_orthoMTL
#' @export
#'
#' @examples
#' # See ?cv_orthoMTL for a full example
print.cv_orthoMTL <- function(x, n_top = 5, ...) {

  cat("Cross-validation results for orthoMTL\n")
  cat("Configurations tested:", x$n_configs, "\n")
  cat("Folds:", x$n_folds, "\n")

  cat("---\n")

  cat("Best configuration:\n")
  cat("  lambda   =", x$best$lambda, "\n")
  cat("  alpha    =", x$best$alpha, "\n")
  cat("  stepsize =", x$best$stepsize, "\n")
  cat("  diag_val =", x$best$diag_val, "\n")
  metric_label <- if (!is.null(x$metric)) x$metric else "C-index"
  cat("  CV ", metric_label, " = ",
      format(x$best$cv_score, digits = 4), "\n", sep = "")

  cat("---\n")

  n_show <- min(n_top, nrow(x$results))
  cat("Top", n_show, "configurations:\n")
  print(x$results[seq_len(n_show), ], row.names = FALSE)

  invisible(x)
}

#' Bootstrap Inference for orthoMTL Coefficients
#'
#' Estimates coefficient variability and statistical relevance by
#' comparing bootstrapped models (real signal) against null models
#' (permuted outcomes). This two-pronged approach answers: (1) how
#' stable is each coefficient across resamples, and (2) is each
#' coefficient distinguishable from what would be obtained by chance.
#'
#' @param X A numeric matrix of predictor variables with dimensions
#'   \code{n x p}.
#' @param Y A numeric matrix of response labels with dimensions
#'   \code{n x numTasks}. May contain \code{NA} for censored
#'   observations.
#' @param lambda Regularisation parameter for the orthogonal penalty.
#' @param alpha Elastic-net mixing parameter in \eqn{[0, 1]}, passed
#'   through to \code{\link{orthoMTL}}. Default: \code{0}.
#' @param step_size Step size for gradient descent. Default: \code{0.1}.
#' @param K Constraint matrix of dimensions
#'   \code{numTasks x numTasks}. Default: \code{NULL} (identity).
#' @param disjoint Logical. Enforce disjoint supports? Default:
#'   \code{FALSE}.
#' @param schedule Character; the gradient-step decay schedule passed to
#'   \code{\link{orthoMTL}} for every resample/permutation fit. One of
#'   \code{"sqrt"} (default), \code{"log"}, \code{"const"}, \code{"linear"}.
#'   Because bootstrap refits the model many times, switching to \code{"log"} or
#'   \code{"const"} -- which reach the same optimum in far fewer iterations --
#'   can substantially cut total runtime; see the \code{schedule} argument of
#'   \code{\link{orthoMTL}} for details.
#' @param survival Logical. Use censored survival loss? Default:
#'   \code{FALSE}.
#' @param censored.mat A numeric indicator matrix of dimensions
#'   \code{n x numTasks}. Required when \code{survival = TRUE}.
#' @param n_repeats Number of bootstrap/permutation repeats. Default:
#'   \code{100}.
#' @param n_cores Number of cores for parallel execution. Default:
#'   \code{2}.
#' @param seed Optional base random seed. Default: \code{NULL} (no seed
#'   is set; repeats vary via the ambient RNG state and the run is not
#'   reproducible). If supplied, repeat \code{i} uses
#'   \code{set.seed(seed + i)} for \code{i} in \code{1:n_repeats}, so the
#'   whole run becomes reproducible while still varying across repeats.
#' @param verbose Logical. Print progress information? Default:
#'   \code{TRUE}.
#'
#' @return An object of class \code{"bootstrap_orthoMTL"} containing:
#'   \describe{
#'     \item{results}{A tidy \code{data.frame} with columns \code{id}
#'       (feature name), \code{time} (task/threshold), \code{coeff}
#'       (coefficient value), \code{group} (\code{"real"} or
#'       \code{"null"}), and \code{repeat_id} (integer).}
#'     \item{coefficients_real}{A list of \code{n_repeats} coefficient
#'       matrices (each \code{p x numTasks}).}
#'     \item{coefficients_null}{A list of \code{n_repeats} coefficient
#'       matrices from permuted outcomes.}
#'     \item{obj_real}{Numeric vector of final objective values for
#'       real bootstrap models.}
#'     \item{obj_null}{Numeric vector of final objective values for
#'       null permutation models.}
#'     \item{n_repeats}{Number of repeats.}
#'     \item{n_features}{Number of features.}
#'     \item{n_tasks}{Number of tasks.}
#'     \item{feature_names}{Character vector of feature names.}
#'     \item{task_names}{Character vector of task names.}
#'     \item{call}{The matched function call.}
#'   }
#'
#' @details
#' \strong{Real bootstrap}: For each repeat, rows of \code{X}, \code{Y},
#' and \code{censored.mat} are resampled \emph{with replacement}. The
#' model is refit with identical hyperparameters. This produces a
#' distribution of coefficient values reflecting estimation variability.
#'
#' \strong{Null permutation}: For each repeat, rows of \code{Y} and
#' \code{censored.mat} are permuted \emph{without replacement} while
#' \code{X} remains fixed. This breaks the association between features
#' and outcomes, producing a null distribution of coefficients.
#'
#' Comparing real vs null distributions for each feature and task
#' indicates whether observed coefficients are distinguishable from
#' noise. Visualise with \code{\link{plot_bootstrap}}.
#'
#' @seealso \code{\link{orthoMTL}}, \code{\link{plot_bootstrap}}
#'
#' @importFrom foreach foreach %dopar%
#' @importFrom doParallel registerDoParallel
#' @importFrom parallel makeCluster stopCluster
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' n <- 30; p <- 5; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' SurvTime <- rexp(n, rate = 0.1)
#' Event <- rbinom(n, 1, 0.7)
#' thresholds <- c(4, 6, 10)
#'
#' Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
#' W <- create_indicator_matrix(Y)
#' K <- create_constraint_matrix(n_tasks)
#'
#' boot_res <- bootstrap_orthoMTL(
#'   X = X, Y = Y, lambda = 1e-3, step_size = 0.1,
#'   K = K, survival = TRUE, censored.mat = W,
#'   n_repeats = 5, n_cores = 1, verbose = FALSE
#' )
#'
#' print(boot_res)
#' head(boot_res$results)
#' }
bootstrap_orthoMTL <- function(X, Y,
                               lambda = 1,
                               alpha = 0,
                               step_size = 0.1,
                               K = NULL,
                               disjoint = FALSE,
                               schedule = c("sqrt", "log", "const", "linear"),
                               survival = FALSE,
                               censored.mat = NULL,
                               n_repeats = 100,
                               n_cores = 2,
                               seed = NULL,
                               verbose = TRUE) {

  cl <- match.call()
  schedule <- match.arg(schedule)

  # ---------------------------
  # Input validation
  # ---------------------------
  if (is.null(X) || is.null(Y)) {
    stop("X and Y must be provided.", call. = FALSE)
  }
  if (nrow(X) != nrow(Y)) {
    stop("X and Y must have the same number of rows.", call. = FALSE)
  }
  if (survival && is.null(censored.mat)) {
    stop("censored.mat must be provided when survival = TRUE.",
         call. = FALSE)
  }
  if (n_repeats < 1) {
    stop("n_repeats must be at least 1.", call. = FALSE)
  }

  n <- nrow(X)
  p <- ncol(X)
  numTasks <- ncol(Y)
  feature_names <- colnames(X)
  task_names <- colnames(Y)

  if (is.null(feature_names)) {
    feature_names <- paste0("V", seq_len(p))
  }
  if (is.null(task_names)) {
    task_names <- paste0("T", seq_len(numTasks))
  }

  # ---------------------------
  # Parallel setup
  # ---------------------------
  if (n_cores > 1L) {
    cl_parallel <- parallel::makeCluster(n_cores)
    doParallel::registerDoParallel(cl_parallel)
    # Workers are fresh R sessions (PSOCK on Windows); ensure they can locate
    # orthoMTL even when it lives in a temporary library (e.g. during R CMD
    # check or load_all) by inheriting the parent's library paths.
    parallel::clusterCall(cl_parallel, function(p) .libPaths(p), .libPaths())
    on.exit(parallel::stopCluster(cl_parallel), add = TRUE)
  } else {
    # Single core: run sequentially in-process. Avoids spawning a PSOCK worker
    # that cannot library(orthoMTL) under R CMD check (the Windows-only
    # vignette build failure).
    foreach::registerDoSEQ()
  }

  # ---------------------------
  # Phase 1: Real bootstrap
  # ---------------------------
  if (verbose) {
    cat("Bootstrap inference for orthoMTL\n")
    cat("  Features:", p, "| Tasks:", numTasks, "| Repeats:", n_repeats, "\n")
    cat("  Cores:", n_cores, "\n")
    cat("  Phase 1/2: Real bootstrap (resampling with replacement)...\n")
  }

  real_results <- foreach::foreach(
    i = seq_len(n_repeats),
    .packages = "orthoMTL"
  ) %dopar% {

    if (!is.null(seed)) set.seed(seed + i)
    idx <- sample(x = seq_len(n), size = n, replace = TRUE)

    fit <- orthoMTL(
      X            = X[idx, , drop = FALSE],
      Y            = Y[idx, , drop = FALSE],
      lambda       = lambda,
      alpha        = alpha,
      step_size    = step_size,
      K            = K,
      disjoint     = disjoint,
      schedule     = schedule,
      survival     = survival,
      censored.mat = if (survival) censored.mat[idx, , drop = FALSE] else NULL,
      seed         = if (!is.null(seed)) seed + i else NULL,
      verbose      = 0
    )

    Bt <- fit$B
    colnames(Bt) <- task_names
    rownames(Bt) <- feature_names

    list(B = Bt, obj = fit$obj)
  }

  # Unpack real results
  coefficients_real <- lapply(real_results, `[[`, "B")
  obj_real <- vapply(real_results, `[[`, numeric(1), "obj")

  if (verbose) cat("  Done.\n")

  # ---------------------------
  # Phase 2: Null permutation
  # ---------------------------
  if (verbose) {
    cat("  Phase 2/2: Null permutation (shuffled outcomes)...\n")
  }

  null_results <- foreach::foreach(
    i = seq_len(n_repeats),
    .packages = "orthoMTL"
  ) %dopar% {

    if (!is.null(seed)) set.seed(seed + i)
    idx <- sample(x = seq_len(n), size = n, replace = FALSE)

    fit <- orthoMTL(
      X            = X,
      Y            = Y[idx, , drop = FALSE],
      lambda       = lambda,
      alpha        = alpha,
      step_size    = step_size,
      K            = K,
      disjoint     = disjoint,
      schedule     = schedule,
      survival     = survival,
      censored.mat = if (survival) censored.mat[idx, , drop = FALSE] else NULL,
      seed         = if (!is.null(seed)) seed + i else NULL,
      verbose      = 0
    )

    Bt <- fit$B
    colnames(Bt) <- task_names
    rownames(Bt) <- feature_names

    list(B = Bt, obj = fit$obj)
  }

  # Unpack null results
  coefficients_null <- lapply(null_results, `[[`, "B")
  obj_null <- vapply(null_results, `[[`, numeric(1), "obj")

  if (verbose) cat("  Done.\n")

  # ---------------------------
  # Build tidy results data.frame
  # ---------------------------
  if (verbose) cat("  Assembling results...\n")

  results_list <- vector("list", length(feature_names))

  for (f_idx in seq_along(feature_names)) {
    feature <- feature_names[f_idx]

    # Real coefficients: n_repeats x numTasks matrix
    real_mat <- do.call("rbind", lapply(coefficients_real,
                                        function(B) B[feature, ]))

    real_df <- data.frame(
      id        = feature,
      time      = rep(task_names, each = n_repeats),
      coeff     = as.vector(real_mat),
      group     = "real",
      repeat_id = rep(seq_len(n_repeats), times = numTasks),
      stringsAsFactors = FALSE
    )

    # Null coefficients: n_repeats x numTasks matrix
    null_mat <- do.call("rbind", lapply(coefficients_null,
                                        function(B) B[feature, ]))

    null_df <- data.frame(
      id        = feature,
      time      = rep(task_names, each = n_repeats),
      coeff     = as.vector(null_mat),
      group     = "null",
      repeat_id = rep(seq_len(n_repeats), times = numTasks),
      stringsAsFactors = FALSE
    )

    results_list[[f_idx]] <- rbind(real_df, null_df)
  }

  results <- do.call("rbind", results_list)
  rownames(results) <- NULL

  if (verbose) cat("  Complete.\n")

  # ---------------------------
  # Return S3 object
  # ---------------------------
  structure(
    list(
      results           = results,
      coefficients_real = coefficients_real,
      coefficients_null = coefficients_null,
      obj_real          = obj_real,
      obj_null          = obj_null,
      n_repeats         = n_repeats,
      n_features        = p,
      n_tasks           = numTasks,
      feature_names     = feature_names,
      task_names        = task_names,
      call              = cl
    ),
    class = "bootstrap_orthoMTL"
  )
}

#' orthoMTL: Multi-Task Learning with Orthogonal Constraints
#'
#' @description
#' Fits regularised multi-task learning models where relationships between
#' tasks are controlled via orthogonality or disjoint-support constraints.
#' Supports regression, classification, and censored survival data.
#'
#' @section Overview:
#' orthoMTL implements a penalised multi-task learning framework where the
#' columns of the coefficient matrix are encouraged to be orthogonal (or
#' have disjoint supports). In survival mode, time-to-event data is
#' converted into binary classification tasks at user-defined thresholds,
#' with censored observations masked in the loss computation.
#'
#' The package is the successor to the \code{orthopen} package and extends
#' it with survival analysis, elastic-net sparsity, cross-validation,
#' bootstrap inference, and a visualisation suite.
#'
#' @section Key Functions:
#' \strong{Modelling:}
#' \itemize{
#'   \item \code{\link{orthoMTL}} — Core solver (regression, classification,
#'     survival)
#'   \item \code{\link{predict.orthoMTL}} — Predictions with column alignment
#'     and monotonicity projection
#'   \item \code{\link{coef.orthoMTL}} — Extract coefficient matrix
#' }
#'
#' \strong{Survival Utilities:}
#' \itemize{
#'   \item \code{\link{create_longitudinal_labels}} — Survival to binary
#'     label conversion
#'   \item \code{\link{create_indicator_matrix}} — Censoring indicator matrix
#'   \item \code{\link{create_constraint_matrix}} — Diffusion constraint
#'     matrix for temporal tasks
#' }
#'
#' \strong{Cross-Validation and Inference:}
#' \itemize{
#'   \item \code{\link{cv_orthoMTL}} — Parallel hyperparameter grid search
#'   \item \code{\link{bootstrap_orthoMTL}} — Bootstrap coefficient
#'     variability and null comparison
#'   \item \code{\link{cindex_mtl}} — Concordance index for multi-task
#'     predictions
#' }
#'
#' \strong{Visualisation:}
#' \itemize{
#'   \item \code{\link{plot_heatmap}} — Coefficient heatmap
#'   \item \code{\link{plot_correlation}} — Task distance map
#'   \item \code{\link{plot_prediction}} — Prediction swimmer plot
#'   \item \code{\link{plot_bootstrap}} — Bootstrap real vs null comparison
#' }
#'
#' \strong{Simulation:}
#' \itemize{
#'   \item \code{\link{simulate_mtl}} — Simulated survival data with
#'     time-varying effects
#' }
#'
#' @section Algorithm:
#' The core optimisation problem is:
#'
#' \deqn{\min_W \frac{1}{2n}\|XW - Y\|^2_{obs} +
#'   \lambda\left[\frac{1-\alpha}{2}\sum_{s,t} K_{st}|W_s^\top W_t| +
#'   \alpha\|W\|_1\right]}
#'
#' where the loss is computed only on non-censored entries, \eqn{K} encodes
#' task relationships, \eqn{\lambda} controls the penalty strength, and
#' \eqn{\alpha \in [0,1]} mixes the orthogonality penalty (\eqn{\alpha=0})
#' with Lasso sparsity (\eqn{\alpha=1}).
#'
#' @section Getting Started:
#' See \code{vignette("introduction", package = "orthoMTL")} for a complete
#' worked example using simulated data.
#'
#' @references
#' Vervier, K., Mahe, P., d'Aspremont, A., Veyrieras, J.-B., and
#' Vert, J.-P. (2014). On Learning Matrices with Orthogonal Columns or
#' Disjoint Supports. \emph{ECML-PKDD 2014}.
#' \url{https://hal.science/hal-00985654}
#'
#' @aliases orthoMTL-package
#' @importFrom ggplot2 aes
#' @importFrom rlang .data
#' @importFrom stats coef predict rnorm rbeta rbinom rexp runif median
#' @keywords internal
"_PACKAGE"

#' Coefficient Heatmap for orthoMTL
#'
#' Displays the coefficient matrix as a heatmap. Features (rows) can
#' optionally be reordered by their mean coefficient across tasks.
#'
#' @param x Either a fitted \code{"orthoMTL"} object or a numeric
#'   coefficient matrix with dimensions \code{p x numTasks}. If an
#'   \code{"orthoMTL"} object, coefficients are extracted via
#'   \code{coef()}.
#' @param reorder Logical. If \code{TRUE} (default), rows are sorted
#'   by mean coefficient across tasks (ascending).
#'
#' @return A \code{ggplot2} object.
#'
#' @seealso \code{\link{coef.orthoMTL}}, \code{\link{plot_correlation}}
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' colnames(Y) <- c("T1", "T2", "T3")
#' K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)
#'
#' # From fitted object
#' plot_heatmap(fit)
#'
#' # From raw matrix
#' plot_heatmap(coef(fit), reorder = FALSE)
plot_heatmap <- function(x, reorder = TRUE) {

  # Extract coefficient matrix
  if (inherits(x, "orthoMTL")) {
    Bt <- coef(x)
  } else if (is.matrix(x)) {
    Bt <- x
  } else {
    stop("'x' must be an 'orthoMTL' object or a numeric matrix.",
         call. = FALSE)
  }

  # Ensure names exist
  if (is.null(rownames(Bt))) {
    rownames(Bt) <- paste0("V", seq_len(nrow(Bt)))
  }
  if (is.null(colnames(Bt))) {
    colnames(Bt) <- paste0("T", seq_len(ncol(Bt)))
  }

  # Optionally reorder rows by mean coefficient
  if (reorder) {
    row_order <- order(apply(Bt, 1, mean), decreasing = FALSE)
    Bt <- Bt[row_order, , drop = FALSE]
  }

  # Build long-format data
  data <- expand.grid(
    X = factor(rownames(Bt), levels = rownames(Bt)),
    Y = factor(colnames(Bt), levels = colnames(Bt))
  )
  data$Z <- as.vector(Bt)

  ggplot2::ggplot(data, ggplot2::aes(.data$Y, .data$X, fill = .data$Z)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = "red", mid = "white", high = "blue"
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = "weight") +
    ggplot2::theme_minimal()
}


#' Task Correlation Map for orthoMTL
#'
#' Displays pairwise distances between task coefficient vectors as a
#' heatmap. Distance is computed as \code{1 - cosine_similarity}.
#' Values near 0 indicate similar coefficient profiles; values near 1
#' indicate orthogonal profiles.
#'
#' @param x Either a fitted \code{"orthoMTL"} object or a numeric
#'   coefficient matrix with dimensions \code{p x numTasks}.
#' @param midpoint Midpoint for the colour scale. Default: \code{0.5}.
#' @param limits Numeric vector of length 2 for the colour scale limits.
#'   Default: \code{c(0, 1)}.
#'
#' @return A \code{ggplot2} object.
#'
#' @seealso \code{\link{plot_heatmap}}
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 100; p <- 10; n_tasks <- 4
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' colnames(Y) <- c("T1", "T2", "T3", "T4")
#' K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)
#'
#' plot_correlation(fit)
plot_correlation <- function(x, midpoint = 0.5, limits = c(0, 1)) {

  # Extract coefficient matrix
  if (inherits(x, "orthoMTL")) {
    Bt <- coef(x)
  } else if (is.matrix(x)) {
    Bt <- x
  } else {
    stop("'x' must be an 'orthoMTL' object or a numeric matrix.",
         call. = FALSE)
  }

  if (is.null(colnames(Bt))) {
    colnames(Bt) <- paste0("T", seq_len(ncol(Bt)))
  }

  # Normalise columns to unit length
  norms <- apply(Bt, 2, function(col) sqrt(sum(col^2)))
  # Guard against zero-norm columns
  norms[norms == 0] <- 1
  normBt <- t(t(Bt) / norms)

  # Cosine distance: 1 - cosine_similarity
  cos_sim <- crossprod(normBt)
  dist_mat <- 1 - cos_sim

  # Clamp to [0, 1] to prevent floating-point overflow → NA in ggplot
  dist_mat[dist_mat < 0] <- 0
  dist_mat[dist_mat > 1] <- 1

  # Reverse row order for display
  task_names <- colnames(Bt)
  task_names_rev <- rev(task_names)
  dist_mat_rev <- dist_mat[task_names_rev, , drop = FALSE]

  # Build long-format data
  data <- expand.grid(
    X = factor(task_names_rev, levels = task_names_rev),
    Y = factor(task_names, levels = task_names)
  )
  data$distance <- as.vector(dist_mat_rev)

  ggplot2::ggplot(data, ggplot2::aes(.data$Y, .data$X, fill = .data$distance)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = "red", mid = "white", high = "blue",
      midpoint = midpoint,
      limits = limits
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = "distance") +
    ggplot2::theme_minimal()
}


#' Prediction Swimmer Plot for orthoMTL
#'
#' Displays a prediction matrix as a heatmap (swimmer plot), where
#' rows are patients and columns are tasks/thresholds.
#'
#' @param x A numeric prediction matrix with dimensions
#'   \code{n x numTasks}, typically produced by
#'   \code{\link{predict.orthoMTL}}.
#'
#' @return A \code{ggplot2} object.
#'
#' @seealso \code{\link{predict.orthoMTL}}
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' n <- 50; p <- 10; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' Y <- X %*% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
#' colnames(Y) <- c("T1", "T2", "T3")
#' K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
#' fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)
#'
#' preds <- predict(fit, newdata = X)
#' plot_prediction(preds)
plot_prediction <- function(x) {

  if (!is.matrix(x)) {
    stop("'x' must be a numeric matrix.", call. = FALSE)
  }

  # Ensure names exist
  if (is.null(rownames(x))) {
    rownames(x) <- paste0("obs_", seq_len(nrow(x)))
  }
  if (is.null(colnames(x))) {
    colnames(x) <- paste0("T", seq_len(ncol(x)))
  }

  # Build long-format data
  data <- expand.grid(
    X = factor(rownames(x), levels = rownames(x)),
    Y = factor(colnames(x), levels = colnames(x))
  )
  data$Z <- as.vector(x)

  ggplot2::ggplot(data, ggplot2::aes(.data$Y, .data$X, fill = .data$Z)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = "red", mid = "white", high = "blue"
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = "predicted") +
    ggplot2::theme_minimal()
}


#' Bootstrap Coefficient Comparison Plot
#'
#' Displays faceted line plots comparing real (bootstrapped) vs null
#' (permuted) coefficient trajectories across tasks for selected or
#' all features. Each panel shows the mean and standard error of the
#' coefficient at each task/threshold.
#'
#' @param x Either a \code{"bootstrap_orthoMTL"} object (as returned
#'   by \code{\link{bootstrap_orthoMTL}}) or a \code{data.frame} with
#'   columns \code{id}, \code{time}, \code{coeff}, and \code{group}.
#' @param features A character vector of feature names to plot. If
#'   \code{NULL} (default), all features are plotted.
#' @param batch_size Number of features per facet page. Default:
#'   \code{9}.
#'
#' @return A list of \code{ggplot2} objects, one per batch/page.
#'
#' @details
#' Blue lines show coefficients from models fitted on bootstrapped
#' (resampled) data — reflecting estimation variability under real
#' signal. Grey lines show coefficients from models fitted on
#' permuted outcomes — reflecting the null distribution.
#'
#' When the blue (real) band is clearly separated from the grey
#' (null) band, the feature's coefficient is distinguishable from
#' noise at that task/threshold.
#'
#' @seealso \code{\link{bootstrap_orthoMTL}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' n <- 30; p <- 5; n_tasks <- 3
#' X <- matrix(rnorm(n * p), n, p)
#' colnames(X) <- paste0("V", seq_len(p))
#' SurvTime <- rexp(n, rate = 0.1)
#' Event <- rbinom(n, 1, 0.7)
#' thresholds <- c(4, 6, 10)
#' Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
#' W <- create_indicator_matrix(Y)
#' K <- create_constraint_matrix(n_tasks)
#' boot_res <- bootstrap_orthoMTL(
#'   X = X, Y = Y, lambda = 1e-3, step_size = 0.1,
#'   K = K, survival = TRUE, censored.mat = W,
#'   n_repeats = 5, n_cores = 1, verbose = FALSE
#' )
#' plots <- plot_bootstrap(boot_res)
#' plots[[1]]
#' }
plot_bootstrap <- function(x, features = NULL, batch_size = 9) {

  # Extract data.frame from S3 object or use directly
  if (inherits(x, "bootstrap_orthoMTL")) {
    df <- x$results
  } else if (is.data.frame(x)) {
    required_cols <- c("id", "time", "coeff", "group")
    missing_cols <- setdiff(required_cols, colnames(x))
    if (length(missing_cols) > 0) {
      stop("data.frame is missing required columns: ",
           paste(missing_cols, collapse = ", "), call. = FALSE)
    }
    df <- x
  } else {
    stop("'x' must be a 'bootstrap_orthoMTL' object or a data.frame ",
         "with columns: id, time, coeff, group.", call. = FALSE)
  }

  # Subset features if requested
  if (!is.null(features)) {
    missing_feats <- setdiff(features, unique(df$id))
    if (length(missing_feats) > 0) {
      warning("The following features were not found and will be skipped: ",
              paste(missing_feats, collapse = ", "), call. = FALSE)
    }
    df <- df[df$id %in% features, , drop = FALSE]
    feature_order <- features[features %in% unique(df$id)]
  } else {
    feature_order <- unique(df$id)
  }

  n_features <- length(feature_order)

  if (n_features == 0) {
    warning("No features to plot.", call. = FALSE)
    return(list())
  }

  # Batch features into pages
  n_batches <- ceiling(n_features / batch_size)
  plot_list <- vector("list", n_batches)

  for (b in seq_len(n_batches)) {
    start_idx <- (b - 1) * batch_size + 1
    end_idx <- min(b * batch_size, n_features)
    batch_features <- feature_order[start_idx:end_idx]

    batch_df <- df[df$id %in% batch_features, , drop = FALSE]
    batch_df$feature <- factor(batch_df$id, levels = batch_features)
    # Safely convert: only reorder numerically if all values can be parsed as numbers
    times_as_num <- suppressWarnings(as.numeric(as.character(batch_df[["time"]])))
    if (!any(is.na(times_as_num))) {
      # All numeric — sort numerically
      batch_df[["time"]] <- factor(batch_df[["time"]],
                                   levels = sort(unique(as.character(batch_df[["time"]])[order(times_as_num)])))
    } else {
      # Non-numeric labels — preserve original order
      batch_df[["time"]] <- factor(batch_df[["time"]],
                                   levels = unique(batch_df[["time"]]))
    }

    plot_list[[b]] <- ggplot2::ggplot(
      batch_df,
      ggplot2::aes(
        x = .data$time,
        y = .data$coeff,
        colour = .data$group,
        group = .data$group
      )
    ) +
      ggplot2::stat_summary(fun = mean, geom = "line") +
      ggplot2::stat_summary(fun.data = ggplot2::mean_se, geom = "errorbar",
                            width = 0.2) +
      ggplot2::facet_wrap(~ feature, scales = "free_y") +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
      ggplot2::xlab("") +
      ggplot2::ylab("model coefficient") +
      ggplot2::theme_minimal()
  }

  return(plot_list)
}

# Tests for plotting functions

test_that("plot_heatmap returns ggplot from orthoMTL object", {
  set.seed(42)
  X <- matrix(rnorm(60), 12, 5)
  colnames(X) <- paste0("V", 1:5)
  Y <- matrix(rnorm(36), 12, 3)
  colnames(Y) <- paste0("T", 1:3)
  fit <- orthoMTL(X, Y, lambda = 0.01)

  p <- plot_heatmap(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_heatmap returns ggplot from raw matrix", {
  Bt <- matrix(rnorm(15), 5, 3)
  rownames(Bt) <- paste0("V", 1:5)
  colnames(Bt) <- paste0("T", 1:3)

  p <- plot_heatmap(Bt)
  expect_s3_class(p, "ggplot")
})

test_that("plot_heatmap respects reorder = FALSE", {
  Bt <- matrix(c(3, 1, 2, 6, 4, 5), 3, 2)
  rownames(Bt) <- c("A", "B", "C")
  colnames(Bt) <- c("T1", "T2")

  p_ordered <- plot_heatmap(Bt, reorder = TRUE)
  p_raw <- plot_heatmap(Bt, reorder = FALSE)

  # Both should be valid ggplots
  expect_s3_class(p_ordered, "ggplot")
  expect_s3_class(p_raw, "ggplot")

  # Extract row order from the data
  raw_levels <- levels(p_raw$data$X)
  expect_equal(raw_levels, c("A", "B", "C"))
})

test_that("plot_heatmap generates names when absent", {
  Bt <- matrix(rnorm(12), 4, 3)  # no row/col names

  p <- plot_heatmap(Bt)
  expect_s3_class(p, "ggplot")
})

test_that("plot_heatmap errors on invalid input", {
  expect_error(plot_heatmap("not_a_matrix"), "must be")
})

test_that("plot_correlation returns ggplot", {
  set.seed(42)
  X <- matrix(rnorm(60), 12, 5)
  colnames(X) <- paste0("V", 1:5)
  Y <- matrix(rnorm(48), 12, 4)
  colnames(Y) <- paste0("T", 1:4)
  fit <- orthoMTL(X, Y, lambda = 0.01)

  p <- plot_correlation(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_correlation accepts custom midpoint and limits", {
  Bt <- matrix(rnorm(20), 5, 4)
  colnames(Bt) <- paste0("T", 1:4)

  p <- plot_correlation(Bt, midpoint = 0.3, limits = c(-0.5, 1.5))
  expect_s3_class(p, "ggplot")
})

test_that("plot_correlation errors on invalid input", {
  expect_error(plot_correlation(list(a = 1)), "must be")
})

test_that("plot_prediction returns ggplot", {
  Mt <- matrix(rnorm(30), 10, 3)
  rownames(Mt) <- paste0("pt_", 1:10)
  colnames(Mt) <- c("T1", "T2", "T3")

  p <- plot_prediction(Mt)
  expect_s3_class(p, "ggplot")
})

test_that("plot_prediction generates names when absent", {
  Mt <- matrix(rnorm(12), 4, 3)  # no names

  p <- plot_prediction(Mt)
  expect_s3_class(p, "ggplot")
})

test_that("plot_prediction errors on non-matrix", {
  expect_error(plot_prediction(data.frame(a = 1)), "must be a numeric matrix")
})

test_that("plot_bootstrap returns list of ggplots from S3 object", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = 3, n_cores = 1, verbose = FALSE
  )

  plots <- plot_bootstrap(boot_res)
  expect_true(is.list(plots))
  expect_true(length(plots) >= 1)
  expect_s3_class(plots[[1]], "ggplot")
})

test_that("plot_bootstrap accepts raw data.frame", {
  df <- data.frame(
    id = rep(c("V1", "V2"), each = 8),
    time = rep(rep(c("T1", "T2"), each = 2), 4),
    coeff = rnorm(16),
    group = rep(c("real", "null"), 8),
    stringsAsFactors = FALSE
  )

  plots <- plot_bootstrap(df)
  expect_true(is.list(plots))
  expect_s3_class(plots[[1]], "ggplot")
})

test_that("plot_bootstrap subsets features correctly", {
  set.seed(42)
  n <- 20; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = 3, n_cores = 1, verbose = FALSE
  )

  plots <- plot_bootstrap(boot_res, features = c("V1", "V3"))
  expect_true(is.list(plots))
  expect_s3_class(plots[[1]], "ggplot")
})

test_that("plot_bootstrap warns on missing features", {
  df <- data.frame(
    id = rep("V1", 4),
    time = rep(c("T1", "T2"), 2),
    coeff = rnorm(4),
    group = rep(c("real", "null"), 2),
    stringsAsFactors = FALSE
  )

  expect_warning(
    plot_bootstrap(df, features = c("V1", "NONEXISTENT")),
    "not found"
  )
})

test_that("plot_bootstrap respects batch_size", {
  df <- data.frame(
    id = rep(paste0("V", 1:10), each = 4),
    time = rep(rep(c("T1", "T2"), each = 2), 10),
    coeff = rnorm(40),
    group = rep(c("real", "null"), 20),
    stringsAsFactors = FALSE
  )

  plots <- suppressWarnings(plot_bootstrap(df, batch_size = 4))
  expect_equal(length(plots), 3)
})

test_that("plot_bootstrap errors on invalid input", {
  expect_error(plot_bootstrap("not_valid"), "must be")
})

test_that("plot_bootstrap errors on data.frame with missing columns", {
  df <- data.frame(id = "V1", time = "T1", coeff = 1.0)
  # missing 'group' column
  expect_error(plot_bootstrap(df), "missing required columns")
})

#' Convert Survival Data to Longitudinal Binary Labels
#'
#' Transforms time-to-event survival data into a matrix of binary labels
#' at user-defined time thresholds. This is the core data transformation
#' that enables survival analysis within the multi-task learning framework.
#'
#' The encoding logic for each patient at each threshold is:
#' \itemize{
#'   \item \code{1} — patient is progression-free at this threshold
#'     (survival time exceeds threshold, regardless of event status)
#'   \item \code{0} — patient experienced an event before this threshold
#'     (survival time < threshold AND event observed)
#'   \item \code{NA} — patient was censored before this threshold
#'     (survival time < threshold AND no event observed); label is unknown
#' }
#'
#' @param SurvTime A numeric vector of length \code{n} containing
#'   observed survival times. Must be non-negative.
#' @param Event A numeric vector of length \code{n} encoding censoring
#'   status. \code{1} = event observed, \code{0} = censored.
#' @param thresholds A numeric vector of length \code{numTasks}
#'   representing the time thresholds at which to evaluate survival
#'   status. Default: \code{c(4, 6)}.
#'
#' @return A numeric matrix of dimensions \code{n x numTasks}. Column
#'   names are set to the threshold values. Contains \code{1}, \code{0},
#'   and \code{NA} values as described above.
#'
#' @seealso \code{\link{create_indicator_matrix}} to convert \code{NA}
#'   values into a binary censoring indicator matrix.
#'
#' @export
#'
#' @examples
#' # Simulate 10 patients
#' set.seed(42)
#' SurvTime <- rexp(10, rate = 0.1)
#' Event <- rbinom(10, 1, 0.7)
#' thresholds <- c(4, 6, 10, 15)
#'
#' Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
#' Y   # 1 = progression-free, 0 = event, NA = censored
create_longitudinal_labels <- function(SurvTime, Event, thresholds = c(4, 6)) {

  if (any(SurvTime < 0)) {
    stop("SurvTime vector should be positive values only", call. = FALSE)
  }
  if (any(Event > 1)) {
    stop("Event vector is encoded as binary values", call. = FALSE)
  }

  Y <- matrix(1, nrow = length(SurvTime), ncol = length(thresholds))

  # Observed events before each threshold
  for (i in seq_along(thresholds)) {
    Y[which(SurvTime < thresholds[i] & Event == 1), i] <- 0
  }

  # Censored observations before each threshold
  for (i in seq_along(thresholds)) {
    Y[which(SurvTime < thresholds[i] & Event == 0), i] <- NA
  }

  colnames(Y) <- thresholds
  return(Y)
}


#' Create Censoring Indicator Matrix
#'
#' Converts \code{NA} entries in a longitudinal label matrix (as produced
#' by \code{\link{create_longitudinal_labels}}) into a binary indicator
#' matrix. This indicator is used by \code{\link{orthoMTL}} to mask
#' censored observations in the loss computation.
#'
#' @param Y A numeric matrix of dimensions \code{n x numTasks}, typically
#'   produced by \code{\link{create_longitudinal_labels}}. May contain
#'   \code{NA} values for censored observations.
#'
#' @return A numeric matrix of the same dimensions as \code{Y}.
#'   \code{1} = label is observed (known), \code{0} = label is censored
#'   (unknown).
#'
#' @seealso \code{\link{create_longitudinal_labels}}
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' SurvTime <- rexp(10, rate = 0.1)
#' Event <- rbinom(10, 1, 0.7)
#' Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
#' W <- create_indicator_matrix(Y)
#' W   # 1 = observed, 0 = censored
create_indicator_matrix <- function(Y) {
  W <- matrix(1, nrow = nrow(Y), ncol = ncol(Y))
  W[which(is.na(Y), arr.ind = TRUE)] <- 0
  return(W)
}

# Tests for simulate_mtl

test_that("simulate_mtl returns S3 class with expected structure", {
  sim <- simulate_mtl(n = 50, p = 10, n_signals = 3, seed = 42)

  expect_s3_class(sim, "simulated_mtl")
  expect_true(is.matrix(sim$X))
  expect_true(is.numeric(sim$SurvTime))
  expect_true(is.numeric(sim$Event))
  expect_true(is.numeric(sim$treatment))
  expect_true(is.list(sim$ground_truth))
  expect_equal(sim$n, 50)
  expect_equal(sim$p, 10)
  expect_equal(sim$n_signals, 3)
})

test_that("simulate_mtl X has correct dimensions and names", {
  sim <- simulate_mtl(n = 40, p = 15, n_continuous = 2, seed = 1)

  expect_equal(dim(sim$X), c(40, 16))  # 15 features + treatment
  expect_equal(ncol(sim$X), sim$p + 1)
  expect_equal(colnames(sim$X), sim$feature_names)

  # First 2 columns should be continuous
  expect_true(all(grepl("^cont_", sim$feature_names[1:2])))

  # Last column is treatment
  expect_equal(sim$feature_names[length(sim$feature_names)], "treatment")
})

test_that("simulate_mtl binary features are 0/1", {
  sim <- simulate_mtl(n = 100, p = 20, n_continuous = 1, seed = 42)

  binary_cols <- grep("^mut_", colnames(sim$X))
  for (j in binary_cols) {
    vals <- unique(sim$X[, j])
    expect_true(all(vals %in% c(0, 1)),
                info = paste("Column", colnames(sim$X)[j],
                             "has non-binary values"))
  }
})

test_that("simulate_mtl treatment is balanced", {
  sim <- simulate_mtl(n = 200, p = 10, seed = 42)

  trt_table <- table(sim$treatment)
  expect_equal(as.numeric(trt_table), c(100, 100))
})

test_that("simulate_mtl SurvTime is positive and Event is binary", {
  sim <- simulate_mtl(n = 100, p = 10, seed = 42)

  expect_true(all(sim$SurvTime > 0))
  expect_true(all(sim$Event %in% c(0, 1)))
})

test_that("simulate_mtl censoring rate is reasonable", {
  sim <- simulate_mtl(n = 500, p = 10, censoring_max = 25, seed = 42)

  event_rate <- mean(sim$Event)
  # Should be between 20% and 90% for reasonable settings
  expect_true(event_rate > 0.2 && event_rate < 0.9,
              info = paste("Event rate:", event_rate))
})

test_that("simulate_mtl ground truth has correct structure", {
  sim <- simulate_mtl(n = 50, p = 20, n_signals = 5, seed = 42)

  gt <- sim$ground_truth

  expect_true(is.matrix(gt$coefficients))
  expect_equal(dim(gt$coefficients), c(21, length(sim$thresholds)))
  expect_equal(length(gt$signal_features), 5)
  expect_equal(length(gt$null_features), 15)
  expect_equal(length(gt$effect_types), 5)

  # Signal features should have non-zero coefficients
  for (feat in gt$signal_features) {
    expect_true(any(gt$coefficients[feat, ] != 0),
                info = paste(feat, "has all-zero coefficients"))
  }

  # Null features should have all-zero coefficients
  for (feat in gt$null_features) {
    expect_true(all(gt$coefficients[feat, ] == 0),
                info = paste(feat, "has non-zero coefficients"))
  }

  # Treatment should have non-zero coefficients
  expect_true(any(gt$coefficients["treatment", ] != 0))
})

test_that("simulate_mtl effect types cycle through templates", {
  sim <- simulate_mtl(n = 50, p = 20, n_signals = 10, seed = 42)

  types <- sim$ground_truth$effect_types
  valid_types <- c("early", "late", "constant", "switch", "decreasing")

  expect_true(all(types %in% valid_types))

  # With 10 signals and 5 templates, each template should appear twice
  type_counts <- table(types)
  expect_true(all(type_counts == 2))
})

test_that("simulate_mtl is reproducible with same seed", {
  sim1 <- simulate_mtl(n = 50, p = 10, seed = 123)
  sim2 <- simulate_mtl(n = 50, p = 10, seed = 123)

  expect_identical(sim1$X, sim2$X)
  expect_identical(sim1$SurvTime, sim2$SurvTime)
  expect_identical(sim1$Event, sim2$Event)
  expect_identical(sim1$ground_truth$coefficients,
                   sim2$ground_truth$coefficients)
})

test_that("simulate_mtl produces different data with different seed", {
  sim1 <- simulate_mtl(n = 50, p = 10, seed = 1)
  sim2 <- simulate_mtl(n = 50, p = 10, seed = 2)

  expect_false(identical(sim1$X, sim2$X))
})

test_that("simulate_mtl validates inputs", {
  expect_error(simulate_mtl(n = 5), "at least 10")
  expect_error(simulate_mtl(p = 0), "at least 1")
  expect_error(simulate_mtl(n_signals = 50, p = 10), "cannot exceed")
  expect_error(simulate_mtl(thresholds = c(10, 5)), "strictly increasing")
  expect_error(simulate_mtl(thresholds = c(5)), "At least 2")
  expect_error(simulate_mtl(baseline_hazard = -1), "positive")
  expect_error(simulate_mtl(censoring_max = 0), "positive")
})

test_that("simulate_mtl integrates with orthoMTL workflow", {
  sim <- simulate_mtl(n = 50, p = 10, n_signals = 3, seed = 42)

  Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, sim$thresholds)
  W <- create_indicator_matrix(Y)
  K <- create_constraint_matrix(length(sim$thresholds))

  fit <- orthoMTL(sim$X, Y, lambda = 1e-3, K = K,
                  survival = TRUE, censored.mat = W)

  expect_s3_class(fit, "orthoMTL")
  expect_equal(dim(fit$B), c(ncol(sim$X), length(sim$thresholds)))
})

test_that("print.simulated_mtl produces output", {
  sim <- simulate_mtl(n = 50, p = 10, n_signals = 3, seed = 42)

  expect_output(print(sim), "Simulated multi-task survival data")
  expect_output(print(sim), "Signal features")
  expect_output(print(sim), "Event rate")
  expect_output(print(sim), "Treatment effect")
})

# Smoke tests for cv_orthoMTL

test_that("cv_orthoMTL returns S3 class with expected structure", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  SurvTime <- rexp(n, 0.1)
  Event <- rbinom(n, 1, 0.7)

  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)
  K <- create_constraint_matrix(n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y, W.train = W, K = K,
    lambdas = c(1e-3), alphas = 0,
    stepsizes = c(0.1), diag_vals = c(0.5),
    survival = TRUE, disjoint = FALSE,
    folds = folds, n_cores = 1, seed = 42, verbose = FALSE
  )

  expect_s3_class(cv_res, "cv_orthoMTL")
  expect_true(is.list(cv_res$best))
  expect_true(is.data.frame(cv_res$results))
  expect_equal(length(cv_res$folds), n)
  expect_true("cv_score" %in% colnames(cv_res$results))
  expect_equal(cv_res$n_folds, 2)
})

test_that("cv_orthoMTL best has all expected fields", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = c(0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )

  expect_true(all(c("lambda", "alpha", "stepsize", "diag_val", "cv_score")
                  %in% names(cv_res$best)))
})

test_that("cv_orthoMTL searches multiple configurations", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = c(0.001, 0.01), stepsizes = c(0.1, 0.5),
    diag_vals = c(0.5, 1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )

  # 2 lambdas x 2 stepsizes x 2 diag_vals x 1 alpha = 8
  expect_equal(cv_res$n_configs, 8)
  expect_equal(nrow(cv_res$results), 8)

  # Non-survival default metric is RMSE (lower is better), so results are
  # sorted best-first == ascending, and the best score is the minimum.
  expect_equal(cv_res$metric, "rmse")
  expect_true(all(diff(cv_res$results$cv_score) >= 0))
  expect_equal(cv_res$best$cv_score, min(cv_res$results$cv_score))
})

test_that("cv_orthoMTL generates folds with warning when not provided", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  expect_warning(
    cv_orthoMTL(X.train = X, Y.train = Y,
                lambdas = c(0.01), stepsizes = c(0.1), diag_vals = c(1),
                survival = FALSE, folds = NULL, n_cores = 1, verbose = FALSE),
    "not provided"
  )
})

test_that("cv_orthoMTL validates inputs", {
  expect_error(
    cv_orthoMTL(X.train = NULL, Y.train = NULL,
                lambdas = 1, stepsizes = 1, diag_vals = 1),
    "must be provided"
  )
})

test_that("print.cv_orthoMTL produces output", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = c(0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )

  expect_output(print(cv_res), "Cross-validation")
  expect_output(print(cv_res), "Best configuration")
})

# Tests for create_longitudinal_labels and create_indicator_matrix

test_that("create_longitudinal_labels produces correct dimensions", {
  SurvTime <- c(2, 5, 8, 12, 20)
  Event <- c(1, 1, 0, 1, 0)
  thresholds <- c(4, 6, 10)

  Y <- create_longitudinal_labels(SurvTime, Event, thresholds)

  expect_true(is.matrix(Y))
  expect_equal(dim(Y), c(5, 3))
  expect_equal(colnames(Y), as.character(thresholds))
})

test_that("create_longitudinal_labels encodes correctly", {
  SurvTime <- c(2, 5, 8, 12)
  Event <-    c(1, 1, 0, 1)
  thresholds <- c(4, 6, 10)

  Y <- create_longitudinal_labels(SurvTime, Event, thresholds)

  # Patient 1: event at 2, before all thresholds

  expect_equal(unname(Y[1, ]), c(0, 0, 0))

  # Patient 2: event at 5, before threshold 6 and 10
  expect_equal(unname(Y[2, ]), c(1, 0, 0))

  # Patient 3: censored at 8, before threshold 10
  expect_equal(unname(Y[3, ]), c(1, 1, NA))

  # Patient 4: event at 12, after all thresholds
  expect_equal(unname(Y[4, ]), c(1, 1, 1))
})

test_that("create_longitudinal_labels validates inputs", {
  expect_error(
    create_longitudinal_labels(c(-1, 5), c(1, 0), c(4, 6)),
    "positive"
  )
  expect_error(
    create_longitudinal_labels(c(2, 5), c(1, 2), c(4, 6)),
    "binary"
  )
})

test_that("create_indicator_matrix marks NAs as zero", {
  Y <- matrix(c(1, 0, NA, 1, NA, 1), nrow = 3, ncol = 2)
  W <- create_indicator_matrix(Y)

  expect_equal(dim(W), dim(Y))
  expect_equal(W[1, ], c(1, 1))
  expect_equal(W[2, ], c(1, 0))
  expect_equal(W[3, ], c(0, 1))
})

test_that("create_indicator_matrix handles all-observed data", {
  Y <- matrix(c(1, 0, 1, 0), nrow = 2, ncol = 2)
  W <- create_indicator_matrix(Y)

  expect_true(all(W == 1))
})

# Smoke tests for bootstrap_orthoMTL

test_that("bootstrap_orthoMTL returns S3 class with expected structure", {
  set.seed(42)
  n <- 20; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01, step_size = 0.1,
    n_repeats = 3, n_cores = 1, verbose = FALSE
  )

  expect_s3_class(boot_res, "bootstrap_orthoMTL")
  expect_true(is.data.frame(boot_res$results))
  expect_true(is.list(boot_res$coefficients_real))
  expect_true(is.list(boot_res$coefficients_null))
  expect_equal(length(boot_res$coefficients_real), 3)
  expect_equal(length(boot_res$coefficients_null), 3)
  expect_equal(length(boot_res$obj_real), 3)
  expect_equal(length(boot_res$obj_null), 3)
  expect_equal(boot_res$n_repeats, 3)
  expect_equal(boot_res$n_features, p)
  expect_equal(boot_res$n_tasks, n_tasks)
})

test_that("bootstrap_orthoMTL results data.frame has expected columns", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = 2, n_cores = 1, verbose = FALSE
  )

  expected_cols <- c("id", "time", "coeff", "group", "repeat_id")
  expect_true(all(expected_cols %in% colnames(boot_res$results)))

  # Should have both groups
  expect_true(all(c("real", "null") %in% unique(boot_res$results$group)))

  # All features should appear
  expect_equal(sort(unique(boot_res$results$id)),
               sort(colnames(X)))
})

test_that("bootstrap_orthoMTL results have correct row count", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)
  n_repeats <- 4

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = n_repeats, n_cores = 1, verbose = FALSE
  )

  # Expected: n_features * n_tasks * n_repeats * 2 (real + null)
  expected_rows <- p * n_tasks * n_repeats * 2
  expect_equal(nrow(boot_res$results), expected_rows)
})

test_that("bootstrap_orthoMTL works in survival mode", {
  set.seed(42)
  n <- 25; p <- 4; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  SurvTime <- rexp(n, 0.1)
  Event <- rbinom(n, 1, 0.7)

  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)
  K <- create_constraint_matrix(n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01, step_size = 0.1,
    K = K, survival = TRUE, censored.mat = W,
    n_repeats = 2, n_cores = 1, verbose = FALSE
  )

  expect_s3_class(boot_res, "bootstrap_orthoMTL")
  expect_equal(boot_res$n_tasks, n_tasks)
})

test_that("bootstrap_orthoMTL obj_real and obj_null are separate", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = 3, n_cores = 1, verbose = FALSE
  )

  # Real and null objectives should generally differ
  # (extremely unlikely to be identical with different data)
  expect_false(identical(boot_res$obj_real, boot_res$obj_null))
})

test_that("bootstrap_orthoMTL validates inputs", {
  expect_error(
    bootstrap_orthoMTL(X = NULL, Y = NULL),
    "must be provided"
  )

  X <- matrix(rnorm(20), 4, 5)
  Y <- matrix(rnorm(12), 4, 3)
  expect_error(
    bootstrap_orthoMTL(X = X, Y = Y, survival = TRUE, censored.mat = NULL),
    "censored.mat"
  )
})

test_that("print.bootstrap_orthoMTL produces output", {
  set.seed(42)
  n <- 20; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  boot_res <- bootstrap_orthoMTL(
    X = X, Y = Y, lambda = 0.01,
    n_repeats = 2, n_cores = 1, verbose = FALSE
  )

  expect_output(print(boot_res), "Bootstrap inference")
  expect_output(print(boot_res), "Real models")
  expect_output(print(boot_res), "Null models")
})

# Tests for the core orthoMTL solver

test_that("orthoMTL returns S3 class with expected structure", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", seq_len(p))
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", seq_len(n_tasks))

  fit <- orthoMTL(X, Y, lambda = 0.01)

  expect_s3_class(fit, "orthoMTL")
  expect_true(is.matrix(fit$B))
  expect_equal(dim(fit$B), c(p, n_tasks))
  expect_identical(fit$B, fit$W)  # alias
  expect_true(is.numeric(fit$obj))
  expect_true(is.numeric(fit$imax))
  expect_true(is.logical(fit$converged))
  expect_true(is.list(fit$hyperparameters))
  expect_equal(fit$n_features, p)
  expect_equal(fit$n_tasks, n_tasks)
  expect_equal(fit$n_obs, n)
  expect_equal(fit$feature_names, colnames(X))
  expect_equal(fit$task_names, colnames(Y))
})

test_that("orthoMTL preserves row/column names on B", {
  X <- matrix(rnorm(40), 8, 5)
  colnames(X) <- paste0("f", 1:5)
  Y <- matrix(rnorm(24), 8, 3)
  colnames(Y) <- c("t1", "t2", "t3")

  fit <- orthoMTL(X, Y, lambda = 0.01)

  expect_equal(rownames(fit$B), colnames(X))
  expect_equal(colnames(fit$B), colnames(Y))
})

test_that("orthoMTL converges on simple regression problem", {
  set.seed(42)
  n <- 100; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  W_true <- matrix(rnorm(p * n_tasks), p, n_tasks)
  Y <- X %*% W_true + matrix(rnorm(n * n_tasks), n) * 0.01

  fit <- orthoMTL(X, Y, lambda = 1e-4, step_size = 0.1)

  expect_true(fit$converged)
  expect_true(fit$imax < 1e6)
})

test_that("orthoMTL respects logistic mode", {
  set.seed(42)
  n <- 50; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(sample(c(-1, 1), n * n_tasks, replace = TRUE), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01, logistic = TRUE)

  expect_s3_class(fit, "orthoMTL")
  expect_true(fit$hyperparameters$logistic)
})

test_that("logistic gradient is correct under label flip (REG-01)", {
  # The logistic loss log(1+exp(-Y*Xw)) is invariant under (Y, W) -> (-Y, -W),
  # so flipping all labels must negate the fitted coefficients. The old
  # gradient (-(Y - sigmoid(Xw)), a y in {0,1} form) breaks this symmetry.
  set.seed(7)
  n <- 60; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(sample(c(-1, 1), n * n_tasks, replace = TRUE), n, n_tasks)

  fit_pos <- orthoMTL(X,  Y, lambda = 0.01, logistic = TRUE)
  fit_neg <- orthoMTL(X, -Y, lambda = 0.01, logistic = TRUE)

  expect_equal(fit_neg$B, -fit_pos$B, tolerance = 1e-6)
})

test_that("logistic loss does not overflow on large scores (REG-02)", {
  # Large |Xw| made the naive log(1+exp(-Y*Xw)) overflow to Inf/NaN.
  set.seed(11)
  n <- 40; p <- 3; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p) * 100
  Y <- matrix(sample(c(-1, 1), n * n_tasks, replace = TRUE), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01, logistic = TRUE, step_size = 1)

  expect_true(is.finite(fit$obj))
  expect_false(anyNA(fit$B))
})

test_that("orthoMTL survival mode requires censored.mat", {
  X <- matrix(rnorm(20), 4, 5)
  Y <- matrix(c(1, 0, NA, 1, 1, 1, NA, 0), 4, 2)

  expect_error(
    orthoMTL(X, Y, survival = TRUE, censored.mat = NULL),
    "censoring information"
  )
})

test_that("orthoMTL survival mode runs with valid censored.mat", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  SurvTime <- rexp(n, 0.1)
  Event <- rbinom(n, 1, 0.7)

  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)

  fit <- orthoMTL(X, Y, lambda = 0.01, survival = TRUE, censored.mat = W)

  expect_s3_class(fit, "orthoMTL")
  expect_true(fit$hyperparameters$survival)
})

test_that("orthoMTL disjoint mode runs", {
  set.seed(42)
  n <- 50; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01, disjoint = TRUE, step_size = 5)

  expect_s3_class(fit, "orthoMTL")
  expect_true(fit$hyperparameters$disjoint)
})

test_that("orthoMTL elastic-net (alpha > 0) runs and stores alpha", {
  set.seed(42)
  n <- 50; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01, alpha = 0.5)

  expect_s3_class(fit, "orthoMTL")
  expect_equal(fit$hyperparameters$alpha, 0.5)
})

test_that("orthoMTL rejects alpha outside [0, 1]", {
  X <- matrix(rnorm(20), 4, 5)
  Y <- matrix(rnorm(8), 4, 2)
  expect_error(orthoMTL(X, Y, alpha = 1.5), "\\[0, 1\\]")
  expect_error(orthoMTL(X, Y, alpha = -0.1), "\\[0, 1\\]")
})

test_that("elastic-net mixing endpoints both produce finite fits", {
  set.seed(1)
  n <- 40; p <- 6; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit0 <- orthoMTL(X, Y, lambda = 0.01, alpha = 0)   # pure orthogonality
  fit1 <- orthoMTL(X, Y, lambda = 0.01, alpha = 1)   # pure Lasso

  expect_false(anyNA(fit0$B))
  expect_false(anyNA(fit1$B))
  expect_true(is.finite(fit0$obj))
  expect_true(is.finite(fit1$obj))
})

test_that("orthoMTL stops on invalid inputs", {
  expect_error(orthoMTL(NULL, NULL), "need to be provided")
  expect_error(orthoMTL(matrix(1, 2, 2), matrix(1, 2, 2), lambda = -1),
               "positive")
})

test_that("orthoMTL tol parameter is used", {
  set.seed(42)
  n <- 50; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit_strict <- orthoMTL(X, Y, lambda = 0.01, tol = 1e-8)
  fit_loose <- orthoMTL(X, Y, lambda = 0.01, tol = 1e-2)

  # Stricter tolerance should take more (or equal) iterations
  expect_true(fit_strict$imax >= fit_loose$imax)
})

test_that("orthoMTL W_0 parameter works", {
  set.seed(42)
  n <- 50; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  W_0 <- matrix(0.1, p, n_tasks)
  fit <- orthoMTL(X, Y, lambda = 0.01, W_0 = W_0)

  expect_s3_class(fit, "orthoMTL")
})

# Tests for predict.orthoMTL

test_that("predict.orthoMTL returns correct dimensions", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  colnames(Y) <- paste0("T", 1:n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01)

  X_new <- matrix(rnorm(10 * p), 10, p)
  colnames(X_new) <- paste0("V", 1:p)

  preds <- predict(fit, newdata = X_new)

  expect_true(is.matrix(preds))
  expect_equal(dim(preds), c(10, n_tasks))
  expect_equal(colnames(preds), paste0("T", 1:n_tasks))
})

test_that("predict.orthoMTL aligns columns correctly", {
  set.seed(42)
  n <- 20; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- c("A", "B", "C", "D")
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01)

  # Reversed column order
  X_new <- matrix(rnorm(5 * p), 5, p)
  colnames(X_new) <- c("D", "C", "B", "A")

  preds <- predict(fit, newdata = X_new)
  expect_equal(dim(preds), c(5, n_tasks))
})

test_that("predict.orthoMTL errors on missing features", {
  set.seed(42)
  n <- 20; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- c("A", "B", "C", "D")
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01)

  X_new <- matrix(rnorm(5 * 3), 5, 3)
  colnames(X_new) <- c("A", "B", "C")  # missing D

  expect_error(predict(fit, newdata = X_new), "missing")
})

test_that("predict.orthoMTL warns on extra features", {
  set.seed(42)
  n <- 20; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- c("A", "B", "C", "D")
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01)

  X_new <- matrix(rnorm(5 * 5), 5, 5)
  colnames(X_new) <- c("A", "B", "C", "D", "EXTRA")

  expect_warning(predict(fit, newdata = X_new), "not used by the model")
})

test_that("predict.orthoMTL errors on dimension mismatch without names", {
  set.seed(42)
  n <- 20; p <- 4; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)  # no colnames
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)

  fit <- orthoMTL(X, Y, lambda = 0.01)

  X_wrong <- matrix(rnorm(5 * 3), 5, 3)
  expect_error(predict(fit, newdata = X_wrong), "columns")
})

test_that("predict.orthoMTL applies monotonicity in survival mode", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  colnames(X) <- paste0("V", 1:p)

  SurvTime <- rexp(n, 0.1)
  Event <- rbinom(n, 1, 0.7)
  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)

  fit <- orthoMTL(X, Y, lambda = 0.01, survival = TRUE, censored.mat = W)

  X_new <- matrix(rnorm(10 * p), 10, p)
  colnames(X_new) <- paste0("V", 1:p)
  preds <- predict(fit, newdata = X_new)

  # Predictions should be non-increasing across columns for each row
  for (i in seq_len(nrow(preds))) {
    diffs <- diff(preds[i, ])
    expect_true(all(diffs <= .Machine$double.eps * 100),
                info = paste("Row", i, "is not non-increasing"))
  }
})

# Tests for print, summary, and coef S3 methods

test_that("print.orthoMTL produces output and returns invisibly", {
  set.seed(42)
  X <- matrix(rnorm(60), 12, 5)
  Y <- matrix(rnorm(36), 12, 3)
  fit <- orthoMTL(X, Y, lambda = 0.01)

  expect_output(print(fit), "orthoMTL model")
  expect_output(print(fit), "Converged")
  expect_output(print(fit), "lambda")

  out <- print(fit)
  expect_s3_class(out, "orthoMTL")
})

test_that("print.orthoMTL shows correct mode", {
  X <- matrix(rnorm(60), 12, 5)

  # Regression
  Y_reg <- matrix(rnorm(36), 12, 3)
  fit_reg <- orthoMTL(X, Y_reg, lambda = 0.01)
  expect_output(print(fit_reg), "regression")

  # Classification
  Y_cls <- matrix(sample(c(-1, 1), 36, replace = TRUE), 12, 3)
  fit_cls <- orthoMTL(X, Y_cls, lambda = 0.01, logistic = TRUE)
  expect_output(print(fit_cls), "classification")
})

test_that("summary.orthoMTL produces detailed output", {
  set.seed(42)
  X <- matrix(rnorm(60), 12, 5)
  colnames(X) <- paste0("V", 1:5)
  Y <- matrix(rnorm(36), 12, 3)
  fit <- orthoMTL(X, Y, lambda = 0.01)

  expect_output(summary(fit), "orthoMTL model")
  expect_output(summary(fit), "Hyperparameters")
  expect_output(summary(fit), "Coefficients")
  expect_output(summary(fit), "Sparsity")
  expect_output(summary(fit), "Top")
})

test_that("summary.orthoMTL shows task thresholds for survival", {
  set.seed(42)
  n <- 30; p <- 5
  X <- matrix(rnorm(n * p), n, p)
  SurvTime <- rexp(n, 0.1)
  Event <- rbinom(n, 1, 0.7)
  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)

  fit <- orthoMTL(X, Y, lambda = 0.01, survival = TRUE, censored.mat = W)

  expect_output(summary(fit), "Task thresholds")
})

test_that("coef.orthoMTL returns the coefficient matrix", {
  set.seed(42)
  X <- matrix(rnorm(60), 12, 5)
  Y <- matrix(rnorm(36), 12, 3)
  fit <- orthoMTL(X, Y, lambda = 0.01)

  B <- coef(fit)
  expect_true(is.matrix(B))
  expect_identical(B, fit$B)
})

# Tests for mode-aware CV scoring (GEN-02)

test_that("cv_orthoMTL defaults to rmse for plain regression", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = c(0.001, 0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )

  expect_equal(cv_res$metric, "rmse")
  # rmse: lower is better, so results are sorted ascending
  expect_true(all(diff(cv_res$results$cv_score) >= 0))
  expect_equal(cv_res$best$cv_score, min(cv_res$results$cv_score))
})

test_that("cv_orthoMTL defaults to auc for logistic mode", {
  set.seed(42)
  n <- 60; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  lin <- X[, 1] - X[, 2]
  Y <- matrix(0, n, n_tasks)
  for (k in seq_len(n_tasks)) {
    Y[, k] <- ifelse(stats::runif(n) < 1 / (1 + exp(-lin)), 1, -1)
  }
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = c(0.001, 0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, logistic = TRUE,
    folds = folds, n_cores = 1, verbose = FALSE
  )

  expect_equal(cv_res$metric, "auc")
  # auc: higher is better, so results are sorted descending
  expect_true(all(diff(cv_res$results$cv_score) <= 0))
  expect_equal(cv_res$best$cv_score, max(cv_res$results$cv_score))
})

test_that("cv_orthoMTL survival default stays cindex", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 3
  X <- matrix(rnorm(n * p), n, p)
  SurvTime <- rexp(n, 0.1); Event <- rbinom(n, 1, 0.7)
  Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
  W <- create_indicator_matrix(Y)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y, W.train = W,
    lambdas = c(0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = TRUE, folds = folds, n_cores = 1, verbose = FALSE
  )
  expect_equal(cv_res$metric, "cindex")
})

test_that("cv_orthoMTL honours an explicit metric argument", {
  set.seed(42)
  n <- 30; p <- 5; n_tasks <- 2
  X <- matrix(rnorm(n * p), n, p)
  Y <- matrix(rnorm(n * n_tasks), n, n_tasks)
  folds <- rep(1:2, length.out = n)

  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y, metric = "r2",
    lambdas = c(0.001, 0.01), stepsizes = c(0.1), diag_vals = c(1),
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )
  expect_equal(cv_res$metric, "r2")
  expect_true(all(diff(cv_res$results$cv_score) <= 0))  # higher better
})

test_that("cv_orthoMTL rejects unknown metric", {
  set.seed(42)
  X <- matrix(rnorm(60), 30, 2)
  Y <- matrix(rnorm(60), 30, 2)
  folds <- rep(1:2, length.out = 30)
  expect_error(
    cv_orthoMTL(X.train = X, Y.train = Y, metric = "bogus",
                lambdas = 0.01, stepsizes = 0.1, diag_vals = 1,
                survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE),
    "'arg'"
  )
})

test_that("print.cv_orthoMTL shows the metric name", {
  set.seed(42)
  X <- matrix(rnorm(60), 30, 2)
  Y <- matrix(rnorm(60), 30, 2)
  folds <- rep(1:2, length.out = 30)
  cv_res <- cv_orthoMTL(
    X.train = X, Y.train = Y,
    lambdas = 0.01, stepsizes = 0.1, diag_vals = 1,
    survival = FALSE, folds = folds, n_cores = 1, verbose = FALSE
  )
  expect_output(print(cv_res), "CV rmse")
})

---
title: "Introduction to orthoMTL: Multi-Task Survival Analysis"
author: "Kevin Vervier"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to orthoMTL: Multi-Task Survival Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  fig.align = "center"
)
library(orthoMTL)
```

## The Problem

In oncology and other therapeutic areas, genomic features associated with
patient outcomes often have **time-varying effects**. A mutation that
predicts early disease progression may be irrelevant for long-term
survival — and vice versa. Standard survival models like the Cox
proportional hazards model assume that each feature's effect is constant
over time. When this assumption is violated, important signals can be
missed.

**orthoMTL** addresses this by reframing survival analysis as a
**multi-task learning** problem. Instead of fitting a single model for
time-to-event, we define binary classification tasks at multiple time
thresholds: *"Is this patient progression-free at 4 months? At 6 months?
At 10? At 15?"* Each threshold becomes a task, and the model learns a
separate coefficient vector for each — while encouraging the coefficient
vectors to be structured through an orthogonality penalty.

### The Objective Function

The optimisation problem solved by `orthoMTL()` is:

$$
\min_W \; \frac{1}{2n}\|XW - Y\|^2_{\text{obs}} \;+\;
\lambda\Bigl[\frac{1-\alpha}{2}\,\Omega_K(W)^2 \;+\; \alpha\,\|W\|_1\Bigr],
\qquad
\Omega_K(W)^2 = \sum_{s,t} K_{st}\,|W_s^\top W_t|
$$

where:

- $X$ is the $n \times p$ feature matrix
- $W$ is the $p \times T$ coefficient matrix ($T$ tasks)
- $Y$ is the $n \times T$ label matrix (1 = progression-free, 0 = event,
  NA = censored)
- $\|\cdot\|_{\text{obs}}$ means the loss is only computed on non-censored
  entries
- $K$ is a constraint matrix encoding task relationships — nearby
  thresholds share more support, distant thresholds are pushed toward
  orthogonality
- $\lambda$ controls the overall penalty strength
- $\alpha \in [0, 1]$ is the elastic-net mixing: $\alpha = 0$ gives the pure
  orthogonality penalty, $\alpha = 1$ gives a pure Lasso

## Simulated Data

We begin with simulated data where the **ground truth is known**. The
`simulate_mtl()` function generates survival data with five types of
time-varying effects: *early*, *late*, *constant*, *increasing*, and
*decreasing*.

```{r simulate}
set.seed(42)

sim <- simulate_mtl(
  n = 300,
  p = 15,
  n_signals = 5,
  thresholds = c(4, 8, 14, 20),
  effect_strength = 1.2
)
sim
```

The signal features and their true temporal effect patterns:

```{r ground-truth}
# True coefficients for signal features
gt <- sim$ground_truth
signal_coefs <- gt$coefficients[gt$signal_features, ]
signal_coefs
```

```{r ground-truth-types}
# Effect types
gt$effect_types
```

## Data Preparation

orthoMTL requires three matrices derived from the survival data.

### Longitudinal binary labels

Each patient's survival time and event indicator are converted into a
binary label at each threshold. Patients who experienced an event before
a threshold are labelled `0`. Patients censored before a threshold have
an unknown label (`NA`). All others are labelled `1` (progression-free).

```{r prepare-data}
thresholds <- sim$thresholds
Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, thresholds)
head(Y, 10)
```

### Censoring indicator matrix

A binary matrix marking which labels are observed (`1`) versus censored
(`0`). This is used by the solver to mask censored entries in the loss.

```{r indicator}
W <- create_indicator_matrix(Y)
cat("Proportion observed per threshold:\n")
colMeans(W)
```

### Constraint matrix

The diffusion constraint matrix $K$ encodes the prior that nearby
thresholds should share coefficient support while distant thresholds can
diverge.

```{r constraint}
K <- create_constraint_matrix(length(thresholds))
K
```

## Model Fitting

We fit an initial model with a single set of hyperparameters.

```{r fit-initial}
fit <- orthoMTL(
  X = sim$X, Y = Y,
  lambda = 1e-3, step_size = 0.5,
  K = K, survival = TRUE, censored.mat = W,
  alpha = 0.5
)
summary(fit)
```

The coefficient heatmap shows each feature's weight across time
thresholds. Blue indicates a protective effect (associated with longer
progression-free survival); red indicates a risk effect.

```{r heatmap-initial, fig.width=8, fig.height=6}
plot_heatmap(fit)
```

Even with untuned hyperparameters, temporal patterns begin to emerge.

## Cross-Validation

We use `cv_orthoMTL()` to search over a grid of hyperparameters and
select the configuration with the highest cross-validated C-index.

```{r cv, results='hide'}
n_tasks <- length(sim[["thresholds"]])
folds <- rep(1:5, length.out = nrow(sim[["X"]]))

cv_res <- cv_orthoMTL(
  X.train   = sim[["X"]],
  Y.train   = Y,
  W.train   = W,
  K         = K,
  lambdas   = c(1e-5, 1e-4, 1e-3),
  alphas    = c(0, 0.5, 1),
  stepsizes = c(1, 2, 5),
  diag_vals = c(0.5, n_tasks, 2 * n_tasks),
  survival  = TRUE,
  folds     = folds,
  n_cores   = 1,
  seed      = 42,
  verbose   = FALSE
)
```

```{r cv-results}
print(cv_res)
```

## Final Model

We refit on the full dataset using the best hyperparameters from
cross-validation.

```{r fit-final}
best <- cv_res$best
K_final <- K
diag(K_final) <- best$diag_val

fit_final <- orthoMTL(
  X = sim$X, Y = Y,
  lambda    = best$lambda,
  alpha     = best$alpha,
  step_size = best$stepsize,
  K         = K_final,
  survival  = TRUE,
  censored.mat = W
)
summary(fit_final)
```

```{r heatmap-final, fig.width=8, fig.height=6}
plot_heatmap(fit_final)
```

The task correlation map shows how similar the coefficient profiles are
across thresholds. Low distance (red) between adjacent thresholds
indicates smooth temporal evolution; high distance (blue) indicates
divergent coefficient structures.

```{r correlation, fig.width=6, fig.height=5}
plot_correlation(fit_final)
```

## Bootstrap Inference

To assess whether each feature's coefficients are distinguishable from
noise, we compare bootstrapped models (resampled data, real signal)
against null models (permuted outcomes, no signal).

```{r bootstrap, results='hide'}
boot_res <- bootstrap_orthoMTL(
  X = sim$X, Y = Y,
  lambda    = best$lambda,
  alpha     = best$alpha,
  step_size = best$stepsize,
  K         = K_final,
  survival  = TRUE,
  censored.mat = W,
  n_repeats = 200,
  n_cores   = 1,
  verbose   = FALSE
)
```

```{r bootstrap-print}
print(boot_res)
```

We select four features for detailed inspection: one true signal of each
pattern, and one null feature.

```{r bootstrap-select}
signal_feats <- gt[["signal_features"]]
null_feats <- gt[["null_features"]]
effect_types <- gt[["effect_types"]]

selected <- c(
  signal_feats[effect_types == "switch"][1],
  signal_feats[effect_types == "constant"][1],
  signal_feats[effect_types == "early"][1],
  null_feats[1]
)

cat("Selected features:\n")
cat("  Switch signal:  ", selected[1], "(Cox should miss)\n")
cat("  Constant signal:", selected[2], "(Cox should find)\n")
cat("  Early signal:   ", selected[3], "(Cox may dilute)\n")
cat("  Null feature:   ", selected[4], "(neither should find)\n")
```

```{r bootstrap-plot, fig.width=8, fig.height=6}
suppressWarnings(
  plot_bootstrap(boot_res, features = selected)
)
```

For the true signal features, the real coefficients (coloured line)
separate clearly from the null distribution (grey line). The temporal
patterns are visible: the *early* feature's effect is strongest at early
thresholds and fades; the *late* feature's effect emerges at later
thresholds; the *constant* feature is stable across all thresholds.

For the null feature, real and null distributions overlap — the model
correctly assigns it no meaningful effect.

## Comparison with Cox Proportional Hazards

The Cox model assumes each feature's effect is constant over time. We fit
a Cox elastic-net model using `glmnet` and compare which features it
detects.

```{r cox-comparison, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)}
library(survival)
library(glmnet)

surv_obj <- Surv(time = sim[["SurvTime"]], event = sim[["Event"]])

# Cross-validate alpha (mixing parameter)
alphas <- seq(0, 1, by = 0.1)
cv_scores <- numeric(length(alphas))

for (i in seq_along(alphas)) {
  set.seed(42)
  cvfit <- cv.glmnet(
    x = sim[["X"]],
    y = surv_obj,
    family = "cox",
    type.measure = "C",
    alpha = alphas[i]
  )
  cv_scores[i] <- cvfit[["cvm"]][cvfit[["index"]]["min", ]]
}

best_alpha <- alphas[which.max(cv_scores)]
cat("Best alpha:", best_alpha, "(CV C-index:", max(cv_scores), ")\n")

# Refit with best alpha
set.seed(42)
cox_fit <- cv.glmnet(
  x = sim[["X"]],
  y = surv_obj,
  family = "cox",
  type.measure = "C",
  alpha = best_alpha
)

cox_coefs <- as.numeric(coef(cox_fit, s = "lambda.min"))
names(cox_coefs) <- colnames(sim[["X"]])
```

```{r cox-table, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)}
# Build comparison table
ortho_mean_abs <- apply(abs(coef(fit_final)), 1, mean)

comparison <- data.frame(
  feature = gt$signal_features,
  effect_type = as.character(gt$effect_types),
  orthoMTL_mean_abs = round(ortho_mean_abs[gt$signal_features], 4),
  cox_coef = round(cox_coefs[gt$signal_features], 4),
  cox_detected = cox_coefs[gt$signal_features] != 0,
  stringsAsFactors = FALSE
)

cat("Signal feature detection comparison:\n\n")
print(comparison, row.names = FALSE)
```

The Cox proportional hazards model is a powerful and well-established
tool for survival analysis. In terms of overall discrimination (C-index),
Cox often matches or exceeds orthoMTL — it directly optimises the
survival likelihood, while orthoMTL solves a regression problem on binary
labels.

The value of orthoMTL is not in replacing Cox but in **complementing**
it. Cox produces a single coefficient per feature — an average effect
across the entire follow-up. orthoMTL produces a coefficient *per
feature per timepoint*, revealing temporal dynamics that a single number
cannot capture.

```{r cox-vs-ortho, fig.width=8, fig.height=4, eval=requireNamespace("glmnet", quietly = TRUE) && requireNamespace("survival", quietly = TRUE)}
# Pick the switch feature — most interesting temporal pattern
switch_feat <- signal_feats[effect_types == "switch"][1]

# What Cox sees: one number
cat(switch_feat, "— Cox coefficient:", 
    round(cox_coefs[switch_feat], 3), "\n")

# What orthoMTL sees: a trajectory
cat(switch_feat, "— orthoMTL coefficients:\n")
print(round(coef(fit_final)[switch_feat, ], 3))

# What the truth is
cat(switch_feat, "— True coefficients (sign-aligned):\n")
print(round(-gt[["coefficients"]][switch_feat, ], 3))
```

Cox reports a single number for this feature. orthoMTL reveals that
its effect changes direction over time — information that could
guide clinical interpretation of early versus late treatment response.

## Ground Truth Recovery

Finally, we compare the estimated coefficients against the true
data-generating coefficients for signal features.

```{r ground-truth-recovery}
true_coefs <- gt$coefficients[gt$signal_features, ]
est_coefs <- coef(fit_final)[gt$signal_features, ]

# Sign convention:
#   Simulation uses log-hazard scale (negative = protective, reduces hazard)
#   orthoMTL models P(progression-free) (positive = protective)
# Negate true coefficients to align
true_aligned <- -true_coefs

task_cors <- sapply(seq_len(ncol(true_aligned)), function(k) {
  cor(true_aligned[, k], est_coefs[, k])
})
names(task_cors) <- colnames(true_aligned)

cat("Correlation between true and estimated coefficients per threshold:\n")
print(round(task_cors, 3))
```

The simulation generates coefficients on the log-hazard scale (negative =
protective), while orthoMTL models the probability of being
progression-free (positive = protective). The true coefficients are
sign-flipped below so both heatmaps share the same interpretation:
blue = protective, red = risk-increasing.

```{r recovery-heatmap, fig.width=8, fig.height=5}
make_long <- function(mat, source_label) {
  data.frame(
    feature = rep(rownames(mat), ncol(mat)),
    threshold = rep(colnames(mat), each = nrow(mat)),
    weight = as.vector(mat),
    source = source_label,
    stringsAsFactors = FALSE
  )
}

# Normalize each matrix to [-1, 1] by dividing by its own max absolute value
normalize <- function(mat) mat / max(abs(mat), na.rm = TRUE)

combined <- rbind(
  make_long(normalize(true_aligned), "True (sign-aligned)"),
  make_long(normalize(est_coefs),    "Estimated")
)

combined$feature <- factor(combined$feature, levels = rev(rownames(true_aligned)))
combined$threshold <- factor(combined$threshold, levels = colnames(true_aligned))
combined$source <- factor(combined$source, levels = c("True (sign-aligned)", "Estimated"))



# Now both panels use the full color range
ggplot2::ggplot(combined, ggplot2::aes(threshold, feature, fill = weight)) +
  ggplot2::geom_tile() +
  ggplot2::scale_fill_gradient2(low = "red", mid = "white", high = "blue", midpoint = 0,
                       limits = c(-1, 1)) +
  ggplot2::facet_wrap(~ source) +
  ggplot2::labs(x = "Threshold (months)", y = NULL, fill = "Normalized\nCoefficient") +
  ggplot2::theme_minimal()
```

## Conclusion

This vignette demonstrated the orthoMTL workflow for survival analysis.

The key difference from standard Cox modelling is not in overall
predictive accuracy but in **interpretability**: orthoMTL reveals how
each feature's effect evolves across time thresholds. This is
particularly relevant when:

- Features have time-varying effects (e.g., early vs late resistance)
- The clinical question is not just "does this feature matter?" but
  "when does it matter?"
- Effect reversal over time (switch patterns) may be clinically
  meaningful but invisible to proportional hazards models

For the application of orthoMTL to real clinical data, see the SOLAR-1
analysis in Annals of Oncology (2026).

For details on individual functions, see the package help pages
(`?orthoMTL`, `?cv_orthoMTL`, `?bootstrap_orthoMTL`, etc.).

### Citation

If you use orthoMTL in your work, please cite:

> Vervier, K., Mahé, P., d'Aspremont, A., Veyrieras, J.-B., & Vert, J.-P.
> (2014). On Learning Matrices with Orthogonal Columns or Disjoint Supports.
> *ECML-PKDD 2014*. <https://hal.science/hal-00985654>

For the survival extension:

> Annals of Oncology (2026). DOI:
> [10.1016/j.annonc.2026.04.003](https://doi.org/10.1016/j.annonc.2026.04.003)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proj_disjoint.R
\name{proj_disjoint}
\alias{proj_disjoint}
\title{Projection onto Disjoint Support Feasible Set}
\usage{
proj_disjoint(w, v)
}
\arguments{
\item{w}{Numeric matrix of weights used in the loss function.}

\item{v}{Numeric matrix of constraints used for projection.}
}
\value{
A list with projected matrices \code{w} and \code{v}.
}
\description{
Projects weight and constraint matrices ...
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_prediction}
\alias{plot_prediction}
\title{Prediction Swimmer Plot for orthoMTL}
\usage{
plot_prediction(x)
}
\arguments{
\item{x}{A numeric prediction matrix with dimensions
\code{n x numTasks}, typically produced by
\code{\link{predict.orthoMTL}}.}
}
\value{
A \code{ggplot2} object.
}
\description{
Displays a prediction matrix as a heatmap (swimmer plot), where
rows are patients and columns are tasks/thresholds.
}
\examples{
set.seed(42)
n <- 50; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
colnames(Y) <- c("T1", "T2", "T3")
K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)

preds <- predict(fit, newdata = X)
plot_prediction(preds)
}
\seealso{
\code{\link{predict.orthoMTL}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nnmaxheap.R
\name{nnmaxheap_C}
\alias{nnmaxheap_C}
\title{Project a Vector onto Non-Negative Non-Increasing Space}
\usage{
nnmaxheap_C(m)
}
\arguments{
\item{m}{A numeric vector to project.}
}
\value{
A numeric vector of the same length as \code{m}, projected
  onto the non-negative non-increasing constraint space.
}
\description{
Applies an isotonic regression-style projection to enforce that the
output vector is non-negative and non-increasing. Used internally
by \code{\link{predict.orthoMTL}} to ensure survival predictions
are monotonically decreasing across time thresholds.
}
\details{
This function implements a pool-adjacent-violators style algorithm.
It replaces the external \code{Iso} package dependency used in the
predecessor \code{orthopen} package.
}
\examples{
# Project a vector onto the non-negative non-increasing space
nnmaxheap_C(c(3, 1, 2, -1))

# Already valid input: returned unchanged
nnmaxheap_C(c(5, 3, 3, 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_bootstrap}
\alias{plot_bootstrap}
\title{Bootstrap Coefficient Comparison Plot}
\usage{
plot_bootstrap(x, features = NULL, batch_size = 9)
}
\arguments{
\item{x}{Either a \code{"bootstrap_orthoMTL"} object (as returned
by \code{\link{bootstrap_orthoMTL}}) or a \code{data.frame} with
columns \code{id}, \code{time}, \code{coeff}, and \code{group}.}

\item{features}{A character vector of feature names to plot. If
\code{NULL} (default), all features are plotted.}

\item{batch_size}{Number of features per facet page. Default:
\code{9}.}
}
\value{
A list of \code{ggplot2} objects, one per batch/page.
}
\description{
Displays faceted line plots comparing real (bootstrapped) vs null
(permuted) coefficient trajectories across tasks for selected or
all features. Each panel shows the mean and standard error of the
coefficient at each task/threshold.
}
\details{
Blue lines show coefficients from models fitted on bootstrapped
(resampled) data — reflecting estimation variability under real
signal. Grey lines show coefficients from models fitted on
permuted outcomes — reflecting the null distribution.

When the blue (real) band is clearly separated from the grey
(null) band, the feature's coefficient is distinguishable from
noise at that task/threshold.
}
\examples{
\donttest{
set.seed(42)
n <- 30; p <- 5; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
SurvTime <- rexp(n, rate = 0.1)
Event <- rbinom(n, 1, 0.7)
thresholds <- c(4, 6, 10)
Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
W <- create_indicator_matrix(Y)
K <- create_constraint_matrix(n_tasks)
boot_res <- bootstrap_orthoMTL(
  X = X, Y = Y, lambda = 1e-3, step_size = 0.1,
  K = K, survival = TRUE, censored.mat = W,
  n_repeats = 5, n_cores = 1, verbose = FALSE
)
plots <- plot_bootstrap(boot_res)
plots[[1]]
}
}
\seealso{
\code{\link{bootstrap_orthoMTL}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orthoMTL.R
\name{orthoMTL}
\alias{orthoMTL}
\title{Multi-task learning with orthogonal constraints}
\usage{
orthoMTL(
  X,
  Y,
  lambda = 1,
  step_size = 0.1,
  tol = 1e-05,
  stop_no_improve = 100,
  max_iter = 1e+06,
  W_0 = NULL,
  seed = NULL,
  K = NULL,
  disjoint = FALSE,
  logistic = FALSE,
  alpha = 0,
  schedule = c("sqrt", "log", "const", "linear"),
  survival = FALSE,
  censored.mat = NULL,
  verbose = 0
)
}
\arguments{
\item{X}{a matrix of predictor variables with dimensions n x p}

\item{Y}{a matrix of response variables with dimensions n x numTasks, where numTasks is the number of response variables. NAs can be used for censored data.}

\item{lambda}{the regularization parameter for the OrthoPen penalty, default is 1}

\item{step_size}{the step size for updating the regression coefficients in gradient descent, default is 0.1}

\item{tol}{Convergence tolerance. The algorithm stops when the
improvement in the objective function is less than \code{tol}.
Default: \code{1e-5}.}

\item{stop_no_improve}{the number of iterations without improvement in the objective function to trigger convergence, default is 100}

\item{max_iter}{the maximum number of iterations, default is 1e+06}

\item{W_0}{a matrix of initial values for the regression coefficients, default is NULL, as not provided and will be randomly attributed}

\item{seed}{an optional random seed for reproducibility, default is
\code{NULL} (no seed is set; the ambient RNG state is used as-is).
Only relevant when \code{W_0} is not provided and \code{disjoint = TRUE},
since that is the only case where \code{orthoMTL} draws random numbers.
Pass an integer to make that random initialisation reproducible.}

\item{K}{a constraint matrix of weights to adjust the OrthoPen penalty, default is an identity matrix with dimensions numTasks x numTasks}

\item{disjoint}{a logical value indicating whether the response variables should have disjoint supports, default is FALSE}

\item{logistic}{a logical value indicating whether logistic regression should be used instead of linear regression, default is FALSE}

\item{alpha}{the elastic-net mixing parameter in \eqn{[0, 1]}, default is 0.
The penalty is \eqn{\lambda [(1 - \alpha)/2\, \Omega_K(W)^2 + \alpha \|W\|_1]};
\code{alpha = 0} gives the pure orthogonality penalty and \code{alpha = 1}
gives a pure Lasso.}

\item{schedule}{Character; the gradient-step decay schedule -- how the
per-iteration scale grows with the iteration index \eqn{i}. The coefficient
update is \eqn{W \leftarrow W - \texttt{step\_size}\, \nabla /
(\texttt{scale} \cdot \|\nabla\|)}, so a faster-growing \code{scale} means a
faster-decaying effective step. One of:
\describe{
  \item{\code{"sqrt"}}{(default) \eqn{\texttt{scale} = \sqrt{i}}, effective
    step \eqn{\propto 1/\sqrt{i}} -- the classic subgradient schedule and
    the historical hardcoded behaviour.}
  \item{\code{"log"}}{\eqn{\texttt{scale} = \log(i) + 1} (slow decay).}
  \item{\code{"const"}}{\eqn{\texttt{scale} = 1} (no decay).}
  \item{\code{"linear"}}{\eqn{\texttt{scale} = i}, effective step
    \eqn{\propto 1/i} (Robbins-Monro).}
}
The default \code{"sqrt"} is retained for backward compatibility: it
reproduces previously published results exactly. An A/B study shipped with
the package (\code{validation/ab_s02_gradient_schedule.R}) found that
\code{"sqrt"} reaches the \emph{correct} optimum but converges to it roughly
12--17x slower than \code{"log"} and \code{"const"} (which reach the same
objective in a small fraction of the iterations), while \code{"linear"}
decays too aggressively and can stall short of the optimum. Prefer
\code{"log"} or \code{"const"} when convergence speed matters; note that
changing the schedule changes the optimisation path and therefore the exact
coefficients, so it should not be altered when reproducing published runs.}

\item{survival}{a logical value indicating whether survival analysis should be performed, default is FALSE}

\item{censored.mat}{a matrix indicating whether observations are censored, used only if survival=TRUE}

\item{verbose}{the level of verbosity, default is 0 (no messages)}
}
\value{
a list containing the following elements:
\item{B}{a matrix of regression coefficients with dimensions p x numTasks}
\item{obj}{the final objective function when algorithms stops}
\item{imax}{the number of iterations}
}
\description{
This function solves a multi-task problem where relationships between tasks can be complex
}
\examples{
# Regression with orthogonal columns
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
W_true <- qr.Q(qr(matrix(rnorm(p * n_tasks), p, n_tasks)))
Y <- X \%*\% W_true + matrix(rnorm(n * n_tasks), n) * 0.1
K <- matrix(1, n_tasks, n_tasks)
diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K, disjoint = FALSE)
fit$B           # coefficient matrix
fit$converged   # did optimisation converge?

}
\references{
Kevin Vervier, Pierre Mahé, Alexandre d’Aspremont, Jean-Baptiste Veyrieras, Jean-Philippe Vert (2014). On learning matrices with orthogonal columns or disjoint supports. https://hal.science/hal-00985654/file/learningDisjointSupports.pdf
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{accuracy_mtl}
\alias{accuracy_mtl}
\title{Classification Accuracy for Multi-Task Predictions}
\usage{
accuracy_mtl(true.label.mat, pred.label.mat, threshold = 0)
}
\arguments{
\item{true.label.mat}{A numeric matrix of true responses
(\code{n x numTasks}). \code{NA} cells are ignored.}

\item{pred.label.mat}{A numeric matrix of predictions of the same
dimensions (as produced by \code{\link{predict.orthoMTL}}).}

\item{threshold}{Decision threshold applied to \code{pred.label.mat}.
Default \code{0} matches raw link scores; pass \code{0.5} for
probabilities from \code{predict(..., type = "response")}.}
}
\value{
A single numeric value in \code{[0, 1]} (higher is better).
}
\description{
Pooled fraction of correctly classified non-\code{NA} cells, for
\code{orthoMTL} fits in classification mode (\code{logistic = TRUE}).
A cell is positive when its true label is \code{> 0} and predicted
positive when its score is \code{> 0}. This works for both the
\eqn{\{-1, +1\}} encoding the solver optimises and a \eqn{\{0, 1\}}
encoding, and for raw \code{"link"} scores or \code{"response"}
probabilities (threshold 0.5 corresponds to a link of 0... see note).
}
\examples{
set.seed(1)
Y <- matrix(sample(c(-1, 1), 30, replace = TRUE), 10, 3)
P <- Y * abs(matrix(rnorm(30), 10, 3))   # mostly correct signs
accuracy_mtl(Y, P)
}
\seealso{
\code{\link{auc_mtl}}, \code{\link{cindex_mtl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constraint_matrices.R
\name{create_constraint_matrix}
\alias{create_constraint_matrix}
\title{Create Diffusion Constraint Matrix}
\usage{
create_constraint_matrix(numTasks, diag_val = 0.5)
}
\arguments{
\item{numTasks}{An integer specifying the number of tasks.}

\item{diag_val}{A numeric value for the diagonal of \code{K}.
Default: \code{0.5}. This value is typically overridden during
cross-validation (see \code{\link{cv_orthoMTL}}).}
}
\value{
A numeric square matrix of dimensions
  \code{numTasks x numTasks}. Off-diagonal entry \code{K[i,j]}
  is larger when tasks \code{i} and \code{j} are further apart.
  Diagonal entries are set to \code{diag_val}.
}
\description{
Builds a square constraint matrix \code{K} encoding the prior belief
that temporally distant tasks should have more orthogonal coefficients.
Off-diagonal entries accumulate weight proportional to the distance
between task indices, implementing a diffusion-like pattern.
}
\details{
This matrix is passed to \code{\link{orthoMTL}} via the \code{K}
argument to control the orthogonality penalty between tasks.


The construction rule: for each distance level \code{h} from
\code{1} to \code{numTasks}, add \code{1} to all entries where
\code{|row - col| > h}. This produces a matrix where nearby tasks
(adjacent thresholds) share more support, while distant tasks
are pushed toward orthogonality.

For survival analysis with time thresholds, this encodes the
assumption that the set of predictive features changes gradually
over time rather than abruptly.
}
\examples{
# 5-task constraint matrix
K <- create_constraint_matrix(5)
K

# Override diagonal for a specific penalty balance
K <- create_constraint_matrix(7, diag_val = 6)
K
}
\seealso{
\code{\link{cv_orthoMTL}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preparation.R
\name{create_indicator_matrix}
\alias{create_indicator_matrix}
\title{Create Censoring Indicator Matrix}
\usage{
create_indicator_matrix(Y)
}
\arguments{
\item{Y}{A numeric matrix of dimensions \code{n x numTasks}, typically
produced by \code{\link{create_longitudinal_labels}}. May contain
\code{NA} values for censored observations.}
}
\value{
A numeric matrix of the same dimensions as \code{Y}.
  \code{1} = label is observed (known), \code{0} = label is censored
  (unknown).
}
\description{
Converts \code{NA} entries in a longitudinal label matrix (as produced
by \code{\link{create_longitudinal_labels}}) into a binary indicator
matrix. This indicator is used by \code{\link{orthoMTL}} to mask
censored observations in the loss computation.
}
\examples{
set.seed(42)
SurvTime <- rexp(10, rate = 0.1)
Event <- rbinom(10, 1, 0.7)
Y <- create_longitudinal_labels(SurvTime, Event, c(4, 6, 10))
W <- create_indicator_matrix(Y)
W   # 1 = observed, 0 = censored
}
\seealso{
\code{\link{create_longitudinal_labels}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.orthoMTL}
\alias{print.orthoMTL}
\title{Print an orthoMTL Object}
\usage{
\method{print}{orthoMTL}(x, ...)
}
\arguments{
\item{x}{A fitted model object of class \code{"orthoMTL"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a compact summary of a fitted \code{orthoMTL} model.
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
fit <- orthoMTL(X, Y, lambda = 1e-3)
print(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap_print.R
\name{print.bootstrap_orthoMTL}
\alias{print.bootstrap_orthoMTL}
\title{Print Bootstrap Inference Results}
\usage{
\method{print}{bootstrap_orthoMTL}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"bootstrap_orthoMTL"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a summary of bootstrap inference results from
\code{\link{bootstrap_orthoMTL}}.
}
\examples{
# See ?bootstrap_orthoMTL for a full example
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.orthoMTL}
\alias{summary.orthoMTL}
\title{Summarise an orthoMTL Object}
\usage{
\method{summary}{orthoMTL}(object, n_top = 5, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"orthoMTL"}.}

\item{n_top}{Number of top features to display. Default: 5.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{object}.
}
\description{
Displays a detailed summary of a fitted \code{orthoMTL} model,
including hyperparameters, coefficient matrix statistics, and
top features by mean absolute coefficient.
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
fit <- orthoMTL(X, Y, lambda = 1e-3)
summary(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.orthoMTL}
\alias{predict.orthoMTL}
\title{Predict from an orthoMTL Model}
\usage{
\method{predict}{orthoMTL}(object, newdata, type = c("link", "response", "class"), ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"orthoMTL"}.}

\item{newdata}{A numeric matrix of new observations with dimensions
\code{n_new x p}. Column names are used for feature alignment if
available in the fitted object.}

\item{type}{Character; scale of the returned predictions. One of:
\describe{
  \item{\code{"link"}}{(default) the raw linear predictor
    \eqn{X B} (after the survival monotonicity projection, if
    applicable). Preserves the historical behaviour.}
  \item{\code{"response"}}{for \code{logistic} fits, the sigmoid
    \eqn{1 / (1 + e^{-XB})} giving \eqn{P(Y = +1)}; for non-logistic
    fits identical to \code{"link"}.}
  \item{\code{"class"}}{for \code{logistic} fits, the predicted
    class label in \eqn{\{-1, +1\}} (\code{sign(XB)}, with 0 mapped
    to \code{+1}). Errors for non-logistic fits.}
}}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric matrix of predictions with dimensions
  \code{n_new x n_tasks}. Column names correspond to task names
  if available.
}
\description{
Generate predictions from a fitted \code{orthoMTL} model for new
observations. In survival mode, predictions are projected onto the
non-negative non-increasing space to ensure monotonicity across tasks.
}
\details{
If the fitted object contains \code{feature_names} (i.e., the training
matrix \code{X} had column names), \code{newdata} columns are aligned
to match. Missing features cause an error. Extra features trigger a
warning and are dropped.

In survival mode (\code{object$hyperparameters$survival == TRUE}),
each row of the raw prediction matrix is projected via
\code{nnmaxheap_C()} to enforce non-negative, non-increasing values
across tasks (time thresholds).
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
W_true <- qr.Q(qr(matrix(rnorm(p * n_tasks), p, n_tasks)))
Y <- X \%*\% W_true + matrix(rnorm(n * n_tasks), n) * 0.1
K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K, disjoint = FALSE)

X_new <- matrix(rnorm(20 * p), 20, p)
colnames(X_new) <- paste0("V", seq_len(p))
preds <- predict(fit, newdata = X_new)
dim(preds)
}
\references{
Vervier, K., Mahe, P., d'Aspremont, A., Veyrieras, J.-B., and
Vert, J.-P. (2014). On Learning Matrices with Orthogonal Columns
or Disjoint Supports. \emph{ECML-PKDD 2014}.
\url{https://hal.science/hal-00985654}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_print.R
\name{print.cv_orthoMTL}
\alias{print.cv_orthoMTL}
\title{Print Cross-Validation Results}
\usage{
\method{print}{cv_orthoMTL}(x, n_top = 5, ...)
}
\arguments{
\item{x}{An object of class \code{"cv_orthoMTL"}.}

\item{n_top}{Number of top configurations to display. Default: 5.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a summary of cross-validation results from
\code{\link{cv_orthoMTL}}.
}
\examples{
# See ?cv_orthoMTL for a full example
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{cindex_mtl}
\alias{cindex_mtl}
\title{Concordance Index for Multi-Task Predictions}
\usage{
cindex_mtl(true.label.mat, pred.label.mat)
}
\arguments{
\item{true.label.mat}{A numeric matrix of true response labels with
dimensions \code{n x numTasks}. May contain \code{NA} for censored
observations (as produced by \code{\link{create_longitudinal_labels}}).}

\item{pred.label.mat}{A numeric matrix of predicted response values
with dimensions \code{n x numTasks} (as produced by
\code{\link{predict.orthoMTL}}).}
}
\value{
A numeric value between 0 and 1 (higher is better).
  A value of 0.5 indicates random concordance.
}
\description{
Computes a concordance index (C-index) adapted for the multi-task
survival framework. Predictions across tasks are aggregated by
row-sum to produce a single score per patient, then concordance
is evaluated over pairs where at least one member has a fully
observed outcome.
}
\details{
The effective survival time for each patient is derived as the
highest task index where the true label is positive (i.e., the
last threshold at which the patient was progression-free).

A patient is considered "uncensored" only if all task labels are
non-\code{NA}. Concordant pairs require both a correct ordering
of effective survival times and a matching ordering of predicted
scores.
}
\section{Known limitations}{

The following limitations are documented and flagged for future
investigation:
\itemize{
  \item No credit for tied predictions or tied survival times
  \item Strict censoring: uncensored requires all tasks observed
  \item Equal weighting of all tasks in the row-sum aggregation
}
}

\examples{
# Simulate a small multi-task prediction scenario
set.seed(42)
n <- 30; n_tasks <- 4
SurvTime <- rexp(n, rate = 0.1)
Event <- rbinom(n, 1, 0.7)
thresholds <- c(4, 6, 10, 15)

Y <- create_longitudinal_labels(SurvTime, Event, thresholds)

# Simulate imperfect predictions (add noise to true labels)
pred <- Y
pred[is.na(pred)] <- 0.5
pred <- pred + matrix(rnorm(n * n_tasks, sd = 0.3), n, n_tasks)

cindex_mtl(Y, pred)
}
\seealso{
\code{\link{predict.orthoMTL}} for generating the
  prediction matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate_mtl}
\alias{simulate_mtl}
\title{Simulate Multi-Task Data with Time-Varying Effects}
\usage{
simulate_mtl(
  n = 200,
  p = 30,
  n_signals = 5,
  n_continuous = 1,
  thresholds = c(4, 6, 10, 15),
  mode = c("survival", "regression", "classification"),
  noise_sd = 1,
  censoring_max = 25,
  baseline_hazard = 0.05,
  effect_strength = 0.8,
  treatment_effect = -0.3,
  seed = NULL
)
}
\arguments{
\item{n}{Number of patients. Default: \code{200}.}

\item{p}{Number of features excluding the treatment column.
Default: \code{30}.}

\item{n_signals}{Number of features with true non-zero effects.
Default: \code{5}. Must be \code{<= p}.}

\item{n_continuous}{Number of continuous features (placed first
in the feature matrix). Default: \code{1}. Remaining features
are binary.}

\item{thresholds}{Numeric vector of time thresholds defining the
task structure. Default: \code{c(4, 6, 10, 15)}.}

\item{mode}{Character; the response type to generate. One of
\code{"survival"} (default), \code{"regression"}, or
\code{"classification"}. The feature matrix and ground-truth
coefficients are generated identically across modes; only the
response differs:
\describe{
  \item{survival}{piecewise-exponential \code{SurvTime}/\code{Event}
    (the historical behaviour).}
  \item{regression}{\code{Y = X \%*\% beta + N(0, noise_sd^2)},
    returned as the \code{n x length(thresholds)} matrix \code{Y}.}
  \item{classification}{binary labels in \eqn{\{-1, +1\}} sampled
    from \eqn{P(Y = +1) = 1 / (1 + e^{-X beta})}, returned as
    \code{Y}. This matches the label encoding \code{orthoMTL}
    optimises with \code{logistic = TRUE}.}
}}

\item{noise_sd}{Standard deviation of the Gaussian noise added in
\code{mode = "regression"}. Ignored otherwise. Default: \code{1}.}

\item{censoring_max}{Maximum censoring time. Censoring times are
drawn from \code{Unif(0, censoring_max)}. Default: \code{25}.}

\item{baseline_hazard}{Baseline hazard rate per interval. Default:
\code{0.05}.}

\item{effect_strength}{Multiplier controlling the magnitude of
feature effects. Default: \code{0.5}.}

\item{treatment_effect}{Effect of treatment on the log-hazard
(negative = protective). Default: \code{-0.3}.}

\item{seed}{Optional random seed for reproducibility. Default:
\code{NULL} (no seed is set; the ambient RNG state is used as-is).
Pass an integer to make the simulated data reproducible.}
}
\value{
An object of class \code{"simulated_mtl"} containing:
  \describe{
    \item{mode}{The response type generated.}
    \item{Y}{For \code{mode = "regression"} / \code{"classification"},
      the \code{n x length(thresholds)} response matrix. \code{NULL}
      in survival mode (use \code{SurvTime}/\code{Event} instead).}
    \item{SurvTime, Event}{Survival mode only (\code{NULL} otherwise):
      observed times and event indicators.}
    \item{X}{Numeric matrix of dimensions \code{n x (p + 1)}.
      Columns include \code{n_continuous} continuous features
      (standard normal), \code{p - n_continuous} binary features
      (prevalences drawn from \code{Beta(2, 10)}), and a
      \code{treatment} column (balanced 1:1).}
    \item{SurvTime}{Numeric vector of observed survival times.}
    \item{Event}{Binary vector: \code{1} = event observed,
      \code{0} = censored.}
    \item{treatment}{Binary vector (also present as last column
      of \code{X}).}
    \item{feature_names}{Character vector of column names of
      \code{X}.}
    \item{ground_truth}{A list containing:
      \describe{
        \item{coefficients}{Matrix of true coefficients with
          dimensions \code{(p + 1) x length(thresholds)}.}
        \item{signal_features}{Names of features with non-zero
          effects.}
        \item{null_features}{Names of features with zero effects.}
        \item{effect_types}{Named character vector mapping signal
          features to their temporal effect template.}
        \item{thresholds}{The thresholds used.}
        \item{baseline_hazard}{The baseline hazard used.}
      }
    }
    \item{n, p, n_signals, n_continuous, thresholds, seed}{Input
      parameters stored for reference.}
    \item{call}{The matched function call.}
  }
}
\description{
Generates a realistic simulated dataset with binary and continuous
features and a known ground-truth coefficient structure. The
\code{mode} argument selects the response type: piecewise-exponential
\strong{survival} times (default), multi-task \strong{regression}
targets, or multi-task binary \strong{classification} labels. Designed
for demonstrating and testing \code{\link{orthoMTL}}.
}
\details{
\strong{Feature structure:}
\itemize{
  \item Continuous features are drawn from \code{N(0, 1)}.
  \item Binary features have prevalences drawn from \code{Beta(2, 10)},
    producing a realistic range (~5-30\%).
  \item Treatment is balanced 1:1 via random assignment.
}

\strong{Effect templates:}
Each signal feature is assigned one of five temporal patterns:
\itemize{
  \item \code{"early"}: strong effect at early thresholds, fading to
    zero at late.
  \item \code{"late"}: zero at early thresholds, emerging at late.
  \item \code{"constant"}: equal effect across all thresholds
    (detectable by standard Cox models).
  \item \code{"increasing"}: effect grows over time.
  \item \code{"decreasing"}: effect shrinks over time.
}
Templates are assigned cyclically across signal features with
random sign (risk-increasing or protective).

\strong{Survival time generation:}
Uses a piecewise-exponential model where the hazard in each
interval is
\code{h_k(i) = baseline_hazard * exp(X[i, ] \%*\% beta[, k])}.
Patients progress through intervals sequentially; an event
occurs when the simulated time within an interval is shorter
than the interval width.

\strong{Censoring:}
Independent of event times. \code{C ~ Unif(0, censoring_max)}.
Observed time = \code{min(T, C)}, event indicator = \code{T <= C}.
}
\examples{
# Generate simulated data
sim <- simulate_mtl(n = 100, p = 20, n_signals = 4, seed = 42)
sim

# Inspect ground truth
sim$ground_truth$signal_features
sim$ground_truth$effect_types
sim$ground_truth$coefficients[sim$ground_truth$signal_features, ]

# Use in orthoMTL workflow
\donttest{
thresholds <- c(4, 6, 10, 15)
Y <- create_longitudinal_labels(sim$SurvTime, sim$Event, thresholds)
W <- create_indicator_matrix(Y)
K <- create_constraint_matrix(length(thresholds))

fit <- orthoMTL(sim$X, Y, lambda = 1e-3, K = K,
                survival = TRUE, censored.mat = W)
summary(fit)
}
}
\seealso{
\code{\link{create_longitudinal_labels}},
  \code{\link{orthoMTL}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preparation.R
\name{create_longitudinal_labels}
\alias{create_longitudinal_labels}
\title{Convert Survival Data to Longitudinal Binary Labels}
\usage{
create_longitudinal_labels(SurvTime, Event, thresholds = c(4, 6))
}
\arguments{
\item{SurvTime}{A numeric vector of length \code{n} containing
observed survival times. Must be non-negative.}

\item{Event}{A numeric vector of length \code{n} encoding censoring
status. \code{1} = event observed, \code{0} = censored.}

\item{thresholds}{A numeric vector of length \code{numTasks}
representing the time thresholds at which to evaluate survival
status. Default: \code{c(4, 6)}.}
}
\value{
A numeric matrix of dimensions \code{n x numTasks}. Column
  names are set to the threshold values. Contains \code{1}, \code{0},
  and \code{NA} values as described above.
}
\description{
Transforms time-to-event survival data into a matrix of binary labels
at user-defined time thresholds. This is the core data transformation
that enables survival analysis within the multi-task learning framework.
}
\details{
The encoding logic for each patient at each threshold is:
\itemize{
  \item \code{1} — patient is progression-free at this threshold
    (survival time exceeds threshold, regardless of event status)
  \item \code{0} — patient experienced an event before this threshold
    (survival time < threshold AND event observed)
  \item \code{NA} — patient was censored before this threshold
    (survival time < threshold AND no event observed); label is unknown
}
}
\examples{
# Simulate 10 patients
set.seed(42)
SurvTime <- rexp(10, rate = 0.1)
Event <- rbinom(10, 1, 0.7)
thresholds <- c(4, 6, 10, 15)

Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
Y   # 1 = progression-free, 0 = event, NA = censored
}
\seealso{
\code{\link{create_indicator_matrix}} to convert \code{NA}
  values into a binary censoring indicator matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap_orthoMTL}
\alias{bootstrap_orthoMTL}
\title{Bootstrap Inference for orthoMTL Coefficients}
\usage{
bootstrap_orthoMTL(
  X,
  Y,
  lambda = 1,
  alpha = 0,
  step_size = 0.1,
  K = NULL,
  disjoint = FALSE,
  schedule = c("sqrt", "log", "const", "linear"),
  survival = FALSE,
  censored.mat = NULL,
  n_repeats = 100,
  n_cores = 2,
  seed = NULL,
  verbose = TRUE
)
}
\arguments{
\item{X}{A numeric matrix of predictor variables with dimensions
\code{n x p}.}

\item{Y}{A numeric matrix of response labels with dimensions
\code{n x numTasks}. May contain \code{NA} for censored
observations.}

\item{lambda}{Regularisation parameter for the orthogonal penalty.}

\item{alpha}{Elastic-net mixing parameter in \eqn{[0, 1]}, passed
through to \code{\link{orthoMTL}}. Default: \code{0}.}

\item{step_size}{Step size for gradient descent. Default: \code{0.1}.}

\item{K}{Constraint matrix of dimensions
\code{numTasks x numTasks}. Default: \code{NULL} (identity).}

\item{disjoint}{Logical. Enforce disjoint supports? Default:
\code{FALSE}.}

\item{schedule}{Character; the gradient-step decay schedule passed to
\code{\link{orthoMTL}} for every resample/permutation fit. One of
\code{"sqrt"} (default), \code{"log"}, \code{"const"}, \code{"linear"}.
Because bootstrap refits the model many times, switching to \code{"log"} or
\code{"const"} -- which reach the same optimum in far fewer iterations --
can substantially cut total runtime; see the \code{schedule} argument of
\code{\link{orthoMTL}} for details.}

\item{survival}{Logical. Use censored survival loss? Default:
\code{FALSE}.}

\item{censored.mat}{A numeric indicator matrix of dimensions
\code{n x numTasks}. Required when \code{survival = TRUE}.}

\item{n_repeats}{Number of bootstrap/permutation repeats. Default:
\code{100}.}

\item{n_cores}{Number of cores for parallel execution. Default:
\code{2}.}

\item{seed}{Optional base random seed. Default: \code{NULL} (no seed
is set; repeats vary via the ambient RNG state and the run is not
reproducible). If supplied, repeat \code{i} uses
\code{set.seed(seed + i)} for \code{i} in \code{1:n_repeats}, so the
whole run becomes reproducible while still varying across repeats.}

\item{verbose}{Logical. Print progress information? Default:
\code{TRUE}.}
}
\value{
An object of class \code{"bootstrap_orthoMTL"} containing:
  \describe{
    \item{results}{A tidy \code{data.frame} with columns \code{id}
      (feature name), \code{time} (task/threshold), \code{coeff}
      (coefficient value), \code{group} (\code{"real"} or
      \code{"null"}), and \code{repeat_id} (integer).}
    \item{coefficients_real}{A list of \code{n_repeats} coefficient
      matrices (each \code{p x numTasks}).}
    \item{coefficients_null}{A list of \code{n_repeats} coefficient
      matrices from permuted outcomes.}
    \item{obj_real}{Numeric vector of final objective values for
      real bootstrap models.}
    \item{obj_null}{Numeric vector of final objective values for
      null permutation models.}
    \item{n_repeats}{Number of repeats.}
    \item{n_features}{Number of features.}
    \item{n_tasks}{Number of tasks.}
    \item{feature_names}{Character vector of feature names.}
    \item{task_names}{Character vector of task names.}
    \item{call}{The matched function call.}
  }
}
\description{
Estimates coefficient variability and statistical relevance by
comparing bootstrapped models (real signal) against null models
(permuted outcomes). This two-pronged approach answers: (1) how
stable is each coefficient across resamples, and (2) is each
coefficient distinguishable from what would be obtained by chance.
}
\details{
\strong{Real bootstrap}: For each repeat, rows of \code{X}, \code{Y},
and \code{censored.mat} are resampled \emph{with replacement}. The
model is refit with identical hyperparameters. This produces a
distribution of coefficient values reflecting estimation variability.

\strong{Null permutation}: For each repeat, rows of \code{Y} and
\code{censored.mat} are permuted \emph{without replacement} while
\code{X} remains fixed. This breaks the association between features
and outcomes, producing a null distribution of coefficients.

Comparing real vs null distributions for each feature and task
indicates whether observed coefficients are distinguishable from
noise. Visualise with \code{\link{plot_bootstrap}}.
}
\examples{
\donttest{
set.seed(42)
n <- 30; p <- 5; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
SurvTime <- rexp(n, rate = 0.1)
Event <- rbinom(n, 1, 0.7)
thresholds <- c(4, 6, 10)

Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
W <- create_indicator_matrix(Y)
K <- create_constraint_matrix(n_tasks)

boot_res <- bootstrap_orthoMTL(
  X = X, Y = Y, lambda = 1e-3, step_size = 0.1,
  K = K, survival = TRUE, censored.mat = W,
  n_repeats = 5, n_cores = 1, verbose = FALSE
)

print(boot_res)
head(boot_res$results)
}
}
\seealso{
\code{\link{orthoMTL}}, \code{\link{plot_bootstrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_orthoMTL.R
\name{cv_orthoMTL}
\alias{cv_orthoMTL}
\title{Cross-Validation for orthoMTL Hyperparameter Selection}
\usage{
cv_orthoMTL(
  X.train,
  Y.train,
  W.train = NULL,
  K = NULL,
  lambdas = c(0.001, 0.01),
  alphas = 0,
  stepsizes = c(0.1, 0.5),
  diag_vals = c(0.5, 1),
  survival = TRUE,
  logistic = FALSE,
  metric = NULL,
  disjoint = FALSE,
  schedule = c("sqrt", "log", "const", "linear"),
  folds = NULL,
  n_cores = 2,
  seed = NULL,
  verbose = TRUE
)
}
\arguments{
\item{X.train}{A numeric matrix of training features with dimensions
\code{n x p}.}

\item{Y.train}{A numeric matrix of training labels with dimensions
\code{n x numTasks}. May contain \code{NA} for censored observations.}

\item{W.train}{A numeric indicator matrix of dimensions
\code{n x numTasks}, where \code{1} = observed and \code{0} = censored.
Required when \code{survival = TRUE}.}

\item{K}{A square constraint matrix of dimensions
\code{numTasks x numTasks}. The diagonal will be overridden by
values in \code{diag_vals} during the search.}

\item{lambdas}{A numeric vector of regularisation parameters to search.}

\item{alphas}{A numeric vector of elastic-net mixing parameters in
\eqn{[0, 1]} to search. Default: \code{0} (pure orthogonality
penalty, no sparsity).}

\item{stepsizes}{A numeric vector of gradient descent step sizes
to search.}

\item{diag_vals}{A numeric vector of diagonal values for the
constraint matrix \code{K} to search.}

\item{survival}{Logical. Use censored survival loss? Default:
\code{TRUE}.}

\item{logistic}{Logical. Fit logistic (classification) models?
Passed through to \code{\link{orthoMTL}}. Default: \code{FALSE}.}

\item{metric}{Character; the scoring metric maximised/minimised over
the grid, or \code{NULL} (default) to choose automatically by mode:
\code{"cindex"} when \code{survival = TRUE}, \code{"auc"} when
\code{logistic = TRUE}, otherwise \code{"rmse"}. Supported values:
\code{"cindex"}, \code{"auc"}, \code{"accuracy"} (higher is better),
\code{"rmse"} (lower is better), \code{"r2"} (higher is better).}

\item{disjoint}{Logical. Enforce disjoint supports? Default:
\code{FALSE}.}

\item{schedule}{Character; the gradient-step decay schedule passed to
\code{\link{orthoMTL}} for every fit. One of \code{"sqrt"} (default),
\code{"log"}, \code{"const"}, \code{"linear"}. \code{"log"} or
\code{"const"} typically reach the same optimum in far fewer iterations
than the default \code{"sqrt"}, which can noticeably speed up the grid
search; see the \code{schedule} argument of \code{\link{orthoMTL}} for the
trade-offs. Applied uniformly to all configurations (it is not part of the
tuning grid).}

\item{folds}{An integer vector of length \code{n} assigning each
training observation to a fold. If \code{NULL}, a 5-fold
assignment is generated with a warning.}

\item{n_cores}{Integer. Number of cores for parallel execution.
Default: \code{2}.}

\item{seed}{Optional integer random seed for reproducibility. Default:
\code{NULL} (no seed is set; the ambient RNG state is used as-is).
Used both for auto-generated fold assignment and for each
\code{\link{orthoMTL}} fit in the grid.}

\item{verbose}{Logical. Print progress information? Default:
\code{TRUE}.}
}
\value{
An object of class \code{"cv_orthoMTL"} containing:
  \describe{
    \item{best}{A list with the best hyperparameters: \code{lambda},
      \code{alpha}, \code{stepsize}, \code{diag_val}, and the
      corresponding \code{cv_score}.}
    \item{results}{A \code{data.frame} of all configurations with
      their mean CV C-index, sorted descending by \code{cv_score}.}
    \item{folds}{The fold assignment vector used.}
    \item{n_configs}{Total number of configurations tested.}
    \item{n_folds}{Number of unique folds.}
    \item{call}{The matched function call.}
  }
}
\description{
Performs a parallelised grid search over hyperparameters for
\code{\link{orthoMTL}}, evaluating each configuration via
cross-validated concordance index. Returns the best configuration
without retraining a final model (that is the caller's responsibility).
}
\details{
The grid is constructed as the full Cartesian product of
\code{lambdas}, \code{alphas}, \code{stepsizes}, and
\code{diag_vals}. Each configuration is evaluated independently
in parallel across cores. Within each configuration, folds are
evaluated sequentially and the per-fold C-indices are averaged.

The best configuration is selected by joint maximisation of the
mean CV C-index over the entire flattened grid (not greedy
sequential search).

This function does \emph{not} retrain a final model. Use the
returned hyperparameters to train via \code{\link{orthoMTL}}.
}
\examples{
\donttest{
set.seed(42)
n <- 50; p <- 5; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
SurvTime <- rexp(n, rate = 0.1)
Event <- rbinom(n, 1, 0.7)
thresholds <- c(4, 6, 10)

Y <- create_longitudinal_labels(SurvTime, Event, thresholds)
W <- create_indicator_matrix(Y)
K <- create_constraint_matrix(n_tasks)

folds <- rep(1:2, length.out = n)

cv_res <- cv_orthoMTL(
  X.train = X, Y.train = Y, W.train = W, K = K,
  lambdas = c(1e-3, 1e-2), alphas = 0,
  stepsizes = c(0.1), diag_vals = c(0.5, 1),
  survival = TRUE, disjoint = FALSE,
  folds = folds, n_cores = 1, seed = 42, verbose = FALSE
)

print(cv_res)
cv_res$best
}
}
\seealso{
\code{\link{orthoMTL}}, \code{\link{cindex_mtl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_correlation}
\alias{plot_correlation}
\title{Task Correlation Map for orthoMTL}
\usage{
plot_correlation(x, midpoint = 0.5, limits = c(0, 1))
}
\arguments{
\item{x}{Either a fitted \code{"orthoMTL"} object or a numeric
coefficient matrix with dimensions \code{p x numTasks}.}

\item{midpoint}{Midpoint for the colour scale. Default: \code{0.5}.}

\item{limits}{Numeric vector of length 2 for the colour scale limits.
Default: \code{c(0, 1)}.}
}
\value{
A \code{ggplot2} object.
}
\description{
Displays pairwise distances between task coefficient vectors as a
heatmap. Distance is computed as \code{1 - cosine_similarity}.
Values near 0 indicate similar coefficient profiles; values near 1
indicate orthogonal profiles.
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 4
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
colnames(Y) <- c("T1", "T2", "T3", "T4")
K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)

plot_correlation(fit)
}
\seealso{
\code{\link{plot_heatmap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.R
\name{coef.orthoMTL}
\alias{coef.orthoMTL}
\title{Extract Coefficients from an orthoMTL Model}
\usage{
\method{coef}{orthoMTL}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"orthoMTL"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric matrix of regression coefficients with dimensions
  \code{n_features x n_tasks}. Row names correspond to feature names
  and column names to task names, if available.
}
\description{
Returns the coefficient matrix from a fitted \code{orthoMTL} model.
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)
coef(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orthoMTL-package.R
\docType{package}
\name{orthoMTL-package}
\alias{orthoMTL-package}
\title{orthoMTL: Multi-Task Learning with Orthogonal Constraints}
\description{
Fits regularised multi-task learning models where relationships between
tasks are controlled via orthogonality or disjoint-support constraints.
Supports regression, classification, and censored survival data.
}
\section{Overview}{

orthoMTL implements a penalised multi-task learning framework where the
columns of the coefficient matrix are encouraged to be orthogonal (or
have disjoint supports). In survival mode, time-to-event data is
converted into binary classification tasks at user-defined thresholds,
with censored observations masked in the loss computation.

The package is the successor to the \code{orthopen} package and extends
it with survival analysis, elastic-net sparsity, cross-validation,
bootstrap inference, and a visualisation suite.
}

\section{Key Functions}{

\strong{Modelling:}
\itemize{
  \item \code{\link{orthoMTL}} — Core solver (regression, classification,
    survival)
  \item \code{\link{predict.orthoMTL}} — Predictions with column alignment
    and monotonicity projection
  \item \code{\link{coef.orthoMTL}} — Extract coefficient matrix
}

\strong{Survival Utilities:}
\itemize{
  \item \code{\link{create_longitudinal_labels}} — Survival to binary
    label conversion
  \item \code{\link{create_indicator_matrix}} — Censoring indicator matrix
  \item \code{\link{create_constraint_matrix}} — Diffusion constraint
    matrix for temporal tasks
}

\strong{Cross-Validation and Inference:}
\itemize{
  \item \code{\link{cv_orthoMTL}} — Parallel hyperparameter grid search
  \item \code{\link{bootstrap_orthoMTL}} — Bootstrap coefficient
    variability and null comparison
  \item \code{\link{cindex_mtl}} — Concordance index for multi-task
    predictions
}

\strong{Visualisation:}
\itemize{
  \item \code{\link{plot_heatmap}} — Coefficient heatmap
  \item \code{\link{plot_correlation}} — Task distance map
  \item \code{\link{plot_prediction}} — Prediction swimmer plot
  \item \code{\link{plot_bootstrap}} — Bootstrap real vs null comparison
}

\strong{Simulation:}
\itemize{
  \item \code{\link{simulate_mtl}} — Simulated survival data with
    time-varying effects
}
}

\section{Algorithm}{

The core optimisation problem is:

\deqn{\min_W \frac{1}{2n}\|XW - Y\|^2_{obs} +
  \lambda\left[\frac{1-\alpha}{2}\sum_{s,t} K_{st}|W_s^\top W_t| +
  \alpha\|W\|_1\right]}

where the loss is computed only on non-censored entries, \eqn{K} encodes
task relationships, \eqn{\lambda} controls the penalty strength, and
\eqn{\alpha \in [0,1]} mixes the orthogonality penalty (\eqn{\alpha=0})
with Lasso sparsity (\eqn{\alpha=1}).
}

\section{Getting Started}{

See \code{vignette("introduction", package = "orthoMTL")} for a complete
worked example using simulated data.
}

\references{
Vervier, K., Mahe, P., d'Aspremont, A., Veyrieras, J.-B., and
Vert, J.-P. (2014). On Learning Matrices with Orthogonal Columns or
Disjoint Supports. \emph{ECML-PKDD 2014}.
\url{https://hal.science/hal-00985654}
}
\author{
\strong{Maintainer}: Kevin Vervier \email{kevin.vervier@novartis.com}

Other contributors:
\itemize{
  \item Novartis Pharma AG [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_heatmap}
\alias{plot_heatmap}
\title{Coefficient Heatmap for orthoMTL}
\usage{
plot_heatmap(x, reorder = TRUE)
}
\arguments{
\item{x}{Either a fitted \code{"orthoMTL"} object or a numeric
coefficient matrix with dimensions \code{p x numTasks}. If an
\code{"orthoMTL"} object, coefficients are extracted via
\code{coef()}.}

\item{reorder}{Logical. If \code{TRUE} (default), rows are sorted
by mean coefficient across tasks (ascending).}
}
\value{
A \code{ggplot2} object.
}
\description{
Displays the coefficient matrix as a heatmap. Features (rows) can
optionally be reordered by their mean coefficient across tasks.
}
\examples{
set.seed(42)
n <- 100; p <- 10; n_tasks <- 3
X <- matrix(rnorm(n * p), n, p)
colnames(X) <- paste0("V", seq_len(p))
Y <- X \%*\% matrix(rnorm(p * n_tasks), p) + matrix(rnorm(n * n_tasks), n) * 0.1
colnames(Y) <- c("T1", "T2", "T3")
K <- matrix(1, n_tasks, n_tasks); diag(K) <- 0.5
fit <- orthoMTL(X, Y, lambda = 1e-3, K = K)

# From fitted object
plot_heatmap(fit)

# From raw matrix
plot_heatmap(coef(fit), reorder = FALSE)
}
\seealso{
\code{\link{coef.orthoMTL}}, \code{\link{plot_correlation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{r2_mtl}
\alias{r2_mtl}
\title{Coefficient of Determination (R-squared) for Multi-Task Regression}
\usage{
r2_mtl(true.label.mat, pred.label.mat)
}
\arguments{
\item{true.label.mat}{A numeric matrix of true responses
(\code{n x numTasks}). \code{NA} cells are ignored.}

\item{pred.label.mat}{A numeric matrix of predictions of the same
dimensions (as produced by \code{\link{predict.orthoMTL}}).}
}
\value{
A single numeric value (higher is better; 1 = perfect).
  Can be negative when predictions are worse than the mean.
}
\description{
Pooled \eqn{R^2 = 1 - SS_{res} / SS_{tot}} over every non-\code{NA}
cell, for \code{orthoMTL} fits in regression mode. \eqn{SS_{tot}} uses
the global mean of the pooled true values.
}
\examples{
set.seed(1)
Y <- matrix(rnorm(30), 10, 3)
P <- Y + matrix(rnorm(30, sd = 0.1), 10, 3)
r2_mtl(Y, P)
}
\seealso{
\code{\link{rmse_mtl}}, \code{\link{cindex_mtl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_print.R
\name{print.simulated_mtl}
\alias{print.simulated_mtl}
\title{Print Simulated Multi-Task Data}
\usage{
\method{print}{simulated_mtl}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"simulated_mtl"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a summary of a simulated dataset from
\code{\link{simulate_mtl}}.
}
\examples{
sim <- simulate_mtl(n = 100, p = 20, n_signals = 4)
print(sim)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{.make_template}
\alias{.make_template}
\title{Build a normalised effect template vector}
\usage{
.make_template(type, n_tasks)
}
\arguments{
\item{type}{Template type: "early", "late", "constant",
"increasing", "decreasing".}

\item{n_tasks}{Number of tasks/thresholds.}
}
\value{
Numeric vector of length \code{n_tasks}, normalised so
  \code{max(abs(values)) == 1}.
}
\description{
Build a normalised effect template vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{auc_mtl}
\alias{auc_mtl}
\title{Area Under the ROC Curve for Multi-Task Predictions}
\usage{
auc_mtl(true.label.mat, pred.label.mat)
}
\arguments{
\item{true.label.mat}{A numeric matrix of true responses
(\code{n x numTasks}). \code{NA} cells are ignored.}

\item{pred.label.mat}{A numeric matrix of predictions of the same
dimensions (as produced by \code{\link{predict.orthoMTL}}).}
}
\value{
A single numeric value in \code{[0, 1]} (higher is better;
  0.5 = random).
}
\description{
Pooled AUC over every non-\code{NA} cell, computed via the
Mann-Whitney U statistic (with 0.5 credit for ties), for
\code{orthoMTL} fits in classification mode. The positive class is
defined by true label \code{> 0}. AUC is invariant to monotone
transforms, so raw \code{"link"} scores and \code{"response"}
probabilities give identical results.
}
\examples{
set.seed(1)
Y <- matrix(sample(c(-1, 1), 30, replace = TRUE), 10, 3)
P <- Y + matrix(rnorm(30), 10, 3)
auc_mtl(Y, P)
}
\seealso{
\code{\link{accuracy_mtl}}, \code{\link{cindex_mtl}}
}
